@@ -331,6 +331,30 @@ pub extern "C" fn syd_lock(state: lock_state_t) -> c_int {
     stat(&format!("/dev/syd/lock:{state}"))
 }
 
+/// Checks if walk sandboxing is enabled.
+///
+/// Returns true if walk sandboxing is enabled, false otherwise.
+#[no_mangle]
+pub extern "C" fn syd_enabled_walk() -> bool {
+    stat("/dev/syd/sandbox/walk?") == 0
+}
+
+/// Enable walk sandboxing.
+///
+/// Returns 0 on success, negated errno on failure.
+#[no_mangle]
+pub extern "C" fn syd_enable_walk() -> c_int {
+    stat("/dev/syd/sandbox/walk:on")
+}
+
+/// Disable walk sandboxing.
+///
+/// Returns 0 on success, negated errno on failure.
+#[no_mangle]
+pub extern "C" fn syd_disable_walk() -> c_int {
+    stat("/dev/syd/sandbox/walk:off")
+}
+
 /// Checks if stat sandboxing is enabled.
 ///
 /// Returns true if stat sandboxing is enabled, false otherwise.
@@ -811,28 +835,52 @@ pub extern "C" fn syd_disable_utime() -> c_int {
     stat("/dev/syd/sandbox/utime:off")
 }
 
-/// Checks if mkdev sandboxing is enabled.
+/// Checks if mkbdev sandboxing is enabled.
 ///
-/// Returns true if mkdev sandboxing is enabled, false otherwise.
+/// Returns true if mkbdev sandboxing is enabled, false otherwise.
 #[no_mangle]
-pub extern "C" fn syd_enabled_mkdev() -> bool {
-    stat("/dev/syd/sandbox/mkdev?") == 0
+pub extern "C" fn syd_enabled_mkbdev() -> bool {
+    stat("/dev/syd/sandbox/mkbdev?") == 0
 }
 
-/// Enable mkdev sandboxing.
+/// Enable mkbdev sandboxing.
 ///
 /// Returns 0 on success, negated errno on failure.
 #[no_mangle]
-pub extern "C" fn syd_enable_mkdev() -> c_int {
-    stat("/dev/syd/sandbox/mkdev:on")
+pub extern "C" fn syd_enable_mkbdev() -> c_int {
+    stat("/dev/syd/sandbox/mkbdev:on")
 }
 
-/// Disable mkdev sandboxing.
+/// Disable mkbdev sandboxing.
 ///
 /// Returns 0 on success, negated errno on failure.
 #[no_mangle]
-pub extern "C" fn syd_disable_mkdev() -> c_int {
-    stat("/dev/syd/sandbox/mkdev:off")
+pub extern "C" fn syd_disable_mkbdev() -> c_int {
+    stat("/dev/syd/sandbox/mkbdev:off")
+}
+
+/// Checks if mkcdev sandboxing is enabled.
+///
+/// Returns true if mkcdev sandboxing is enabled, false otherwise.
+#[no_mangle]
+pub extern "C" fn syd_enabled_mkcdev() -> bool {
+    stat("/dev/syd/sandbox/mkcdev?") == 0
+}
+
+/// Enable mkcdev sandboxing.
+///
+/// Returns 0 on success, negated errno on failure.
+#[no_mangle]
+pub extern "C" fn syd_enable_mkcdev() -> c_int {
+    stat("/dev/syd/sandbox/mkcdev:on")
+}
+
+/// Disable mkcdev sandboxing.
+///
+/// Returns 0 on success, negated errno on failure.
+#[no_mangle]
+pub extern "C" fn syd_disable_mkcdev() -> c_int {
+    stat("/dev/syd/sandbox/mkcdev:off")
 }
 
 /// Checks if mkfifo sandboxing is enabled.
@@ -1027,6 +1075,17 @@ pub extern "C" fn syd_disable_tpe() -> c_int {
     stat("/dev/syd/sandbox/tpe:off")
 }
 
+/// Set the default action for Walk Sandboxing.
+#[no_mangle]
+pub extern "C" fn syd_default_walk(action: action_t) -> c_int {
+    // Convert action_t enum to corresponding action string.
+    let action = match Action::try_from(action) {
+        Ok(action) => action,
+        Err(_) => return -EINVAL,
+    };
+    stat(&format!("/dev/syd/default/walk:{action}"))
+}
+
 /// Set the default action for Stat Sandboxing.
 #[no_mangle]
 pub extern "C" fn syd_default_stat(action: action_t) -> c_int {
@@ -1247,15 +1306,26 @@ pub extern "C" fn syd_default_utime(action: action_t) -> c_int {
     stat(&format!("/dev/syd/default/utime:{action}"))
 }
 
-/// Set the default action for Mkdev Sandboxing.
+/// Set the default action for Mkbdev Sandboxing.
 #[no_mangle]
-pub extern "C" fn syd_default_mkdev(action: action_t) -> c_int {
+pub extern "C" fn syd_default_mkbdev(action: action_t) -> c_int {
     // Convert action_t enum to corresponding action string.
     let action = match Action::try_from(action) {
         Ok(action) => action,
         Err(_) => return -EINVAL,
     };
-    stat(&format!("/dev/syd/default/mkdev:{action}"))
+    stat(&format!("/dev/syd/default/mkbdev:{action}"))
+}
+
+/// Set the default action for Mkcdev Sandboxing.
+#[no_mangle]
+pub extern "C" fn syd_default_mkcdev(action: action_t) -> c_int {
+    // Convert action_t enum to corresponding action string.
+    let action = match Action::try_from(action) {
+        Ok(action) => action,
+        Err(_) => return -EINVAL,
+    };
+    stat(&format!("/dev/syd/default/mkcdev:{action}"))
 }
 
 /// Set the default action for Mkfifo Sandboxing.
@@ -1444,6 +1514,52 @@ pub extern "C" fn syd_force_clr() -> c_int {
     stat("/dev/syd/force^")
 }
 
+/// Adds to the given actionlist of walk sandboxing.
+///
+/// Returns 0 on success, negated errno on failure.
+#[no_mangle]
+pub extern "C" fn syd_walk_add(action: action_t, glob: *const c_char) -> c_int {
+    // Convert action_t enum to corresponding action string.
+    let action = match Action::try_from(action) {
+        Ok(action) => action,
+        Err(_) => return -EINVAL,
+    };
+
+    // Call magic function with add operator.
+    esyd(&format!("{action}/walk"), glob, b'+')
+}
+
+/// Removes the first instance from the end of the given actionlist of
+/// walk sandboxing.
+///
+/// Returns 0 on success, negated errno on failure.
+#[no_mangle]
+pub extern "C" fn syd_walk_del(action: action_t, glob: *const c_char) -> c_int {
+    // Convert action_t enum to corresponding action string.
+    let action = match Action::try_from(action) {
+        Ok(action) => action,
+        Err(_) => return -EINVAL,
+    };
+
+    // Call magic function with del operator.
+    esyd(&format!("{action}/walk"), glob, b'-')
+}
+
+/// Removes all matching patterns from the given actionlist of walk sandboxing.
+///
+/// Returns 0 on success, negated errno on failure.
+#[no_mangle]
+pub extern "C" fn syd_walk_rem(action: action_t, glob: *const c_char) -> c_int {
+    // Convert action_t enum to corresponding action string.
+    let action = match Action::try_from(action) {
+        Ok(action) => action,
+        Err(_) => return -EINVAL,
+    };
+
+    // Call magic function with rem operator.
+    esyd(&format!("{action}/walk"), glob, b'^')
+}
+
 /// Adds to the given actionlist of stat sandboxing.
 ///
 /// Returns 0 on success, negated errno on failure.
@@ -2364,11 +2480,11 @@ pub extern "C" fn syd_utime_rem(action: action_t, glob: *const c_char) -> c_int
     esyd(&format!("{action}/utime"), glob, b'^')
 }
 
-/// Adds to the given actionlist of mkdev sandboxing.
+/// Adds to the given actionlist of mkbdev sandboxing.
 ///
 /// Returns 0 on success, negated errno on failure.
 #[no_mangle]
-pub extern "C" fn syd_mkdev_add(action: action_t, glob: *const c_char) -> c_int {
+pub extern "C" fn syd_mkbdev_add(action: action_t, glob: *const c_char) -> c_int {
     // Convert action_t enum to corresponding action string.
     let action = match Action::try_from(action) {
         Ok(action) => action,
@@ -2376,15 +2492,15 @@ pub extern "C" fn syd_mkdev_add(action: action_t, glob: *const c_char) -> c_int
     };
 
     // Call magic function with add operator.
-    esyd(&format!("{action}/mkdev"), glob, b'+')
+    esyd(&format!("{action}/mkbdev"), glob, b'+')
 }
 
 /// Removes the first instance from the end of the given actionlist of
-/// mkdev sandboxing.
+/// mkbdev sandboxing.
 ///
 /// Returns 0 on success, negated errno on failure.
 #[no_mangle]
-pub extern "C" fn syd_mkdev_del(action: action_t, glob: *const c_char) -> c_int {
+pub extern "C" fn syd_mkbdev_del(action: action_t, glob: *const c_char) -> c_int {
     // Convert action_t enum to corresponding action string.
     let action = match Action::try_from(action) {
         Ok(action) => action,
@@ -2392,14 +2508,14 @@ pub extern "C" fn syd_mkdev_del(action: action_t, glob: *const c_char) -> c_int
     };
 
     // Call magic function with del operator.
-    esyd(&format!("{action}/mkdev"), glob, b'-')
+    esyd(&format!("{action}/mkbdev"), glob, b'-')
 }
 
-/// Removes all matching patterns from the given actionlist of mkdev sandboxing.
+/// Removes all matching patterns from the given actionlist of mkbdev sandboxing.
 ///
 /// Returns 0 on success, negated errno on failure.
 #[no_mangle]
-pub extern "C" fn syd_mkdev_rem(action: action_t, glob: *const c_char) -> c_int {
+pub extern "C" fn syd_mkbdev_rem(action: action_t, glob: *const c_char) -> c_int {
     // Convert action_t enum to corresponding action string.
     let action = match Action::try_from(action) {
         Ok(action) => action,
@@ -2407,7 +2523,53 @@ pub extern "C" fn syd_mkdev_rem(action: action_t, glob: *const c_char) -> c_int
     };
 
     // Call magic function with rem operator.
-    esyd(&format!("{action}/mkdev"), glob, b'^')
+    esyd(&format!("{action}/mkbdev"), glob, b'^')
+}
+
+/// Adds to the given actionlist of mkcdev sandboxing.
+///
+/// Returns 0 on success, negated errno on failure.
+#[no_mangle]
+pub extern "C" fn syd_mkcdev_add(action: action_t, glob: *const c_char) -> c_int {
+    // Convert action_t enum to corresponding action string.
+    let action = match Action::try_from(action) {
+        Ok(action) => action,
+        Err(_) => return -EINVAL,
+    };
+
+    // Call magic function with add operator.
+    esyd(&format!("{action}/mkcdev"), glob, b'+')
+}
+
+/// Removes the first instance from the end of the given actionlist of
+/// mkcdev sandboxing.
+///
+/// Returns 0 on success, negated errno on failure.
+#[no_mangle]
+pub extern "C" fn syd_mkcdev_del(action: action_t, glob: *const c_char) -> c_int {
+    // Convert action_t enum to corresponding action string.
+    let action = match Action::try_from(action) {
+        Ok(action) => action,
+        Err(_) => return -EINVAL,
+    };
+
+    // Call magic function with del operator.
+    esyd(&format!("{action}/mkcdev"), glob, b'-')
+}
+
+/// Removes all matching patterns from the given actionlist of mkcdev sandboxing.
+///
+/// Returns 0 on success, negated errno on failure.
+#[no_mangle]
+pub extern "C" fn syd_mkcdev_rem(action: action_t, glob: *const c_char) -> c_int {
+    // Convert action_t enum to corresponding action string.
+    let action = match Action::try_from(action) {
+        Ok(action) => action,
+        Err(_) => return -EINVAL,
+    };
+
+    // Call magic function with rem operator.
+    esyd(&format!("{action}/mkcdev"), glob, b'^')
 }
 
 /// Adds to the given actionlist of mkfifo sandboxing.
