@@ -6,19 +6,18 @@
 //
 // SPDX-License-Identifier: GPL-3.0
 
-use std::{collections::HashMap, os::fd::RawFd};
+use std::{os::fd::RawFd, sync::LazyLock};
 
-use once_cell::sync::Lazy;
 use serde::ser::{Serialize, SerializeMap, Serializer};
 
 use crate::{
-    hash::SydRandomState,
+    hash::SydHashMap,
     path::PATH_MAX,
-    sandbox::{Version, ACTION_STR, CAP_STR},
+    sandbox::{Version, ACTION_STR, SANDBOX_CAPS},
 };
 
-/// Lazy instance of the syd(2) API spec.
-pub static API_SPEC: Lazy<Api> = Lazy::new(|| Api {
+/// LazyLock instance of the syd(2) API spec.
+pub static API_SPEC: LazyLock<Api> = LazyLock::new(|| Api {
     root: "/dev/syd".into(),
     version: crate::config::API_VERSION,
     methods: vec![
@@ -186,7 +185,7 @@ pub static API_SPEC: Lazy<Api> = Lazy::new(|| Api {
             exc: Some(
                 [("$action".into(), vec!["allow".into()])]
                     .into_iter()
-                    .collect::<HashMap<_, _, SydRandomState>>(),
+                    .collect::<SydHashMap<_, _>>(),
             ),
             ..Default::default()
         },
@@ -228,7 +227,7 @@ pub static API_SPEC: Lazy<Api> = Lazy::new(|| Api {
                     ],
                 )]
                 .into_iter()
-                .collect::<HashMap<_, _, SydRandomState>>(),
+                .collect::<SydHashMap<_, _>>(),
             ),
             ..Default::default()
         },
@@ -254,7 +253,7 @@ pub static API_SPEC: Lazy<Api> = Lazy::new(|| Api {
                     ],
                 )]
                 .into_iter()
-                .collect::<HashMap<_, _, SydRandomState>>(),
+                .collect::<SydHashMap<_, _>>(),
             ),
             ..Default::default()
         },
@@ -280,7 +279,7 @@ pub static API_SPEC: Lazy<Api> = Lazy::new(|| Api {
                     ],
                 )]
                 .into_iter()
-                .collect::<HashMap<_, _, SydRandomState>>(),
+                .collect::<SydHashMap<_, _>>(),
             ),
             ..Default::default()
         },
@@ -306,7 +305,7 @@ pub static API_SPEC: Lazy<Api> = Lazy::new(|| Api {
                     ],
                 )]
                 .into_iter()
-                .collect::<HashMap<_, _, SydRandomState>>(),
+                .collect::<SydHashMap<_, _>>(),
             ),
             ..Default::default()
         },
@@ -332,7 +331,7 @@ pub static API_SPEC: Lazy<Api> = Lazy::new(|| Api {
                     ],
                 )]
                 .into_iter()
-                .collect::<HashMap<_, _, SydRandomState>>(),
+                .collect::<SydHashMap<_, _>>(),
             ),
             ..Default::default()
         },
@@ -469,6 +468,24 @@ pub static API_SPEC: Lazy<Api> = Lazy::new(|| Api {
             op: Some(Operator::Set),
             ..Default::default()
         },
+        Method {
+            name: "allow_unsafe_hardlinks".into(),
+            desc: "Allow unsafe access to hardlinks".into(),
+            path: Some("trace/allow_unsafe_hardlinks".into()),
+            argv: Some(vec![Arg::Boolean]),
+            argc: Some(vec![1]),
+            op: Some(Operator::Set),
+            ..Default::default()
+        },
+        Method {
+            name: "allow_unsafe_proc_pid_status".into(),
+            desc: "Allow unfiltered access to proc_pid_status(5)".into(),
+            path: Some("trace/allow_unsafe_proc_pid_status".into()),
+            argv: Some(vec![Arg::Boolean]),
+            argc: Some(vec![1]),
+            op: Some(Operator::Set),
+            ..Default::default()
+        },
         Method {
             name: "allow_unsafe_magiclinks".into(),
             desc: "Allow unsafe access to procfs magiclinks".into(),
@@ -497,9 +514,9 @@ pub static API_SPEC: Lazy<Api> = Lazy::new(|| Api {
             ..Default::default()
         },
         Method {
-            name: "allow_unsafe_open_cdev".into(),
-            desc: "Allow unsafe continue of character device opens".into(),
-            path: Some("trace/allow_unsafe_open_cdev".into()),
+            name: "allow_unsafe_open_kfd".into(),
+            desc: "Allow unsafe continue of AMD KFD character device opens".into(),
+            path: Some("trace/allow_unsafe_open_kfd".into()),
             argv: Some(vec![Arg::Boolean]),
             argc: Some(vec![1]),
             op: Some(Operator::Set),
@@ -613,6 +630,15 @@ pub static API_SPEC: Lazy<Api> = Lazy::new(|| Api {
             op: Some(Operator::Set),
             ..Default::default()
         },
+        Method {
+            name: "force_no_xdev".into(),
+            desc: "Force RESOLVE_NO_XDEV at open(2) boundary".into(),
+            path: Some("trace/force_no_xdev".into()),
+            argv: Some(vec![Arg::Boolean]),
+            argc: Some(vec![1]),
+            op: Some(Operator::Set),
+            ..Default::default()
+        },
         Method {
             name: "force_umask".into(),
             desc: "Force given umask mode, -1 to unset".into(),
@@ -715,7 +741,12 @@ pub static API_SPEC: Lazy<Api> = Lazy::new(|| Api {
         Type {
             name: "$caps".into(),
             desc: "Sandbox capabilities".into(),
-            enums: Some(CAP_STR.iter().map(|p| p.to_string()).collect::<Vec<_>>()),
+            enums: Some(
+                SANDBOX_CAPS
+                    .keys()
+                    .map(|p| p.to_string())
+                    .collect::<Vec<_>>(),
+            ),
             impls: Some(vec!["$str".into()]),
             ..Default::default()
         },
@@ -780,6 +811,7 @@ pub static API_SPEC: Lazy<Api> = Lazy::new(|| Api {
                 "silent".into(),
                 "trace".into(),
                 "tty".into(),
+                "tty_native".into(),
                 "user".into(),
             ]),
             impls: Some(vec!["$str".into()]),
@@ -901,7 +933,7 @@ struct Method {
     // Argument separator.
     args: Option<String>,
     // Exclusions for argument validations
-    exc: Option<HashMap<String, Vec<String>, SydRandomState>>,
+    exc: Option<SydHashMap<String, Vec<String>>>,
 }
 
 impl Serialize for Method {
