@@ -6,7 +6,7 @@
 // SPDX-License-Identifier: GPL-3.0
 
 use std::{
-    arch::asm,
+    ffi::CStr,
     fmt::Display,
     os::{
         fd::{AsFd, AsRawFd, RawFd},
@@ -19,7 +19,8 @@ use std::{
 };
 
 use btoi::btoi;
-use libc::{c_int, c_ulong, prctl, EACCES, EOPNOTSUPP};
+use libc::{c_int, c_ulong, prctl, EACCES, EOPNOTSUPP, RTLD_LOCAL, RTLD_NOLOAD, RTLD_NOW};
+use libloading::os::unix::Library;
 use libseccomp::{scmp_cmp, ScmpAction, ScmpArch, ScmpFilterContext, ScmpSyscall};
 use nix::{
     dir::Dir,
@@ -30,19 +31,20 @@ use nix::{
     sys::{
         personality::Persona,
         stat::Mode,
-        wait::{waitpid, Id, WaitPidFlag},
+        wait::{Id, WaitPidFlag},
     },
-    unistd::{fork, ForkResult, Gid, Pid, Uid},
+    unistd::{fork, read, write, ForkResult, Gid, Pid, Uid},
 };
 use procfs_core::process::{MMPermissions, MMapPath, MemoryMap};
 use serde::Serialize;
 
 use crate::{
     caps,
-    compat::waitid,
-    config::{HAVE_RWF_NOAPPEND, MMAP_MIN_ADDR, UNSAFE_PERSONAS},
+    compat::{waitid, WaitStatus},
+    config::{FADVISE_SYSCALLS, HAVE_RWF_NOAPPEND, MMAP_MIN_ADDR, UNSAFE_PERSONAS},
+    cookie::safe_socket,
     err::{err2no, SydResult},
-    fs::{readlinkat, safe_clone, FileType, AT_BADFD},
+    fs::{nlmsg_align, readlinkat, retry_on_eintr, safe_clone, FileType, AT_BADFD},
     info,
     landlock::{
         path_beneath_rules, Access, AccessFs, AccessNet, NetPort, RestrictSelfFlags,
@@ -53,6 +55,12 @@ use crate::{
     sandbox::{IoctlMap, Sandbox},
 };
 
+// Used as dummy errno(3) after SECCOMP_IOCTL_NOTIF_ADDFD with SECCOMP_ADDFD_FLAG_SEND flag.
+pub(crate) const EIDRM: i32 = -libc::EIDRM;
+
+// Used as dummy errno(3) to initiate Ghost Mode.
+pub(crate) const EOWNERDEAD: i32 = -libc::EOWNERDEAD;
+
 /// Confine current process using MDWE prctl(2).
 ///
 /// Use `no_inherit` to prevent inheriting the restriction to children.
@@ -356,6 +364,55 @@ pub fn confine_scmp_pwritev2(ssb: bool) -> SydResult<()> {
     Ok(())
 }
 
+/// Allow clone(2) operations without namespaces.
+pub fn confine_scmp_clone(ctx: &mut ScmpFilterContext) -> SydResult<()> {
+    let syscall = match ScmpSyscall::from_name("clone") {
+        Ok(s) => s,
+        Err(_) => {
+            info!("ctx": "confine", "op": "allow_syscall",
+                "msg": "invalid or unsupported syscall clone");
+            return Ok(());
+        }
+    };
+
+    let ns_mask = CloneFlags::CLONE_NEWNS
+        | CloneFlags::CLONE_NEWUTS
+        | CloneFlags::CLONE_NEWIPC
+        | CloneFlags::CLONE_NEWUSER
+        | CloneFlags::CLONE_NEWNET
+        | CloneFlags::CLONE_NEWPID
+        | CloneFlags::CLONE_NEWCGROUP
+        | CLONE_NEWTIME;
+    #[allow(clippy::cast_sign_loss)]
+    let ns_mask = ns_mask.bits() as u64;
+
+    // On s390/s390x the first two parameters to clone are switched.
+    let filter = if !cfg!(target_arch = "s390x") {
+        scmp_cmp!($arg0 & ns_mask == 0)
+    } else {
+        scmp_cmp!($arg1 & ns_mask == 0)
+    };
+    ctx.add_rule_conditional(ScmpAction::Allow, syscall, &[filter])?;
+
+    Ok(())
+}
+
+/// Deny clone3(2) with ENOSYS for compatibility.
+pub fn confine_scmp_clone3(ctx: &mut ScmpFilterContext) -> SydResult<()> {
+    let syscall = match ScmpSyscall::from_name("clone3") {
+        Ok(s) => s,
+        Err(_) => {
+            info!("ctx": "confine", "op": "allow_syscall",
+                "msg": "invalid or unsupported syscall clone3");
+            return Ok(());
+        }
+    };
+
+    ctx.add_rule(ScmpAction::Errno(libc::ENOSYS), syscall)?;
+
+    Ok(())
+}
+
 /// Allow writes to sandbox `SYD_LOG_FD` only.
 ///
 /// If logging is disabled:
@@ -408,6 +465,20 @@ pub fn confine_scmp_write(
     Ok(())
 }
 
+/// Allow fadvise family system calls.
+pub fn confine_scmp_fadvise(ctx: &mut ScmpFilterContext) -> SydResult<()> {
+    for sysname in FADVISE_SYSCALLS {
+        if let Ok(syscall) = ScmpSyscall::from_name(sysname) {
+            ctx.add_rule(ScmpAction::Allow, syscall)?;
+        } else {
+            info!("ctx": "confine", "op": "allow_syscall",
+                "msg": format!("invalid or unsupported syscall {sysname}"));
+        }
+    }
+
+    Ok(())
+}
+
 /// Convenience `Command` run wrapper which returns:
 ///
 /// - Same exit code as the process on clean exit.
@@ -793,11 +864,12 @@ pub(crate) fn op2name(op: u8) -> &'static str {
         0x2 => "bind",
         0x3 => "connect",
         0x5 => "accept",
+        0x6 => "getsockname",
         0xb => "sendto",
         0x10 => "sendmsg",
         0x12 => "accept4",
         0x14 => "sendmmsg",
-        _ => unreachable!(),
+        _ => unreachable!("BUG: op2name called with unsupported op:{op:#x}, report a bug!"),
     }
 }
 
@@ -915,8 +987,8 @@ pub fn syd_enabled() -> bool {
     // SAFETY: In libc, we trust.
     match unsafe { fork() } {
         Ok(ForkResult::Parent { child, .. }) => {
-            match waitpid(child, None) {
-                Ok(nix::sys::wait::WaitStatus::Exited(_, code)) => {
+            match waitid(Id::Pid(child), WaitPidFlag::WEXITED) {
+                Ok(WaitStatus::Exited(_, code)) => {
                     // Check the child's exit status.
                     // Exit status of 0 means syd is enabled.
                     code == 0
@@ -940,11 +1012,11 @@ pub fn syd_enabled() -> bool {
             }
 
             if ctx.load().is_err() && Errno::last() == Errno::EBUSY {
-                // seccomp filter exists
-                // syd is in business.
+                // seccomp(2) filter exists,
+                // Syd is in business.
                 exit(0);
             } else {
-                // seccomp filter does not exist
+                // seccomp(2) filter does not exist.
                 exit(1);
             }
         }
@@ -1645,21 +1717,9 @@ pub(crate) const NAMESPACE_FLAGS_ALL: libc::c_int = libc::CLONE_NEWNS
     | CLONE_NEWTIME.bits();
 
 pub(crate) const NAMESPACE_NAMES: &[&str] = &[
-    "mount", "ipc", "net", "pid", "uts", "user", "cgroup", "time",
+    "user", "mount", "ipc", "net", "pid", "uts", "cgroup", "time",
 ];
 
-/// Convert CLONE namespace flags to a Vector of Strings.
-pub fn nsflags_name(flags: libc::c_int) -> Vec<String> {
-    let mut names = Vec::with_capacity(NAMESPACE_FLAGS.len());
-    for &flag in NAMESPACE_FLAGS {
-        if flags & flag != 0 {
-            names.push(nsflag_name(flag));
-        }
-    }
-
-    names
-}
-
 /// Convert a CLONE namespace flag to its String representation.
 pub fn nsflag_name(flag: libc::c_int) -> String {
     match flag {
@@ -1676,6 +1736,232 @@ pub fn nsflag_name(flag: libc::c_int) -> String {
     .to_string()
 }
 
+/// Check for CONFIG_CROSS_MEMORY_ATTACH support.
+pub fn check_cross_memory_attach() -> bool {
+    // SAFETY: We explicitly call the system call with
+    // invalid arguments just to check for host Linux
+    // kernel support.
+    !matches!(
+        Errno::result(unsafe {
+            libc::process_vm_readv(0, std::ptr::null(), 0, std::ptr::null(), 0, 0)
+        }),
+        Err(Errno::ENOSYS)
+    )
+}
+
+/// Check whether getrandom(2) is in VDSO.
+pub fn check_vdso_has_getrandom() -> bool {
+    has_vdso_symbol(c"__vdso_getrandom")
+}
+
+/// Enumerate vDSO call names present in this process by probing the already-mapped vDSO.
+///
+/// Uses dlopen(3) with RTLD_NOLOAD and dlsym(3).
+pub fn vdso_list_calls() -> Result<Vec<&'static CStr>, libloading::Error> {
+    const KERN_LEN: usize = 9; // "__kernel_"
+    const VDSO_LEN: usize = 7; // "__vdso_"
+    const VDSO_IDX: usize = 10; // index into array.
+    const VDSO_CALL_NAMES: &[&CStr] = &[
+        c"__kernel_clock_getres",
+        c"__kernel_clock_getres_time64",
+        c"__kernel_clock_gettime",
+        c"__kernel_clock_gettime64",
+        c"__kernel_getcpu",
+        c"__kernel_getrandom",
+        c"__kernel_gettimeofday",
+        c"__kernel_get_tbfreq",    // powerpc
+        c"__kernel_riscv_hwprobe", // riscv
+        c"__kernel_time",
+        c"__vdso_clock_getres",
+        c"__vdso_clock_getres_time64",
+        c"__vdso_clock_gettime",
+        c"__vdso_clock_gettime64",
+        c"__vdso_getcpu",
+        c"__vdso_getrandom",
+        c"__vdso_gettimeofday",
+        c"__vdso_get_tbfreq",    // powerpc
+        c"__vdso_riscv_hwprobe", // riscv
+        c"__vdso_time",
+    ];
+
+    // SAFETY: Library::open with RTLD_NOLOAD only obtains a handle
+    // to the already-mapped vDSO; it does not load code. Handle used
+    // immediately and dropped.
+    let vdso =
+        unsafe { Library::open(Some("linux-vdso.so.1"), RTLD_NOLOAD | RTLD_LOCAL | RTLD_NOW) }?;
+
+    let mut out = Vec::with_capacity(VDSO_CALL_NAMES.len());
+    for (idx, sym) in VDSO_CALL_NAMES.iter().enumerate() {
+        let sym = sym.to_bytes_with_nul();
+
+        // SAFETY: We perform a pure lookup with a NUL-terminated name;
+        // no call or dereference occurs.
+        if unsafe { vdso.get::<*const ()>(sym) }.is_ok() {
+            let plen = if idx < VDSO_IDX { KERN_LEN } else { VDSO_LEN };
+            // SAFETY:
+            // 1. All entries start with "__vdso_"
+            // 2. We keep the trailing NUL, producing a valid CStr
+            //    that borrows from the static symbol literal.
+            out.push(unsafe { CStr::from_bytes_with_nul_unchecked(&sym[plen..]) });
+        }
+    }
+    Ok(out)
+}
+
+/// Probe vDSO for symbol WITHOUT loading anything.
+///
+/// Returns true if the vDSO object exports the symbol.
+pub fn has_vdso_symbol(sym: &CStr) -> bool {
+    // SAFETY: Library::open with RTLD_NOLOAD only obtains a handle
+    // to the already-mapped vDSO; it does not load code. Handle used
+    // immediately and dropped.
+    let vdso = if let Ok(vdso) =
+        unsafe { Library::open(Some("linux-vdso.so.1"), RTLD_NOLOAD | RTLD_LOCAL | RTLD_NOW) }
+    {
+        vdso
+    } else {
+        return false;
+    };
+
+    // SAFETY: We perform a pure lookup with a NUL-terminated name;
+    // no call or dereference occurs.
+    unsafe { vdso.get::<*const ()>(sym.to_bytes_with_nul()).is_ok() }
+}
+
+/// Check support for CONFIG_UNIX_DIAG in Linux kernel.
+#[allow(clippy::arithmetic_side_effects)]
+#[allow(clippy::cast_possible_truncation)]
+pub fn check_unix_diag() -> Result<bool, Errno> {
+    const SOCK_DIAG_BY_FAMILY: u16 = 20;
+    const NL_HDR_LEN: usize = 16;
+    const UD_REQ_LEN: usize = 24;
+
+    let nlmsg_done: u16 = libc::NLMSG_DONE as u16;
+    let nlmsg_error: u16 = libc::NLMSG_ERROR as u16;
+
+    // Open NETLINK_SOCK_DIAG.
+    let nl = match safe_socket(
+        libc::AF_NETLINK,
+        libc::SOCK_DGRAM | libc::SOCK_CLOEXEC,
+        libc::NETLINK_SOCK_DIAG,
+    ) {
+        Ok(fd) => fd,
+        Err(Errno::EPROTONOSUPPORT | Errno::EAFNOSUPPORT | Errno::ENOTSUP) => return Ok(false),
+        Err(errno) => return Err(errno),
+    };
+
+    // Build AF_UNIX dump request; match all states, no exact lookup.
+    let total_len = (NL_HDR_LEN + UD_REQ_LEN) as u32;
+    let mut req = [0u8; NL_HDR_LEN + UD_REQ_LEN];
+    let mut p = 0usize;
+
+    // nlmsghdr
+    req[p..p + 4].copy_from_slice(&total_len.to_ne_bytes());
+    p += 4;
+    req[p..p + 2].copy_from_slice(&SOCK_DIAG_BY_FAMILY.to_ne_bytes());
+    p += 2;
+    let nl_flags = (libc::NLM_F_REQUEST | libc::NLM_F_DUMP) as u16; // ROOT|MATCH
+    req[p..p + 2].copy_from_slice(&nl_flags.to_ne_bytes());
+    p += 2;
+    req[p..p + 4].copy_from_slice(&1u32.to_ne_bytes());
+    p += 4; // seq
+    req[p..p + 4].copy_from_slice(&0u32.to_ne_bytes());
+    p += 4; // pid
+
+    // unix_diag_req
+    req[p] = libc::AF_UNIX as u8;
+    p += 1; // sdiag_family
+    req[p] = 0;
+    p += 1; // sdiag_protocol
+    req[p..p + 2].copy_from_slice(&0u16.to_ne_bytes());
+    p += 2; // pad
+    req[p..p + 4].copy_from_slice(&u32::MAX.to_ne_bytes());
+    p += 4; // udiag_states = all
+    req[p..p + 4].copy_from_slice(&0u32.to_ne_bytes());
+    p += 4; // udiag_ino = 0 (dump)
+    req[p..p + 4].copy_from_slice(&0u32.to_ne_bytes());
+    p += 4; // udiag_show = none
+    req[p..p + 4].copy_from_slice(&0u32.to_ne_bytes());
+    p += 4; // cookie[0]
+    req[p..p + 4].copy_from_slice(&0u32.to_ne_bytes());
+    p += 4; // cookie[1]
+    assert_eq!(p, req.len());
+
+    // Send
+    let mut off = 0;
+    while off < req.len() {
+        let n = retry_on_eintr(|| write(&nl, &req[off..]))?;
+        if n == 0 {
+            return Err(Errno::EIO);
+        }
+        off += n;
+    }
+
+    // Classify first reply: ENOENT => no handler; anything else => handler exists.
+    let mut rbuf = [0u8; 8192];
+    loop {
+        let n = retry_on_eintr(|| read(&nl, &mut rbuf))?;
+        if n == 0 {
+            return Err(Errno::EIO);
+        }
+
+        let mut pos = 0usize;
+        while pos + NL_HDR_LEN <= n {
+            let nlmsg_len = {
+                let b: [u8; 4] = rbuf[pos..pos + 4].try_into().or(Err(Errno::EOVERFLOW))?;
+                u32::from_ne_bytes(b) as usize
+            };
+            if nlmsg_len == 0 || pos + nlmsg_len > n {
+                return Err(Errno::EIO);
+            }
+            let nlmsg_type = {
+                let b: [u8; 2] = rbuf[pos + 4..pos + 6]
+                    .try_into()
+                    .or(Err(Errno::EOVERFLOW))?;
+                u16::from_ne_bytes(b)
+            };
+
+            if nlmsg_type == nlmsg_error {
+                if nlmsg_len < NL_HDR_LEN + 4 {
+                    return Err(Errno::EIO);
+                }
+                let b: [u8; 4] = rbuf[pos + NL_HDR_LEN..pos + NL_HDR_LEN + 4]
+                    .try_into()
+                    .or(Err(Errno::EOVERFLOW))?;
+                let neg = i32::from_ne_bytes(b);
+
+                if neg == 0 {
+                    return Ok(true);
+                } // ACK
+                if neg == -libc::ENOENT {
+                    return Ok(false);
+                } // no handler
+                  // Any other error => handler exists but rejected the request.
+                return Ok(true);
+            }
+
+            if nlmsg_type == SOCK_DIAG_BY_FAMILY || nlmsg_type == nlmsg_done {
+                return Ok(true); // got data or DONE => handler present.
+            }
+
+            pos = nlmsg_align(pos + nlmsg_len);
+        }
+    }
+}
+
+/// Probes the process link-map (RTLD_DEFAULT)
+/// for a symbol by name using dlsym(3).
+pub fn has_symbol(sym: &CStr) -> bool {
+    // SAFETY:
+    // 1. Library::this() yields a handle to
+    //    the current process namespace (RTLD_DEFAULT),
+    //    loads nothing new, and the handle is used only
+    //    within this function.
+    // 2. We perform a pure lookup with a NUL-terminated name;
+    //    no call or dereference occurs.
+    unsafe { Library::this().get::<*const ()>(sym.to_bytes_with_nul()) }.is_ok()
+}
+
 /// Check for file descriptor leaks above the standard input, output, and error.
 ///
 /// This function examines the `/proc/self/fd` directory to identify
@@ -1903,7 +2189,7 @@ impl ExportMode {
 ///
 /// Unsafe to be fast!
 pub unsafe fn fork_fast() {
-    asm!(
+    std::arch::asm!(
         "mov eax, 0x2", // 0x2 is the syscall number for fork on x86
         "int 0x80",     // Interrupt to make the syscall
         out("eax") _,
@@ -1919,7 +2205,7 @@ pub unsafe fn fork_fast() {
 /// Unsafe to be fast!
 pub unsafe fn fork_fast() {
     // Inline assembly for x86-64
-    asm!(
+    std::arch::asm!(
         "mov rax, 57", // 57 is the syscall number for fork on x86-64
         "syscall",
         out("rax") _,
@@ -1934,7 +2220,7 @@ pub unsafe fn fork_fast() {
 ///
 /// Unsafe to be fast!
 pub unsafe fn fork_fast() {
-    asm!(
+    std::arch::asm!(
         "mov x0, 17",  // SIGCHLD
         "mov x1, 0",   // child_stack (null, not recommended)
         "mov x8, 220", // syscall number for clone
@@ -1951,7 +2237,7 @@ pub unsafe fn fork_fast() {
 ///
 /// Unsafe to be fast!
 pub unsafe fn fork_fast() {
-    asm!(
+    std::arch::asm!(
         "mov r7, #2", // 2 is the syscall number for fork on ARM
         "swi #0",     // Software interrupt to make the syscall
         out("r0") _,
@@ -1969,7 +2255,7 @@ pub unsafe fn fork_fast() {
 ///
 /// Unsafe to be fast!
 pub unsafe fn fork_fast() {
-    asm!(
+    std::arch::asm!(
         "li 0, 2",     // Load immediate 2 into register r0 (syscall number for fork)
         "sc",          // System call
         out("r3") _,   // Output from r3 (return value of fork)
@@ -1985,7 +2271,7 @@ pub unsafe fn fork_fast() {
 ///
 /// Unsafe to be fast!
 pub unsafe fn fork_fast() {
-    asm!(
+    std::arch::asm!(
         "li a7, 220",  // syscall number for clone on riscv64
         "li a0, 17",   // SIGCHLD
         "li a1, 0",    // child_stack (null, not recommended)
@@ -2005,7 +2291,7 @@ pub unsafe fn fork_fast() {
 ///
 /// Unsafe to be fast!
 pub unsafe fn fork_fast() {
-    asm!(
+    std::arch::asm!(
         "lgr %r1, 2", // Load syscall number for fork (2) directly into %r1.
         "svc 0",      // Supervisor Call to invoke the syscall.
     );
