@@ -17,7 +17,6 @@ use std::{
 #[allow(clippy::disallowed_types)]
 use ahash::{AHasher, RandomState};
 use bitflags::bitflags;
-use constant_time_eq::constant_time_eq;
 use crc::{Crc, CRC_32_ISO_HDLC, CRC_64_ECMA_182};
 use data_encoding::{HEXLOWER, HEXLOWER_PERMISSIVE, HEXUPPER};
 use lexis::ToName;
@@ -37,15 +36,17 @@ use nix::{
 use procfs_core::{SelfTest, Type};
 use sha1::Sha1;
 use sha3::{Digest, Sha3_256, Sha3_384, Sha3_512};
+use subtle::ConstantTimeEq;
 use zeroize::Zeroizing;
 
 use crate::{
     config::*,
     cookie::safe_accept4,
     err::SydResult,
-    fs::{create_memfd, fillrandom, mkstempat, retry_on_eintr, set_append, set_nonblock},
+    fs::{create_memfd, retry_on_eintr, set_append, set_nonblock},
     path::XPath,
     proc::proc_crypto_read,
+    rng::{fillrandom, mkstempat},
 };
 
 /// Defines hash functions supported by Syd.
@@ -102,6 +103,9 @@ pub const SHA256_BLOCK_SIZE: usize = 64;
 /// HMAC tag size
 pub const HMAC_TAG_SIZE: usize = SHA256_DIGEST_SIZE;
 
+/// SYD3 encrypted file header size
+pub const SYD3_HDR_SIZE: u64 = (CRYPT_MAGIC.len() + HMAC_TAG_SIZE + IV_SIZE) as u64;
+
 /// AlgAddr for AES.
 static AES_ADDR: LazyLock<AlgAddr> = LazyLock::new(|| AlgAddr::new("skcipher", "ctr(aes)"));
 
@@ -844,8 +848,8 @@ pub fn aes_ctr_init<F: AsRawFd>(fd: &F, nonblock: bool) -> Result<OwnedFd, Errno
 }
 
 /// Encrypts a chunk of data using the initialized AES-CTR socket.
-pub fn aes_ctr_enc<S: AsRawFd>(
-    sock: &S,
+pub fn aes_ctr_enc<Fd: AsFd>(
+    sock: Fd,
     chunk: &[u8],
     iv: Option<&IV>,
     more: bool,
@@ -867,9 +871,9 @@ pub fn aes_ctr_enc<S: AsRawFd>(
             ControlMessage::AlgSetOp(&libc::ALG_OP_ENCRYPT),
             ControlMessage::AlgSetIv(iv.as_ref()),
         ][..];
-        retry_on_eintr(|| sendmsg::<()>(sock.as_raw_fd(), iov, cmsgs, flags, None))
+        retry_on_eintr(|| sendmsg::<()>(sock.as_fd().as_raw_fd(), iov, cmsgs, flags, None))
     } else {
-        retry_on_eintr(|| sendmsg::<()>(sock.as_raw_fd(), iov, &[], flags, None))
+        retry_on_eintr(|| sendmsg::<()>(sock.as_fd().as_raw_fd(), iov, &[], flags, None))
     }
 }
 
@@ -1197,7 +1201,7 @@ pub fn aes_ctr_tmp<Fd: AsFd>(
 
         // Compare computed HMAC with the HMAC tag read from the file.
         // SAFETY: Compare in constant time!
-        if !constant_time_eq(computed_hmac.as_slice(), hmac_tag.as_slice()) {
+        if hmac_tag.ct_ne(&computed_hmac).into() {
             // HMAC verification failed.
             return Err(Errno::EBADMSG);
         }
@@ -1386,6 +1390,14 @@ pub type SydHashMap<K, V> = std::collections::HashMap<K, V, SydRandomState>;
 #[allow(clippy::disallowed_types)]
 pub type SydHashSet<K> = std::collections::HashSet<K, SydRandomState>;
 
+/// Convenience alias for IndexMap with `SydRandomState`
+#[allow(clippy::disallowed_types)]
+pub type SydIndexMap<K, V> = indexmap::IndexMap<K, V, SydRandomState>;
+
+/// Convenience alias for IndexSet with `SydRandomState`
+#[allow(clippy::disallowed_types)]
+pub type SydIndexSet<K> = indexmap::IndexSet<K, SydRandomState>;
+
 #[cfg(test)]
 mod tests {
     use std::io::Cursor;
@@ -1579,6 +1591,65 @@ mod tests {
     ),
     ];
 
+    fn check_kernel_crypto_support() -> bool {
+        let key = Key::random().unwrap();
+        let key_id = match add_key(
+            "user",
+            "SYD-3-CRYPT-TEST",
+            key.as_ref(),
+            KEY_SPEC_USER_KEYRING,
+        ) {
+            Ok(key_id) => key_id,
+            Err(Errno::EAFNOSUPPORT | Errno::ENOSYS) => {
+                eprintln!("Test requires Linux keyrings(7) API, skipping!");
+                return false;
+            }
+            Err(Errno::EACCES) => {
+                eprintln!("Is your session keyring attached to your user keyring?");
+                eprintln!("Test requires Linux keyrings(7) API, skipping!");
+                return false;
+            }
+            Err(errno) => {
+                eprintln!("Failed to test for Linux keyrings(7) API: {errno}");
+                return false;
+            }
+        };
+        match aes_ctr_setup(key_id) {
+            Ok(fd) => drop(fd),
+            Err(Errno::EAFNOSUPPORT) => {
+                eprintln!("Test requires Linux Kernel Cryptography API, skipping!");
+                return false;
+            }
+            Err(Errno::EACCES) => {
+                eprintln!("Is your session keyring attached to your user keyring?");
+                eprintln!("Test requires Linux keyrings(7) API, skipping!");
+                return false;
+            }
+            Err(errno) => {
+                eprintln!("Failed to test for Linux Kernel Cryptography API: {errno}");
+                return false;
+            }
+        }
+        match hmac_sha256_setup(key_id) {
+            Ok(fd) => drop(fd),
+            Err(Errno::EAFNOSUPPORT) => {
+                eprintln!("Test requires Linux Kernel Cryptography API, skipping!");
+                return false;
+            }
+            Err(Errno::EACCES) => {
+                eprintln!("Is your session keyring attached to your user keyring?");
+                eprintln!("Test requires Linux keyrings(7) API, skipping!");
+                return false;
+            }
+            Err(errno) => {
+                eprintln!("Failed to test for Linux Kernel Cryptography API: {errno}");
+                return false;
+            }
+        }
+
+        true
+    }
+
     #[test]
     fn test_hash_simple() {
         let mut errors = Vec::new();
@@ -1685,6 +1756,10 @@ mod tests {
 
     #[test]
     fn test_hmac_sha256_simple() {
+        if !check_kernel_crypto_support() {
+            return;
+        }
+
         let mut errors = Vec::new();
 
         for (i, test_case) in HMAC_TEST_CASES.iter().enumerate() {
@@ -1797,6 +1872,10 @@ mod tests {
 
     #[test]
     fn test_aes_ctr_setup() {
+        if !check_kernel_crypto_support() {
+            return;
+        }
+
         let key = Key::random().unwrap();
         assert!(!key.is_zero(), "key is all zeros!");
         let key_id = add_key(
@@ -1825,6 +1904,10 @@ mod tests {
 
     #[test]
     fn test_aes_ctr_init() {
+        if !check_kernel_crypto_support() {
+            return;
+        }
+
         let key = Key::random().unwrap();
         assert!(!key.is_zero(), "key is all zeros!");
         let key_id = add_key(
@@ -1856,6 +1939,10 @@ mod tests {
 
     #[test]
     fn test_aes_ctr_enc_and_dec() {
+        if !check_kernel_crypto_support() {
+            return;
+        }
+
         let key = Key::random().unwrap();
         assert!(!key.is_zero(), "key is all zeros!");
         let key_id = add_key(
@@ -1915,6 +2002,10 @@ mod tests {
     #[test]
     #[ignore]
     fn test_aes_ctr_enc_with_more_flag() {
+        if !check_kernel_crypto_support() {
+            return;
+        }
+
         let key = Key::random().unwrap();
         assert!(!key.is_zero(), "key is all zeros!");
         let key_id = add_key(
@@ -1994,6 +2085,10 @@ mod tests {
 
     #[test]
     fn test_aes_ctr_enc_and_dec_tmp() {
+        if !check_kernel_crypto_support() {
+            return;
+        }
+
         let key = Key::random().unwrap();
         assert!(!key.is_zero(), "key is all zeros!");
         let enc_key_id = add_key(
