@@ -49,7 +49,7 @@ pub(crate) fn sysenter_chdir<'a>(
     let process = RemoteProcess::new(pid);
 
     // SAFETY: This is a ptrace hook, the PID cannot be validated.
-    let (path, _, _, _) = unsafe { process.read_path(sandbox, data.args, arg, false, None) }?;
+    let (path, _, _, _) = process.read_path(sandbox, data.args, arg, false, None)?;
 
     // Check for chroot, allow for the
     // common `cd /` use case.
