@@ -23,8 +23,10 @@ use crate::{
 };
 
 pub(crate) fn sys_fchown(request: UNotifyEventRequest) -> ScmpNotifResp {
+    // WANT_READ: fchown(2) does not work with O_PATH fds.
     let argv = &[SysArg {
         dirfd: Some(0),
+        fsflags: FsFlags::MUST_PATH | FsFlags::WANT_READ,
         ..Default::default()
     }];
     syscall_path_handler(request, "fchown", argv, |path_args, request, sandbox| {
@@ -55,7 +57,6 @@ pub(crate) fn sys_fchown(request: UNotifyEventRequest) -> ScmpNotifResp {
 pub(crate) fn sys_chown(request: UNotifyEventRequest) -> ScmpNotifResp {
     let argv = &[SysArg {
         path: Some(0),
-        fsflags: FsFlags::MUST_PATH,
         ..Default::default()
     }];
     syscall_path_handler(request, "chown", argv, |path_args, request, sandbox| {
@@ -152,15 +153,12 @@ fn syscall_chown_handler(
     #[allow(clippy::disallowed_methods)]
     let path = args.0.as_ref().unwrap();
 
+    // We use MUST_PATH, dir refers to the file.
+    assert!(
+        path.base.is_empty(),
+        "BUG: MUST_PATH returned a directory for chown, report a bug!"
+    );
     let fd = path.dir.as_ref().map(|fd| fd.as_fd()).ok_or(Errno::EBADF)?;
 
-    let flags = if path.base.is_empty() {
-        // FD-only call, e.g remote-fd transfer due to fchown(2).
-        AtFlags::AT_EMPTY_PATH
-    } else {
-        // SAFETY: Do not resolve symlinks in base to prevent TOCTTOU.
-        AtFlags::AT_SYMLINK_NOFOLLOW
-    };
-
-    fchownat(fd, path.base, owner, group, flags).map(|_| request.return_syscall(0))
+    fchownat(fd, c"", owner, group, AtFlags::AT_EMPTY_PATH).map(|_| request.return_syscall(0))
 }
