@@ -11,7 +11,7 @@ use std::{fs::File, io::Seek, os::fd::AsRawFd};
 use libseccomp::ScmpNotifResp;
 use nix::{
     errno::Errno,
-    fcntl::{AtFlags, OFlag},
+    fcntl::{AtFlags, OFlag, ResolveFlag},
     sys::signal::{kill, Signal},
     unistd::Pid,
     NixPath,
@@ -21,7 +21,7 @@ use crate::{
     config::{PROC_FILE, PTRACE_DATA_EXECVE, PTRACE_DATA_EXECVEAT},
     elf::{ElfError, ElfFileType, ElfType, ExecutableFile, LinkingType},
     err2no, error,
-    fs::{is_executable, safe_open_magicsym, FileType, FsFlags, MaybeFd, AT_EXECVE_CHECK},
+    fs::{is_executable, safe_open_msym, FileType, FsFlags, MaybeFd, AT_EXECVE_CHECK},
     hook::{RemoteProcess, SysArg, SysFlags, UNotifyEventRequest},
     kernel::{sandbox_path, to_atflags},
     path::XPathBuf,
@@ -91,8 +91,7 @@ pub(crate) fn sysenter_exec(
     let process = RemoteProcess::new(pid);
 
     // SAFETY: This is a ptrace hook, the PID cannot be validated.
-    let (mut path, _, _, empty_path) =
-        unsafe { process.read_path(sandbox, data.args, arg, false, None) }?;
+    let (mut path, _, _, empty_path) = process.read_path(sandbox, data.args, arg, false, None)?;
 
     // Call sandbox access checker if Exec sandboxing is on.
     let caps = sandbox.getcaps(Capability::CAP_EXEC | Capability::CAP_TPE);
@@ -152,7 +151,7 @@ pub(crate) fn sysenter_exec(
         if action != Action::Filter {
             let (_, bin) = path.abs().split();
             error!("ctx": "segvguard",
-                "err": format!("max crashes {} exceeded, execution of `{bin}' denied",
+                "msg": format!("max crashes {} exceeded, execution of `{bin}' denied",
                     sandbox.segvguard_maxcrashes),
                 "tip": "increase `segvguard/maxcrashes'",
                 "pid": pid.as_raw(), "path": path.abs());
@@ -188,7 +187,7 @@ pub(crate) fn sysenter_exec(
             // TODO: Fix proc_mmap to work in ptrace hooks.
             let msg = msg.as_deref().unwrap_or("?");
             error!("ctx": "trusted_path_execution",
-                "err": format!("exec from untrusted path blocked: {msg}"),
+                "msg": format!("exec from untrusted path blocked: {msg}"),
                 "pid": pid.as_raw(), "path": path.abs(),
                 "sys": syscall_name, "arch": info.arch, "args": data.args,
                 "tip": "move the binary to a safe location or use `sandbox/tpe:off'");
@@ -223,7 +222,8 @@ pub(crate) fn sysenter_exec(
     let mut file = match path.dir.take() {
         Some(MaybeFd::Owned(fd)) => {
             let pfd = XPathBuf::from_self_fd(fd.as_raw_fd());
-            safe_open_magicsym(PROC_FILE(), &pfd, OFlag::O_RDONLY).map(File::from)?
+            safe_open_msym(PROC_FILE(), &pfd, OFlag::O_RDONLY, ResolveFlag::empty())
+                .map(File::from)?
         }
         _ => return Err(Errno::ENOEXEC),
     };
@@ -245,7 +245,7 @@ pub(crate) fn sysenter_exec(
             let errno = err2no(&err);
             if !sandbox.filter_path(Capability::CAP_EXEC, path.abs()) {
                 error!("ctx": "parse_elf",
-                    "err": format!("io error: {errno}"),
+                    "msg": format!("io error: {errno}"),
                     "pid": pid.as_raw(), "path": path.abs());
             }
             return Err(errno);
@@ -253,7 +253,7 @@ pub(crate) fn sysenter_exec(
         Err(ElfError::BadMagic) => {
             if !sandbox.filter_path(Capability::CAP_EXEC, path.abs()) {
                 error!("ctx": "parse_elf",
-                    "err": "invalid ELF file",
+                    "msg": "invalid ELF file",
                     "pid": pid.as_raw(), "path": path.abs());
             }
             return Err(Errno::ENOEXEC);
@@ -261,7 +261,7 @@ pub(crate) fn sysenter_exec(
         Err(ElfError::Malformed) => {
             if !sandbox.filter_path(Capability::CAP_EXEC, path.abs()) {
                 error!("ctx": "parse_elf",
-                    "err": "malformed ELF file",
+                    "msg": "malformed ELF file",
                     "pid": pid.as_raw(), "path": path.abs());
             }
             return Err(Errno::ENOEXEC);
@@ -272,7 +272,7 @@ pub(crate) fn sysenter_exec(
     if is_script && deny_script {
         if !sandbox.filter_path(Capability::CAP_EXEC, path.abs()) {
             error!("ctx": "deny_script",
-                "err": "script execution denied",
+                "msg": "script execution denied",
                 "pid": pid.as_raw(), "path": path.abs(),
                 "exe": format!("{exe}"));
         }
@@ -291,7 +291,7 @@ pub(crate) fn sysenter_exec(
     {
         if !sandbox.filter_path(Capability::CAP_EXEC, path.abs()) {
             error!("ctx": "check_elf",
-                "err": "ld.so(8) exec-indirection prevented",
+                "msg": "ld.so(8) exec-indirection prevented",
                 "pid": pid.as_raw(), "path": path.abs(),
                 "tip": "configure `trace/allow_unsafe_ldso:1'",
                 "exe": format!("{exe}"));
@@ -301,7 +301,7 @@ pub(crate) fn sysenter_exec(
 
     if !is_script && restrict_pie && matches!(exe, ExecutableFile::Elf { pie: false, .. }) {
         if !sandbox.filter_path(Capability::CAP_EXEC, path.abs()) {
-            error!("ctx": "check_elf", "err": "not PIE",
+            error!("ctx": "check_elf", "msg": "ELF is not Position Independent Executable (PIE)",
                 "pid": pid.as_raw(), "path": path.abs(),
                 "tip": "configure `trace/allow_unsafe_nopie:1'",
                 "exe": format!("{exe}"));
@@ -311,7 +311,7 @@ pub(crate) fn sysenter_exec(
 
     if !is_script && restrict_xs && matches!(exe, ExecutableFile::Elf { xs: true, .. }) {
         if !sandbox.filter_path(Capability::CAP_EXEC, path.abs()) {
-            error!("ctx": "check_elf", "err": "execstack",
+            error!("ctx": "check_elf", "msg": "ELF has executable stack",
                 "pid": pid.as_raw(), "path": path.abs(),
                 "tip": "configure `trace/allow_unsafe_stack:1'",
                 "exe": format!("{exe}"));
@@ -330,7 +330,7 @@ pub(crate) fn sysenter_exec(
         )
     {
         if !sandbox.filter_path(Capability::CAP_EXEC, path.abs()) {
-            error!("ctx": "check_elf", "err": "32-bit",
+            error!("ctx": "check_elf", "msg": "32-bit ELF execution prevented",
                 "pid": pid.as_raw(), "path": path.abs(),
                 "tip": "configure `trace/deny_elf32:0'",
                 "exe": format!("{exe}"));
@@ -349,7 +349,7 @@ pub(crate) fn sysenter_exec(
         )
     {
         if !sandbox.filter_path(Capability::CAP_EXEC, path.abs()) {
-            error!("ctx": "check_elf", "err": "dynamic-link",
+            error!("ctx": "check_elf", "msg": "dynamic linked ELF execution prevented",
                 "pid": pid.as_raw(), "path": path.abs(),
                 "tip": "configure `trace/deny_elf_dynamic:0'",
                 "exe": format!("{exe}"));
@@ -368,7 +368,7 @@ pub(crate) fn sysenter_exec(
         )
     {
         if !sandbox.filter_path(Capability::CAP_EXEC, path.abs()) {
-            error!("ctx": "check_elf", "err": "static-link",
+            error!("ctx": "check_elf", "msg": "static linked ELF execution prevented",
                 "pid": pid.as_raw(), "path": path,
                 "tip": "configure `trace/deny_elf_static:0'",
                 "exe": format!("{exe}"));
@@ -425,7 +425,7 @@ pub(crate) fn sysenter_exec(
             Ok(Action::Panic) => panic!(),
             Err(IntegrityError::Sys(errno)) => {
                 error!("ctx": "verify_elf",
-                    "err": format!("system error during ELF checksum calculation: {errno}"),
+                    "msg": format!("system error during ELF checksum calculation: {errno}"),
                     "pid": pid.as_raw(), "path": path.abs(),
                     "tip": format!("configure `force+{path}:<checksum>'"));
                 return Err(Errno::EACCES);
@@ -437,7 +437,7 @@ pub(crate) fn sysenter_exec(
             }) => {
                 if !matches!(action, Action::Allow | Action::Filter) {
                     error!("ctx": "verify_elf", "act": action,
-                        "err": format!("ELF checksum mismatch: {found} is not {expected}"),
+                        "msg": format!("ELF checksum mismatch: {found} is not {expected}"),
                         "pid": pid.as_raw(), "path": path.abs(),
                         "tip": format!("configure `force+{path}:<checksum>'"));
                 }
