@@ -12,10 +12,8 @@ use libseccomp::ScmpNotifResp;
 use nix::errno::Errno;
 
 use crate::{
-    config::PROC_FILE,
-    fs::{readlinkat, FileInfo},
+    fs::{CanonicalPath, FileInfo},
     hook::UNotifyEventRequest,
-    path::XPathBuf,
     sandbox::Capability,
 };
 
@@ -32,10 +30,13 @@ pub(crate) fn sys_fcntl(request: UNotifyEventRequest) -> ScmpNotifResp {
         return request.fail_syscall(Errno::EBADF);
     };
 
-    let mut pfd = XPathBuf::from_pid(req.pid());
-    pfd.push(b"fd");
-    pfd.push_fd(fd);
-    let path = match readlinkat(PROC_FILE(), &pfd) {
+    let fd = if let Ok(fd) = request.get_fd(fd) {
+        fd
+    } else {
+        return request.fail_syscall(Errno::EBADF);
+    };
+
+    let path = match CanonicalPath::new_fd(fd.into(), req.pid()) {
         Ok(path) => {
             if !request.is_valid() {
                 return request.fail_syscall(Errno::ESRCH);
@@ -47,7 +48,7 @@ pub(crate) fn sys_fcntl(request: UNotifyEventRequest) -> ScmpNotifResp {
 
     let sandbox = request.get_sandbox();
     let is_crypt = sandbox.enabled(Capability::CAP_CRYPT);
-    let is_append = sandbox.is_append(&path);
+    let is_append = sandbox.is_append(path.abs());
     drop(sandbox); // release the read-lock.
 
     if is_append {
@@ -55,20 +56,19 @@ pub(crate) fn sys_fcntl(request: UNotifyEventRequest) -> ScmpNotifResp {
         return request.return_syscall(0);
     }
 
+    #[allow(clippy::disallowed_methods)]
     if is_crypt {
-        let fd = if let Ok(fd) = request.get_fd(fd) {
-            fd
-        } else {
-            return request.fail_syscall(Errno::EBADF);
-        };
-        if let Ok(info) = FileInfo::from_fd(&fd) {
-            #[allow(clippy::disallowed_methods)]
+        let fd = path.dir.as_ref().unwrap();
+        if let Ok(info) = FileInfo::from_fd(fd) {
             let files = request.crypt_map.as_ref().unwrap();
-            for map in files.read().unwrap_or_else(|err| err.into_inner()).values() {
-                if info == map.info {
-                    // Deny with EACCES, caller should know.
-                    return request.fail_syscall(Errno::EACCES);
-                }
+            let check = {
+                let files = files.0.lock().unwrap_or_else(|err| err.into_inner());
+                files.values().any(|map| map.info == info)
+            }; // Lock is released here.
+
+            if check {
+                // Deny with EACCES, caller should know.
+                return request.fail_syscall(Errno::EACCES);
             }
         }
     }
