@@ -12,7 +12,7 @@ use libseccomp::ScmpNotifResp;
 use nix::{errno::Errno, sys::inotify::AddWatchFlags, NixPath};
 
 use crate::{
-    fs::{fd_mode, inotify_add_watch, is_sidechannel_device, FsFlags},
+    fs::{file_type, inotify_add_watch, FsFlags},
     hook::{SysArg, UNotifyEventRequest},
     kernel::syscall_path_handler,
     path::XPathBuf,
@@ -53,6 +53,8 @@ pub(crate) fn sys_inotify_add_watch(request: UNotifyEventRequest) -> ScmpNotifRe
         "inotify_add_watch",
         argv,
         |path_args, request, sandbox| {
+            let restrict_notify_bdev = !sandbox.flags.allow_unsafe_notify_bdev();
+            let restrict_notify_cdev = !sandbox.flags.allow_unsafe_notify_cdev();
             drop(sandbox); // release the read-lock.
 
             // SAFETY: SysArg has one element.
@@ -68,20 +70,24 @@ pub(crate) fn sys_inotify_add_watch(request: UNotifyEventRequest) -> ScmpNotifRe
             #[allow(clippy::disallowed_methods)]
             let dfd = path.dir.as_ref().map(|fd| fd.as_fd()).unwrap();
 
-            // SAFETY:
-            // 1. Strip IN_{ACCESS,MODIFY} if we're marking a sidechannel device.
-            // 2. Strip IN_DONT_FOLLOW which has already been handled during canonicalization.
-            let st_mode = fd_mode(dfd)?;
             let mut mask = mask & !AddWatchFlags::IN_DONT_FOLLOW;
-            if is_sidechannel_device(st_mode) {
-                mask.remove(AddWatchFlags::IN_ACCESS);
-                mask.remove(AddWatchFlags::IN_MODIFY);
+            if restrict_notify_bdev || restrict_notify_cdev {
+                // SAFETY:
+                // 1. Strip IN_{ACCESS,MODIFY} if we're marking a sidechannel device.
+                // 2. Strip IN_DONT_FOLLOW which has already been handled during canonicalization.
+                let filetype = file_type(dfd, None, false)?;
+                if (restrict_notify_bdev && filetype.is_block_device())
+                    || (restrict_notify_cdev && filetype.is_char_device())
+                {
+                    mask.remove(AddWatchFlags::IN_ACCESS);
+                    mask.remove(AddWatchFlags::IN_MODIFY);
+                }
             }
 
             // SAFETY: We open a FD to the path and then use the
-            // proc path /proc/self/fd/$fd in address' path argument
-            // to avoid symlink TOCTOU.
-            let mut pfd = XPathBuf::from("/proc/self/fd");
+            // proc path /proc/thread-self/fd/$fd in address' path
+            // argument to avoid symlink TOCTOU.
+            let mut pfd = XPathBuf::from("/proc/thread-self/fd");
             pfd.push_fd(dfd.as_raw_fd());
 
             // Get the INotify FD.
