@@ -51,7 +51,7 @@ pub(crate) fn sys_ioctl(request: UNotifyEventRequest) -> ScmpNotifResp {
             Some(&request),
             &sandbox,
             request.scmpreq.pid(), // Unused when request.is_some()
-            &path.abs(),
+            path.abs(),
             caps,
             true,
             "ioctl",
