@@ -451,19 +451,14 @@ where
 
                     // Handle ftruncate etc. for files with encryption in progress.
                     let crypt_path = if crypt {
+                        #[allow(clippy::disallowed_methods)]
+                        let files = request.crypt_map.as_ref().unwrap();
                         if let Ok(info) = FileInfo::from_fd(&fd) {
-                            let mut found = None;
-                            #[allow(clippy::disallowed_methods)]
-                            let files = request.crypt_map.as_ref().unwrap();
-                            for (path, map) in
-                                files.read().unwrap_or_else(|err| err.into_inner()).iter()
-                            {
-                                if info == map.info {
-                                    found = Some(path.clone());
-                                    break;
-                                }
-                            }
-                            found
+                            let files = files.0.lock().unwrap_or_else(|e| e.into_inner());
+                            files
+                                .iter()
+                                .find_map(|(path, map)| (map.info == info).then(|| path.clone()))
+                            // Lock is released here.
                         } else {
                             None
                         }
