@@ -5,7 +5,7 @@
 //
 // SPDX-License-Identifier: GPL-3.0
 
-use std::os::fd::{AsRawFd, FromRawFd, OwnedFd};
+use std::os::fd::{AsFd, OwnedFd};
 
 use libseccomp::ScmpNotifResp;
 use nix::{
@@ -15,10 +15,11 @@ use nix::{
 
 use crate::{
     compat::getsockdomain,
+    confine::op2errno,
+    cookie::safe_accept4,
     fs::{get_nonblock, has_recv_timeout},
     hook::UNotifyEventRequest,
-    kernel::net::{sandbox_addr_inet, sandbox_addr_inet6},
-    op2errno,
+    kernel::net::sandbox_addr,
     sandbox::Capability,
 };
 
@@ -29,16 +30,15 @@ pub(crate) fn handle_accept(
     op: u8,
 ) -> Result<ScmpNotifResp, Errno> {
     // Determine the socket family.
-    let fml = getsockdomain(&fd).or(Err(op2errno(op)))?;
-    let ipv6 = match fml {
-        libc::AF_INET6 => true,
-        libc::AF_INET => false,
-        _ => {
-            // Not an IPv{4,6} socket, continue.
-            // SAFETY: No pointer-dereference in access check.
-            return unsafe { Ok(request.continue_syscall()) };
-        }
-    };
+    if !matches!(
+        getsockdomain(&fd).map_err(|_| op2errno(op))?,
+        libc::AF_INET6 | libc::AF_INET
+    ) {
+        // Not an IPv{4,6} socket, continue system call.
+        //
+        // SAFETY: No pointer-dereference in access check.
+        return unsafe { Ok(request.continue_syscall()) };
+    }
 
     // Determine address length if specified.
     let addrlen = if args[2] != 0 {
@@ -65,8 +65,8 @@ pub(crate) fn handle_accept(
     };
 
     let sandbox = request.get_sandbox();
-    let force_cloexec = sandbox.force_cloexec();
-    let force_rand_fd = sandbox.force_rand_fd();
+    let force_cloexec = sandbox.flags.force_cloexec();
+    let force_rand_fd = sandbox.flags.force_rand_fd();
     drop(sandbox); // release read-lock.
 
     let mut flags = if op == 0x12 {
@@ -88,18 +88,12 @@ pub(crate) fn handle_accept(
     };
 
     // Do the accept call.
-    let result = do_accept4(fd, request, flags, is_blocking, ignore_restart);
+    let (fd, addr, addrlen_out) = do_accept4(fd, request, flags, is_blocking, ignore_restart)?;
 
-    // Only now, bail if accept failed.
-    let (fd, addr, addrlen_out) = result?;
-
-    // Check the returned address for access.
+    // Check the source address against the IP blocklist.
+    // No port filtering is done here for simplicity and efficiency.
     let sandbox = request.get_sandbox();
-    if ipv6 {
-        sandbox_addr_inet6(request, &sandbox, &addr, op, Capability::CAP_NET_CONNECT)?;
-    } else {
-        sandbox_addr_inet(request, &sandbox, &addr, op, Capability::CAP_NET_CONNECT)?;
-    }
+    sandbox_addr(request, &sandbox, &addr, &None, op, Capability::empty())?;
     drop(sandbox); // release the read lock.
 
     // Write address buffer as necessary.
@@ -133,8 +127,8 @@ pub(crate) fn handle_accept(
     request.send_fd(fd, cloexec, force_rand_fd)
 }
 
-fn do_accept4(
-    fd: OwnedFd,
+fn do_accept4<Fd: AsFd>(
+    fd: Fd,
     request: &UNotifyEventRequest,
     flags: SockFlag,
     is_blocking: bool,
@@ -157,9 +151,10 @@ fn do_accept4(
             .add_sys_block(request.scmpreq, ignore_restart)?;
     };
 
-    // SAFETY: In libc we trust.
-    let result =
-        Errno::result(unsafe { libc::accept4(fd.as_raw_fd(), ptr, &mut len, flags.bits()) });
+    // Make the accept4(2) call.
+    //
+    // SAFETY: ptr and len are valid pointers.
+    let result = unsafe { safe_accept4(fd, ptr, &raw mut len, flags) };
 
     // Remove invalidation record unless interrupted.
     if is_blocking {
@@ -168,10 +163,8 @@ fn do_accept4(
             .del_sys_block(request.scmpreq.id, matches!(result, Err(Errno::EINTR)))?;
     }
 
-    let fd = result.map(|fd| {
-        // SAFETY: accept4 returns a valid FD.
-        unsafe { OwnedFd::from_raw_fd(fd) }
-    })?;
+    // Check for accept4 errors after invalidation.
+    let fd = result?;
 
     // SAFETY:
     // Convert the raw address into a SockaddrStorage structure.
