@@ -17,6 +17,7 @@ use std::{
     },
 };
 
+use ipnet::IpNet;
 use libseccomp::ScmpNotifResp;
 use nix::{
     errno::Errno,
@@ -35,12 +36,13 @@ use crate::{
         bind::handle_bind,
         connect::handle_connect,
         getsockname::handle_getsockname,
+        getsockopt::handle_getsockopt,
         sendmsg::{handle_sendmmsg, handle_sendmsg},
         sendto::handle_sendto,
         socket::handle_socket,
     },
     path::{XPath, XPathBuf},
-    sandbox::{Action, Capability, SandboxGuard},
+    sandbox::{Action, AddressPattern, Capability, CidrRule, SandboxGuard},
     warn,
 };
 
@@ -48,6 +50,7 @@ pub(crate) mod accept;
 pub(crate) mod bind;
 pub(crate) mod connect;
 pub(crate) mod getsockname;
+pub(crate) mod getsockopt;
 pub(crate) mod sendmsg;
 pub(crate) mod sendto;
 pub(crate) mod socket;
@@ -65,12 +68,13 @@ pub(crate) fn sys_socketcall(request: UNotifyEventRequest) -> ScmpNotifResp {
     // 0x5 => accept
     // 0x6 => getsockname
     // 0xb => sendto
+    // 0xf => getsockopt
     // 0x10 => sendmsg
     // 0x12 => accept4
     // 0x14 => sendmmsg
     #[allow(clippy::cast_possible_truncation)]
     let op = match req.data.args[0] {
-        n @ (0x1 | 0x2 | 0x3 | 0x5 | 0x6 | 0xb | 0x10 | 0x12 | 0x14) => n as u8,
+        n @ (0x1 | 0x2 | 0x3 | 0x5 | 0x6 | 0xb | 0xf | 0x10 | 0x12 | 0x14) => n as u8,
         _ => {
             // SAFETY: Safe network call, continue.
             // No pointer-dereference in access check.
@@ -145,6 +149,11 @@ pub(crate) fn sys_getsockname(request: UNotifyEventRequest) -> ScmpNotifResp {
     syscall_network_handler(request, &req.data.args, 0x6)
 }
 
+pub(crate) fn sys_getsockopt(request: UNotifyEventRequest) -> ScmpNotifResp {
+    let req = request.scmpreq;
+    syscall_network_handler(request, &req.data.args, 0xf)
+}
+
 pub(crate) fn sys_connect(request: UNotifyEventRequest) -> ScmpNotifResp {
     let req = request.scmpreq;
     syscall_network_handler(request, &req.data.args, 0x3)
@@ -165,17 +174,7 @@ pub(crate) fn sys_sendmmsg(request: UNotifyEventRequest) -> ScmpNotifResp {
     syscall_network_handler(request, &req.data.args, 0x14)
 }
 
-/// A helper function to handle network-related syscalls.
-///
-/// This function abstracts the common logic involved in handling network syscalls such as `bind`,
-/// `connect`, `and `sendto` in a seccomp-based sandboxing environment. It reduces code duplication
-/// across different syscall handler functions.
-///
-/// # Returns
-///
-/// Returns `ScmpNotifResp` indicating the result of the syscall handling:
-/// - If successful, it contains a continued syscall.
-/// - If an error occurs, it contains a failed syscall with an `EACCES` error code.
+// A helper function to handle network-related syscalls.
 #[allow(clippy::cognitive_complexity)]
 fn syscall_network_handler(request: UNotifyEventRequest, args: &[u64; 6], op: u8) -> ScmpNotifResp {
     syscall_handler!(request, |request: UNotifyEventRequest| {
@@ -188,16 +187,23 @@ fn syscall_network_handler(request: UNotifyEventRequest, args: &[u64; 6], op: u8
 
         let cap = match op {
             0x1 => {
-                // a. socket(2) system call.
-                // b. socketcall(2) -> socket(2) indirection.
+                // a. socket system call.
+                // b. socketcall -> socket indirection.
                 let flags = *sandbox.flags;
                 let nlfam = sandbox.netlink_families;
                 drop(sandbox); // drop read-lock before emulation.
 
                 return handle_socket(&request, args, flags, nlfam);
             }
-            0x2 | 0x6 => Capability::CAP_NET_BIND,
-            _ => Capability::CAP_NET_CONNECT,
+            0x5 | 0x6 | 0x12 => {
+                // accept, accept4, getsockname:
+                //
+                // accept{,4} are IP blocklist only.
+                // getsockname is informational.
+                Capability::empty()
+            }
+            0x2 /* bind */ => Capability::CAP_NET_BIND,
+            _ /* connect, send{to,{m,}msg} */ => Capability::CAP_NET_CONNECT,
         };
         drop(sandbox); // release the read-lock before get-fd.
 
@@ -221,13 +227,18 @@ fn syscall_network_handler(request: UNotifyEventRequest, args: &[u64; 6], op: u8
 
         match op {
             0x5 | 0x12 => {
-                // accept{,4} uses a different data structure, so we handle it in its own branch.
+                // accept{,4} uses a different data structure,
+                // so we handle it in its own branch.
                 return handle_accept(fd, &request, args, op);
             }
             0x6 => {
                 // getsockname is used for informational purposes only.
                 return handle_getsockname(fd, &request, args);
             }
+            0xf => {
+                // getsockopt is used for informational purposes only.
+                return handle_getsockopt(fd, &request, args);
+            }
             0x10 => {
                 // sendmsg uses a different data structure, so we handle it in its own branch.
                 return handle_sendmsg(
@@ -294,23 +305,17 @@ fn syscall_network_handler(request: UNotifyEventRequest, args: &[u64; 6], op: u8
                 // SAFETY: We do not check Netlink address for access.
                 // We apply filtering on netlink families at socket level.
             }
-            PF_ALG if allow_safe_kcapi && op == 0x2 => {
+            PF_ALG if allow_safe_kcapi && op == 0x2 /*bind*/ => {
                 // SAFETY: Admin requested access to KCAPI.
             }
             PF_ALG => {
                 // a. SAFETY: Access to KCAPI is disabled by default.
-                // b. Non-bind() call is not supported for AF_ALG socket.
+                // b. Non-bind(2) call is not supported for AF_ALG socket.
                 return Err(Errno::EOPNOTSUPP);
             }
-            n if n >= PF_MAX => return Err(Errno::EINVAL),
-            _ if allow_unsupp_socket => {
-                // SAFETY: This is also restricted at socket(2) boundary
-                // where there is no pointer-dereference in access
-                // check. However, socketcall() multiplexing is an
-                // exception which introduces a TOCTOU window here.
-                return unsafe { Ok(request.continue_syscall()) };
-            }
-            _ => return Err(Errno::EAFNOSUPPORT),
+            n if n >= PF_MAX => return Err(Errno::EAFNOSUPPORT),
+            _ if !allow_unsupp_socket => return Err(Errno::EAFNOSUPPORT),
+            _ => {}, // fall-through to emulate, continue here is unsafe.
         };
         drop(sandbox); // release the read-lock.
 
@@ -329,14 +334,21 @@ fn get_addr(
     addr_remote: u64,
     addr_len: libc::socklen_t,
 ) -> Result<SockaddrStorage, Errno> {
-    // SAFETY: Do not fully trust addr_len.
+    // SAFETY:
+    // 1. Do not fully trust addr_len.
+    // 2. Return EINVAL on negative or zero addr_len.
+    let addr_len: usize = addr_len.try_into().or(Err(Errno::EINVAL))?;
+    if addr_len < 3 {
+        return Err(Errno::EINVAL);
+    }
     #[allow(clippy::arithmetic_side_effects)]
-    #[allow(clippy::cast_possible_truncation)]
-    let addr_len =
-        addr_len.min((std::mem::size_of::<libc::sockaddr_un>() + UNIX_PATH_MAX) as libc::socklen_t);
+    let addr_len = addr_len.min(std::mem::size_of::<libc::sockaddr_un>() + UNIX_PATH_MAX);
+
     let mut addr = Vec::new();
-    addr.try_reserve(addr_len as usize).or(Err(Errno::ENOMEM))?;
-    addr.resize(addr_len as usize, 0);
+    addr.try_reserve(addr_len).or(Err(Errno::ENOMEM))?;
+    addr.resize(addr_len, 0);
+    let addr_len = addr_len.try_into().or(Err(Errno::EINVAL))?;
+
     request.read_mem(&mut addr, addr_remote)?;
     let addr = addr.as_ptr().cast();
 
@@ -348,6 +360,18 @@ fn get_addr(
         #[allow(clippy::cast_ptr_alignment)]
         SockaddrStorage::from_raw(addr, Some(addr_len))
     } {
+        Some(addr)
+            if addr.as_sockaddr_in().is_some()
+                && (addr_len as usize) < std::mem::size_of::<libc::sockaddr_in>() =>
+        {
+            Err(Errno::EINVAL)
+        }
+        Some(addr)
+            if addr.as_sockaddr_in6().is_some()
+                && (addr_len as usize) < std::mem::size_of::<libc::sockaddr_in6>() =>
+        {
+            Err(Errno::EINVAL)
+        }
         Some(addr) => Ok(addr),
         None => {
             // Invalid socket address.
@@ -497,10 +521,17 @@ pub(crate) fn sandbox_addr(
     op: u8,
     caps: Capability,
 ) -> Result<(), Errno> {
+    // bind, connect, accept, sendto, sendmsg, accept4, sendmmsg
+    assert!(
+        matches!(op, 0x2 | 0x3 | 0x5 | 0xb | 0x10 | 0x12 | 0x14),
+        "BUG: sandbox_addr called with invalid op:{op:#x}, report a bug!"
+    );
+
     match addr.family() {
         Some(AddressFamily::Unix) => sandbox_addr_unix(request, sandbox, addr, root, op, caps),
-        Some(AddressFamily::Inet) => sandbox_addr_inet(request, sandbox, addr, op, caps),
-        Some(AddressFamily::Inet6) => sandbox_addr_inet6(request, sandbox, addr, op, caps),
+        Some(AddressFamily::Inet | AddressFamily::Inet6) => {
+            sandbox_addr_inet(request, sandbox, addr, op, caps)
+        }
         Some(_) | None => sandbox_addr_notsup(sandbox),
     }
 }
@@ -515,6 +546,12 @@ pub(crate) fn sandbox_addr_unix(
     op: u8,
     caps: Capability,
 ) -> Result<(), Errno> {
+    // bind, connect, sendto, sendmsg, sendmmsg
+    assert!(
+        matches!(op, 0x2 | 0x3 | 0xb | 0x10 | 0x14),
+        "BUG: sandbox_addr_unix called with invalid op:{op:#x}, report a bug!"
+    );
+
     if sandbox.getcaps(caps).is_empty() {
         // Sandboxing is off.
         return Ok(());
@@ -570,16 +607,15 @@ pub(crate) fn sandbox_addr_unix(
 
     if !filter {
         let sys = op2name(op);
-        let grp = if sys == "bind" { "bind" } else { "connect" };
         if sandbox.verbose {
             warn!("ctx": "access", "cap": caps, "act": action,
                 "sys": sys, "unix": &path, "abs": abs,
-                "tip": format!("configure `allow/net/{grp}+{path}'"),
+                "tip": format!("configure `allow/{caps}+{path}'"),
                 "req": request);
         } else {
             warn!("ctx": "access", "cap": caps, "act": action,
                 "sys": sys, "unix": &path, "abs": abs,
-                "tip": format!("configure `allow/net/{grp}+{path}'"),
+                "tip": format!("configure `allow/{caps}+{path}'"),
                 "pid": request.scmpreq.pid);
         }
     }
@@ -597,7 +633,7 @@ pub(crate) fn sandbox_addr_unix(
     }
 }
 
-/// Process an `AddressFamily::Inet` socket address.
+/// Process an IPv4 or IPv6 address.
 #[allow(clippy::cognitive_complexity)]
 pub(crate) fn sandbox_addr_inet(
     request: &UNotifyEventRequest,
@@ -606,97 +642,76 @@ pub(crate) fn sandbox_addr_inet(
     op: u8,
     caps: Capability,
 ) -> Result<(), Errno> {
-    if sandbox.getcaps(caps).is_empty() {
+    // accept(2) and accept4(2) are treated specially:
+    // No ACL is done, only IP blocklist check.
+    //
+    // First branch is accept, accept4.
+    // Second branch is bind, connect, send{to,{m,}msg}.
+    if matches!(op, 0x5 | 0x12) {
+        assert!(
+            caps.is_empty(),
+            "BUG: sandbox_addr_inet called with op:{op:#x} and {caps}, report a bug!"
+        );
+    } else if matches!(op, 0x2 | 0x3 | 0xb | 0x10 | 0x14) {
+        assert!(
+            !caps.is_empty(),
+            "BUG: sandbox_addr_inet called with op:{op:#x} and without caps, report a bug!"
+        );
+    } else {
+        unreachable!("BUG: sandbox_addr_inet called with op:{op:#x}, report a bug!");
+    }
+
+    if !caps.is_empty() && sandbox.getcaps(caps).is_empty() {
         // Sandboxing is off.
         return Ok(());
     }
 
-    let addr = addr.as_sockaddr_in().ok_or(Errno::EINVAL)?;
-    let port = addr.port();
-    let addr = IpAddr::V4(addr.ip());
-
-    let (action, filter) = if matches!(op, 0x5 | 0x12) {
-        // accept{,4}: Check for IP blocklist.
-        sandbox.check_block(addr)
+    let (addr, port) = if let Some(sin) = addr.as_sockaddr_in() {
+        (IpAddr::V4(sin.ip()), sin.port())
+    } else if let Some(sa6) = addr.as_sockaddr_in6() {
+        if let Some(v4addr) = sa6.ip().to_ipv4_mapped() {
+            // Mapped IPv4 address, convert to IPv4.
+            (IpAddr::V4(v4addr), sa6.port())
+        } else {
+            // Regular IPv6 address, check as is.
+            (IpAddr::V6(sa6.ip()), sa6.port())
+        }
     } else {
-        // Check for access.
-        sandbox.check_ip(caps, addr, port)
+        return Err(Errno::EINVAL);
     };
 
-    if !filter {
+    // Check for access and IP blocklist as necessary.
+    // caps.is_empty() implies accept{,4}(2) here.
+    let (action, filter) = sandbox.check_ip(caps, addr, port);
+
+    if caps.is_empty() && !filter {
+        // accept{,4}
+        let ipv = if addr.is_ipv6() { 6 } else { 4 };
         let sys = op2name(op);
-        let grp = if sys == "bind" { "bind" } else { "connect" };
         if sandbox.verbose {
-            warn!("ctx": "access", "cap": caps, "act": action,
-                "sys": sys, "addr": format!("{addr}!{port}"),
-                "tip": format!("configure `allow/net/{grp}+{addr}!{port}'"),
+            warn!("ctx": "block", "act": action,
+                "sys": sys, "addr": format!("{addr}!{port}"), "ipv": ipv,
+                "tip": format!("configure `block-{addr}'"),
                 "req": request);
         } else {
-            warn!("ctx": "access", "cap": caps, "act": action,
-                "sys": sys, "addr": format!("{addr}!{port}"),
-                "tip": format!("configure `allow/net/{grp}+{addr}!{port}'"),
+            warn!("ctx": "block", "act": action,
+                "sys": sys, "addr": format!("{addr}!{port}"), "ipv": ipv,
+                "tip": format!("configure `block-{addr}'"),
                 "pid": request.scmpreq.pid);
         }
-    }
-
-    match action {
-        Action::Allow | Action::Warn => Ok(()),
-        Action::Deny | Action::Filter => Err(op2errno(op)),
-        Action::Panic => panic!(),
-        Action::Exit => std::process::exit(op2errno(op) as i32),
-        action => {
-            // Stop|Kill
-            let _ = request.kill(action);
-            Err(op2errno(op))
-        }
-    }
-}
-
-/// Process an `AddressFamily::Inet6` socket address.
-#[allow(clippy::cognitive_complexity)]
-pub(crate) fn sandbox_addr_inet6(
-    request: &UNotifyEventRequest,
-    sandbox: &SandboxGuard,
-    addr: &SockaddrStorage,
-    op: u8,
-    caps: Capability,
-) -> Result<(), Errno> {
-    if sandbox.getcaps(caps).is_empty() {
-        // Sandboxing is off.
-        return Ok(());
-    }
-
-    let addr = addr.as_sockaddr_in6().ok_or(Errno::EINVAL)?;
-    let port = addr.port();
-    // Check if the IPv6 address is a mapped IPv4 address
-    let (addr, ipv) = if let Some(v4addr) = addr.ip().to_ipv4_mapped() {
-        // It's a mapped IPv4 address, convert to IPv4
-        (IpAddr::V4(v4addr), 4)
-    } else {
-        // It's a regular IPv6 address
-        (IpAddr::V6(addr.ip()), 6)
-    };
-
-    let (action, filter) = if matches!(op, 0x5 | 0x12) {
-        // accept{,4}: Check for IP blocklist.
-        sandbox.check_block(addr)
-    } else {
-        // Check for access.
-        sandbox.check_ip(caps, addr, port)
-    };
-
-    if !filter {
+    } else if !filter {
+        // connect, sendto, send{m,}msg
+        let ipv = if addr.is_ipv6() { 6 } else { 4 };
         let sys = op2name(op);
-        let grp = if sys == "bind" { "bind" } else { "connect" };
         if sandbox.verbose {
             warn!("ctx": "access", "cap": caps, "act": action,
                 "sys": sys, "addr": format!("{addr}!{port}"), "ipv": ipv,
-                "tip": format!("configure `allow/net/{grp}+{addr}!{port}'"),
+                "tip": format!("configure `allow/{caps}+{addr}!{port}'"),
                 "req": request);
         } else {
             warn!("ctx": "access", "cap": caps, "act": action,
                 "sys": sys, "addr": format!("{addr}!{port}"), "ipv": ipv,
-                "tip": format!("configure `allow/net/{grp}+{addr}!{port}'"),
+                "tip": format!("configure `allow/{caps}+{addr}!{port}'"),
                 "pid": request.scmpreq.pid);
         }
     }
@@ -737,15 +752,14 @@ pub(crate) fn handle_safe_bind<Fd: AsFd>(
 ) -> Result<(), Errno> {
     let addr = getsockname::<SockaddrStorage>(fd.as_fd().as_raw_fd())?;
 
-    let addr = if let Some(addr) = addr.as_sockaddr_in() {
+    let (addr, port) = if let Some(addr) = addr.as_sockaddr_in() {
         let port = addr.port();
         if port == 0 {
             return Ok(());
         }
 
         // Allow implicit bind with safe_bind.
-        let addr = IpAddr::V4(addr.ip());
-        format!("{addr}!{port}")
+        (IpNet::new_assert(IpAddr::V4(addr.ip()), 32), port)
     } else if let Some(addr) = addr.as_sockaddr_in6() {
         let port = addr.port();
         if port == 0 {
@@ -753,27 +767,27 @@ pub(crate) fn handle_safe_bind<Fd: AsFd>(
         }
 
         // Allow implicit bind with safe_bind.
-        let addr = IpAddr::V6(addr.ip());
-        format!("{addr}!{port}")
+        (IpNet::new_assert(IpAddr::V6(addr.ip()), 128), port)
     } else {
         return Ok(());
     };
 
-    // Configure sandbox, note we remove
-    // and readd the address so repeated
-    // binds to the same address cannot
-    // overflow the vector.
-    let config: &[String] = &[
-        format!("allow/net/connect-{addr}"),
-        format!("allow/net/connect+{addr}"),
-    ];
-
-    // TODO: Log errors!
+    // Configure sandbox:
+    // Remove and re-add the address so repeated binds to the
+    // same address cannot overflow the vector.
+    let rule = CidrRule {
+        act: Action::Allow,
+        cap: Capability::CAP_NET_CONNECT,
+        pat: AddressPattern {
+            addr,
+            port: Some(port..=port),
+        },
+    };
     let mut sandbox = request.get_mut_sandbox();
-    for cmd in config {
-        sandbox.config(cmd)?;
+    if let Some(idx) = sandbox.cidr_rules.iter().position(|r| *r == rule) {
+        sandbox.cidr_rules.remove(idx);
     }
-    drop(sandbox);
+    sandbox.cidr_rules.push_front(rule)?;
 
     Ok(())
 }
