@@ -60,11 +60,11 @@ pub(crate) fn handle_sendmsg(
     let size = if is32 {
         // Note, socketcall is the same number on all:
         // x86, mips, mipsel, ppc, ppc64, ppc64le, s390 and s390x.
-        std::mem::size_of::<msghdr32>()
+        size_of::<msghdr32>()
     } else {
-        std::mem::size_of::<msghdr>()
+        size_of::<msghdr>()
     };
-    let mut buf = Vec::new();
+    let mut buf = Zeroizing::new(Vec::new());
     buf.try_reserve(size).or(Err(Errno::ENOMEM))?;
     buf.resize(size, 0);
     request.read_mem(&mut buf, args[1])?;
@@ -156,6 +156,8 @@ pub(crate) fn handle_sendmsg(
         ref control_ipv6_packet_infos,
         ref control_rxq_ovfls,
         ref control_tx_times,
+        ref control_ipv4_toses,
+        ref control_ipv6_tclasses,
     )) = control_data
     {
         // Check for sendfd access as necessary.
@@ -247,6 +249,14 @@ pub(crate) fn handle_sendmsg(
         for tx_time in control_tx_times {
             control_messages.push(ControlMessage::TxTime(tx_time));
         }
+
+        for tos in control_ipv4_toses {
+            control_messages.push(ControlMessage::Ipv4Tos(tos));
+        }
+
+        for tclass in control_ipv6_tclasses {
+            control_messages.push(ControlMessage::Ipv6TClass(tclass));
+        }
     }
     drop(sandbox); // release the read-lock.
 
@@ -262,12 +272,12 @@ pub(crate) fn handle_sendmsg(
             .or(Err(Errno::EINVAL))?
             .min(1000000); // Cap count at 1mio
         let size = if is32 {
-            len.checked_mul(std::mem::size_of::<crate::compat::iovec32>())
+            len.checked_mul(size_of::<crate::compat::iovec32>())
         } else {
-            len.checked_mul(std::mem::size_of::<libc::iovec>())
+            len.checked_mul(size_of::<libc::iovec>())
         }
         .ok_or(Errno::EINVAL)?;
-        let mut buf = Vec::new();
+        let mut buf = Zeroizing::new(Vec::new());
         buf.try_reserve(size).or(Err(Errno::ENOMEM))?;
         buf.resize(size, 0);
         request.read_mem(&mut buf, msg.msg_iov as u64)?;
@@ -281,14 +291,14 @@ pub(crate) fn handle_sendmsg(
         // correct and `buf` is correctly sized and aligned.
         let mut iovecs: Vec<libc::iovec> = Vec::new();
         if is32 {
-            for chunk in buf.chunks(std::mem::size_of::<crate::compat::iovec32>()) {
+            for chunk in buf.chunks(size_of::<crate::compat::iovec32>()) {
                 // SAFETY: See above.
                 let iov32: crate::compat::iovec32 =
                     unsafe { std::ptr::read_unaligned(chunk.as_ptr() as *const _) };
                 iovecs.push(iov32.into());
             }
         } else {
-            for chunk in buf.chunks(std::mem::size_of::<libc::iovec>()) {
+            for chunk in buf.chunks(size_of::<libc::iovec>()) {
                 // SAFETY: See above.
                 iovecs.push(unsafe { std::ptr::read_unaligned(chunk.as_ptr() as *const _) });
             }
@@ -401,21 +411,21 @@ pub(crate) fn handle_sendmmsg(
     let is32 = scmp_arch_bits(req.data.arch) == 32;
 
     let size = if is32 {
-        vlen.checked_mul(std::mem::size_of::<crate::compat::mmsghdr32>())
+        vlen.checked_mul(size_of::<crate::compat::mmsghdr32>())
     } else {
-        vlen.checked_mul(std::mem::size_of::<crate::compat::mmsghdr>())
+        vlen.checked_mul(size_of::<crate::compat::mmsghdr>())
     }
     .ok_or(Errno::EINVAL)?;
 
     // Read mmsghdr structures from remote process memory
-    let mut buf = Vec::new();
+    let mut buf = Zeroizing::new(Vec::new());
     buf.try_reserve(size).or(Err(Errno::ENOMEM))?;
     buf.resize(size, 0);
     request.read_mem(&mut buf, addr)?;
 
     let mut mmsghdrs: Vec<crate::compat::mmsghdr> = Vec::new();
     if is32 {
-        for chunk in buf.chunks(std::mem::size_of::<crate::compat::mmsghdr32>()) {
+        for chunk in buf.chunks(size_of::<crate::compat::mmsghdr32>()) {
             // SAFETY: See the relevant comment in handle_sendmsg.
             let mmsghdr: crate::compat::mmsghdr32 =
                 unsafe { std::ptr::read_unaligned(chunk.as_ptr() as *const _) };
@@ -423,7 +433,7 @@ pub(crate) fn handle_sendmmsg(
             mmsghdrs.push(mmsghdr.into());
         }
     } else {
-        for chunk in buf.chunks(std::mem::size_of::<crate::compat::mmsghdr>()) {
+        for chunk in buf.chunks(size_of::<crate::compat::mmsghdr>()) {
             mmsghdrs.try_reserve(1).or(Err(Errno::ENOMEM))?;
             // SAFETY: See the relevant comment in handle_sendmsg.
             mmsghdrs.push(unsafe { std::ptr::read_unaligned(chunk.as_ptr() as *const _) });
@@ -514,6 +524,8 @@ pub(crate) fn handle_sendmmsg(
                 ref control_ipv6_packet_infos,
                 ref control_rxq_ovfls,
                 ref control_tx_times,
+                ref control_ipv4_toses,
+                ref control_ipv6_tclasses,
             )) = &control_datum
             {
                 // Check for sendfd access as necessary.
@@ -606,6 +618,14 @@ pub(crate) fn handle_sendmmsg(
                 for tx_time in control_tx_times {
                     control_messages.push(ControlMessage::TxTime(tx_time));
                 }
+
+                for tos in control_ipv4_toses {
+                    control_messages.push(ControlMessage::Ipv4Tos(tos));
+                }
+
+                for tclass in control_ipv6_tclasses {
+                    control_messages.push(ControlMessage::Ipv6TClass(tclass));
+                }
             }
         }
         drop(sandbox); // release the read-lock before emulation.
@@ -622,19 +642,19 @@ pub(crate) fn handle_sendmmsg(
                 .or(Err(Errno::EINVAL))?
                 .min(1000000); // Cap count at 1mio.
             let size = if is32 {
-                len.checked_mul(std::mem::size_of::<crate::compat::iovec32>())
+                len.checked_mul(size_of::<crate::compat::iovec32>())
             } else {
-                len.checked_mul(std::mem::size_of::<libc::iovec>())
+                len.checked_mul(size_of::<libc::iovec>())
             }
             .ok_or(Errno::EINVAL)?;
-            let mut buf = Vec::new();
+            let mut buf = Zeroizing::new(Vec::new());
             buf.try_reserve(size).or(Err(Errno::ENOMEM))?;
             buf.resize(size, 0);
             request.read_mem(&mut buf, msg.msg_iov as u64)?;
 
             let mut iovecs: Vec<libc::iovec> = Vec::new();
             if is32 {
-                for chunk in buf.chunks(std::mem::size_of::<crate::compat::iovec32>()) {
+                for chunk in buf.chunks(size_of::<crate::compat::iovec32>()) {
                     // SAFETY: See the relevant comment in handle_sendmsg.
                     let iov32: crate::compat::iovec32 =
                         unsafe { std::ptr::read_unaligned(chunk.as_ptr() as *const _) };
@@ -642,7 +662,7 @@ pub(crate) fn handle_sendmmsg(
                     iovecs.push(iov32.into());
                 }
             } else {
-                for chunk in buf.chunks(std::mem::size_of::<libc::iovec>()) {
+                for chunk in buf.chunks(size_of::<libc::iovec>()) {
                     iovecs.try_reserve(1).or(Err(Errno::ENOMEM))?;
                     // SAFETY: See the relevant comment in handle_sendmsg.
                     iovecs.push(unsafe { std::ptr::read_unaligned(chunk.as_ptr() as *const _) });
@@ -713,13 +733,13 @@ pub(crate) fn handle_sendmmsg(
     drop(fd); // Close our copy of the socket.
 
     // Write back mmsghdr structures to remote process memory
-    let mut buf: Vec<u8> = Vec::new();
+    let mut buf: Zeroizing<Vec<u8>> = Zeroizing::new(Vec::new());
     if is32 {
         for mmsghdr in &mmsghdrs {
             let mmsghdr32: crate::compat::mmsghdr32 = (*mmsghdr).into();
             // SAFETY: Convert each mmsghdr (or mmsghdr32 within the
             // conversion logic) back to its byte representation.
-            let bytes: [u8; std::mem::size_of::<crate::compat::mmsghdr32>()] =
+            let bytes: [u8; size_of::<crate::compat::mmsghdr32>()] =
                 unsafe { std::mem::transmute(mmsghdr32) };
             buf.try_reserve(bytes.len()).or(Err(Errno::ENOMEM))?;
             buf.extend_from_slice(&bytes);
@@ -727,7 +747,7 @@ pub(crate) fn handle_sendmmsg(
     } else {
         for mmsghdr in &mmsghdrs {
             // SAFETY: See above.
-            let bytes: [u8; std::mem::size_of::<crate::compat::mmsghdr>()] =
+            let bytes: [u8; size_of::<crate::compat::mmsghdr>()] =
                 unsafe { std::mem::transmute(*mmsghdr) };
             buf.try_reserve(bytes.len()).or(Err(Errno::ENOMEM))?;
             buf.extend_from_slice(&bytes);
@@ -757,6 +777,8 @@ fn parse_control_messages(
         Vec<libc::in6_pktinfo>,
         Vec<u32>, // rxq_ovfls
         Vec<u64>, // tx_times
+        Vec<u8>,  // ipv4 tos
+        Vec<i32>, // ipv6 tclass
     ),
     Errno,
 > {
@@ -770,6 +792,8 @@ fn parse_control_messages(
     let mut control_ipv6_packet_infos = Vec::new();
     let mut control_rxq_ovfls = Vec::new();
     let mut control_tx_times = Vec::new();
+    let mut control_ipv4_toses = Vec::new();
+    let mut control_ipv6_tclasses = Vec::new();
 
     let mut offset = 0;
     let req = request.scmpreq;
@@ -820,17 +844,17 @@ fn parse_control_messages(
         match (cmsg_header.cmsg_level, cmsg_header.cmsg_type) {
             (libc::SOL_SOCKET, libc::SCM_RIGHTS) => {
                 let fd_count = data_len
-                    .checked_div(std::mem::size_of::<RawFd>())
+                    .checked_div(size_of::<RawFd>())
                     .ok_or(Errno::EINVAL)?;
 
                 let mut fds = Vec::new();
                 fds.try_reserve(fd_count).or(Err(Errno::ENOMEM))?;
 
-                // SAFETY: Multiplying `i` by `std::mem::size_of::<RawFd>()` calculates the
+                // SAFETY: Multiplying `i` by `size_of::<RawFd>()` calculates the
                 // offset for each file descriptor in the control message data. This is safe
                 // under the assumption that `data_len` (used to derive `fd_count`)
                 // correctly represents a buffer containing `RawFd`s. `data_len` is checked
-                // to ensure it's an exact multiple of `std::mem::size_of::<RawFd>()`,
+                // to ensure it's an exact multiple of `size_of::<RawFd>()`,
                 // preventing out-of-bounds access. Accessing the file descriptor using this
                 // offset and converting it with `request.get_fd()` is based on the valid and
                 // expected layout of file descriptors in the control message. This layout
@@ -839,7 +863,7 @@ fn parse_control_messages(
                 // file descriptor values from the buffer.
                 for i in 0..fd_count {
                     #[allow(clippy::arithmetic_side_effects)]
-                    let fd_offset = i * std::mem::size_of::<RawFd>();
+                    let fd_offset = i * size_of::<RawFd>();
                     #[allow(clippy::cast_ptr_alignment)]
                     // SAFETY: See the comment above.
                     let fd = unsafe { *(data[fd_offset..].as_ptr() as *const RawFd) };
@@ -902,13 +926,13 @@ fn parse_control_messages(
                 // assumes the beginning of `data` is correctly aligned for an `i32`, which
                 // is valid if the control message was constructed correctly by the sender
                 // according to the `AF_ALG` socket requirements. The check `data_len !=
-                // std::mem::size_of::<libc::c_int>()` ensures that the buffer length
+                // size_of::<libc::c_int>()` ensures that the buffer length
                 // exactly matches the size of an `i32`, mitigating the risk of undefined
                 // behavior due to incorrect buffer size. However, this operation bypasses
                 // Rust's guarantees on data alignment, relying on the correct alignment by
                 // the sender and adherence to the protocol's specification, which mandates
                 // proper alignment for control message data.
-                if data_len != std::mem::size_of::<libc::c_int>() {
+                if data_len != size_of::<libc::c_int>() {
                     return Err(Errno::EINVAL); // Data length mismatch
                 }
                 #[allow(clippy::cast_ptr_alignment)]
@@ -923,13 +947,13 @@ fn parse_control_messages(
                 // sender properly constructing the control message with the
                 // `ALG_SET_AEAD_ASSOCLEN` type, ensuring the alignment meets the
                 // requirements for `u32` data. The prerequisite check `data_len !=
-                // std::mem::size_of::<u32>()` ensures the buffer is exactly the size of a
+                // size_of::<u32>()` ensures the buffer is exactly the size of a
                 // `u32`, mitigating risks associated with accessing beyond the buffer or
                 // misinterpreting the data type.  While this operation inherently trusts
                 // the message sender to adhere to alignment requirements, it aligns with
                 // common practices for handling similarly structured control messages in
                 // systems programming, where protocol adherence guarantees data alignment.
-                if data_len != std::mem::size_of::<u32>() {
+                if data_len != size_of::<u32>() {
                     return Err(Errno::EINVAL); // Data length mismatch
                 }
                 #[allow(clippy::cast_ptr_alignment)]
@@ -943,14 +967,14 @@ fn parse_control_messages(
                 // aligned for `u16`. This is based on the expectation that the sender of
                 // the control message aligns the data according to the `u16` requirements
                 // when constructing the message for `UDP_SEGMENT`. The check `data_len !=
-                // std::mem::size_of::<u16>()` ensures that the buffer is precisely the size
+                // size_of::<u16>()` ensures that the buffer is precisely the size
                 // expected for a single `u16` value, thus avoiding potential overreads or
                 // misinterpretation of the buffer content. This cast and dereference
                 // operation is predicated on the alignment and size of the data being
                 // appropriate for a `u16`, as per the protocol's definition for UDP segment
                 // control messages, thereby justifying the bypass of Rust's alignment
                 // safety checks.
-                if data_len != std::mem::size_of::<u16>() {
+                if data_len != size_of::<u16>() {
                     return Err(Errno::EINVAL); // Data length mismatch
                 }
                 #[allow(clippy::cast_ptr_alignment)]
@@ -970,7 +994,7 @@ fn parse_control_messages(
                 // dereference to access the `in_pktinfo` is then justified under the
                 // assumption of proper alignment and correct data length, as mandated by
                 // the control message's protocol definition.
-                if data_len != std::mem::size_of::<libc::in_pktinfo>() {
+                if data_len != size_of::<libc::in_pktinfo>() {
                     return Err(Errno::EINVAL); // Data length mismatch
                 }
                 #[allow(clippy::cast_ptr_alignment)]
@@ -990,7 +1014,7 @@ fn parse_control_messages(
                 // is therefore considered safe, assuming adherence to the protocol by the
                 // message sender and that the data has been formatted and aligned correctly
                 // for the type of control message being processed.
-                if data_len != std::mem::size_of::<libc::in6_pktinfo>() {
+                if data_len != size_of::<libc::in6_pktinfo>() {
                     return Err(Errno::EINVAL); // Data length mismatch
                 }
                 #[allow(clippy::cast_ptr_alignment)]
@@ -1011,7 +1035,7 @@ fn parse_control_messages(
                 // represent a single `u32` value, corresponding to the RX queue overflow
                 // count. The correctness of this operation depends on adherence to these
                 // alignment and size specifications by the sender.
-                if data_len != std::mem::size_of::<u32>() {
+                if data_len != size_of::<u32>() {
                     return Err(Errno::EINVAL); // Data length mismatch
                 }
                 #[allow(clippy::cast_ptr_alignment)]
@@ -1031,7 +1055,7 @@ fn parse_control_messages(
                 // This cast and dereference assume that the control message's composition
                 // and alignment practices properly account for the alignment needs of a
                 // `u64`, making the operation safe under these controlled conditions.
-                if data_len != std::mem::size_of::<u64>() {
+                if data_len != size_of::<u64>() {
                     return Err(Errno::EINVAL); // Data length mismatch
                 }
                 #[allow(clippy::cast_ptr_alignment)]
@@ -1039,7 +1063,29 @@ fn parse_control_messages(
                 let tx_time = unsafe { *(data.as_ptr() as *const u64) };
                 control_tx_times.push(tx_time); // Store the Tx time
             }
-            // TODO: DO we want to CONTINUE here if allow_unsupported_socket?
+            (libc::IPPROTO_IP, libc::IP_TOS) => {
+                // IP_TOS ancillary data is an int.
+                if data_len != size_of::<libc::c_int>() {
+                    return Err(Errno::EINVAL);
+                }
+                // SAFETY: Exact int-sized payload validated; bounds and alignment guaranteed
+                // by prior checks and cmsg parsing semantics. Single c_int read is safe.
+                #[allow(clippy::cast_ptr_alignment)]
+                let val = unsafe { *(data.as_ptr() as *const libc::c_int) };
+                // See ip_cmd_send() in Linux' net/ipv4/ip_sockglue.c.
+                let val: u8 = val.try_into().or(Err(Errno::EINVAL))?;
+                control_ipv4_toses.push(val);
+            }
+            (libc::IPPROTO_IPV6, libc::IPV6_TCLASS) => {
+                if data_len != size_of::<i32>() {
+                    return Err(Errno::EINVAL);
+                }
+                // SAFETY: Exact int-sized payload validated; bounds and alignment guaranteed
+                // by prior checks and cmsg parsing semantics. Single c_int read is safe.
+                #[allow(clippy::cast_ptr_alignment)]
+                let val = unsafe { *(data.as_ptr() as *const i32) };
+                control_ipv6_tclasses.push(val);
+            }
             _ => return Err(Errno::EINVAL),
         }
 
@@ -1076,5 +1122,7 @@ fn parse_control_messages(
         control_ipv6_packet_infos,
         control_rxq_ovfls,
         control_tx_times,
+        control_ipv4_toses,
+        control_ipv6_tclasses,
     ))
 }
