@@ -26,7 +26,7 @@ use nix::{
 use crate::{
     compat::{fstatx, STATX_MODE, STATX_UID},
     config::{
-        ENV_ID, ESYD_SH, MAGIC_PREFIX, NULL_FD, PINK_FLOYD, PROC_FILE, ROOT_FILE, SAFE_MFD_FLAGS,
+        ENV_ID, ESYD_SH, MAGIC_PREFIX, NULL_FD, PINK_FLOYD, PROC_FILE, SAFE_MFD_FLAGS,
         SEE_EMILY_PLAY, SYD_EL, THE_PIPER,
     },
     confine::EIDRM,
@@ -232,6 +232,11 @@ fn syscall_open_handler(
             return Err(Errno::EINVAL);
         }
 
+        // O_TMPFILE must be combined with O_WRONLY or O_RDWR.
+        if o_tmpfl && !caps.contains(Capability::CAP_WRITE) {
+            return Err(Errno::EINVAL);
+        }
+
         let req = request.scmpreq;
         let sandbox = request.get_sandbox();
 
@@ -848,17 +853,14 @@ fn syscall_open_handler(
 
                 let fd: OwnedFd = if crypted {
                     // Handle Encryption.
-                    if let Some(fd) = handle_crypt(
-                        setup_fds,
-                        &request,
-                        &path,
-                        crypt_tmp,
-                        flags,
-                        mode,
-                        force_umask,
+                    let opts = CryptOpts {
                         o_cloexec,
                         o_rand_fd,
-                    )? {
+                        force_umask,
+                        setup_fds,
+                        tmpdir: crypt_tmp,
+                    };
+                    if let Some(fd) = handle_crypt(&request, &path, flags, mode, opts)? {
                         // read-only encryption.
                         fd
                     } else {
@@ -870,7 +872,7 @@ fn syscall_open_handler(
                         return Ok(ScmpNotifResp::new(0, 0, EIDRM, 0));
                     }
                 } else if path.base.is_empty() && !o_tmpfl {
-                    // Existing path: Construct path to /proc magic symlink,
+                    // Existing path: Construct path to proc(5) magic symlink,
                     // or pre-open file descriptor.
                     //
                     // SAFETY: Note, the path may be borrowed here, ie
@@ -957,7 +959,7 @@ fn syscall_open_handler(
                     // SAFETY: If we're creating the file,
                     // we need to change per-thread umask and
                     // still honour POSIX ACLs.
-                    handle_creat(&request, &path, flags, mode, force_umask)?
+                    do_creat(&request, &path, flags, mode, force_umask)?
                 } else {
                     unreachable!("BUG: MUST_PATH returned a directory for open, report a bug!");
                 };
@@ -978,58 +980,23 @@ fn syscall_open_handler(
     })
 }
 
-/// Handle open() calls.
-fn do_open(path: &CanonicalPath, mut safe_flags: OFlag) -> Result<OwnedFd, Errno> {
-    if path.base.is_empty() {
-        // Existing path: Construct path to proc magic symlink.
-        //
-        // SAFETY: ^^ empty base asserts dir is Some.
-        #[allow(clippy::disallowed_methods)]
-        let fd = path.dir.as_ref().map(|fd| fd.as_raw_fd()).unwrap();
-        let pfd = XPathBuf::from_self_fd(fd);
-
-        // Reopen the `O_PATH` path fd with the requested flags.
-        safe_flags.remove(OFlag::O_NOFOLLOW);
-
-        safe_open_msym(PROC_FILE(), &pfd, safe_flags, ResolveFlag::empty())
-    } else {
-        // Return a read-only fd to the underlying encrypted file.
-        let how = OpenHow::new().flags(safe_flags).resolve(
-            ResolveFlag::RESOLVE_BENEATH
-                | ResolveFlag::RESOLVE_NO_MAGICLINKS
-                | ResolveFlag::RESOLVE_NO_SYMLINKS,
-        );
-
-        let (fd, base) = if let Some(ref fd) = path.dir {
-            (fd.as_fd(), path.base)
-        } else if path.abs().is_proc() {
-            (
-                PROC_FILE(),
-                XPath::from_bytes(&path.abs().as_bytes()[b"/proc/".len()..]),
-            )
-        } else {
-            (
-                ROOT_FILE(),
-                XPath::from_bytes(&path.abs().as_bytes()[b"/".len()..]),
-            )
-        };
-
-        safe_openat2(fd, base, how)
-    }
+/// Options for `handle_crypt` function.
+#[derive(Copy, Clone, Debug)]
+struct CryptOpts {
+    o_cloexec: bool,
+    o_rand_fd: bool,
+    force_umask: Option<Mode>,
+    tmpdir: Option<RawFd>,
+    setup_fds: Result<(RawFd, RawFd), Errno>,
 }
 
 /// Handle open() calls for encrypted files.
-#[allow(clippy::too_many_arguments)]
 fn handle_crypt(
-    setup_fds: Result<(RawFd, RawFd), Errno>,
     request: &UNotifyEventRequest,
     path: &CanonicalPath,
-    tmpdir: Option<RawFd>,
     safe_flags: OFlag,
     mode: Mode,
-    force_umask: Option<Mode>,
-    o_cloexec: bool,
-    o_rand_fd: bool,
+    opts: CryptOpts,
 ) -> Result<Option<OwnedFd>, Errno> {
     #[allow(clippy::disallowed_methods)]
     let files = request.crypt_map.as_ref().unwrap();
@@ -1037,23 +1004,23 @@ fn handle_crypt(
 
     // Check for concurrent read access to the same crypt fd.
     if my_mode == AesMod::Read {
-        // Lock AesMap for read:
-        // We keep the read lock until we open the file,
-        // to ensure no races occur with a subsequent Syd thread.
-        let files = files.read().unwrap_or_else(|err| err.into_inner());
-
+        // Lock the mutex and attempt to take the crypt fd.
         // Concurrent access is only allowed if both modes are Read.
-        let entry = files
-            .get(path.abs())
-            .filter(|&entry| entry.mode == AesMod::Read);
+        let crypt_fd_opt = {
+            let files = files.0.lock().unwrap_or_else(|err| err.into_inner());
+            files
+                .get(path.abs())
+                .filter(|&entry| entry.mode == AesMod::Read)
+                .map(|aes_val| aes_val.crypt_fd)
+        }; // Lock is released here.
 
-        if let Some(entry) = entry {
+        if let Some(crypt_fd) = crypt_fd_opt {
             // Open a new file description.
             // Lock it for read and pass to sandbox process.
             let mut flags = safe_flags;
             flags.remove(OFlag::O_NOFOLLOW);
 
-            let pfd = XPathBuf::from_self_fd(entry.crypt_fd);
+            let pfd = XPathBuf::from_self_fd(crypt_fd);
             match safe_open_msym(PROC_FILE(), &pfd, flags, ResolveFlag::empty()) {
                 Ok(fd) if retry_on_eintr(|| lock_fd(&fd, false, false)).is_ok() => {
                     // Return shared fd.
@@ -1091,12 +1058,25 @@ fn handle_crypt(
     // Open or create the file as read-write.
     //
     // Note, if `path.base.is_empty()` we have an existing file
-    // which do_open is going to reopen using proc magic symlink.
-    let create = !path.base.is_empty() && safe_flags.contains(OFlag::O_CREAT);
-    let enc_fd = if create {
-        handle_creat(request, path, flags, mode, force_umask)
+    // which do_open_crypt is going to reopen using proc(5) magic
+    // symlink.
+    let create = if path.base.is_empty() {
+        // Strip O_CREAT when NOT creating.
+        flags.remove(OFlag::O_CREAT);
+        // O_TMPFILE is valid if base is empty, when
+        // we create a temporary file under parent-fd.
+        flags.contains(OFlag::O_TMPFILE)
     } else {
-        do_open(path, flags)
+        assert!(
+            !flags.contains(OFlag::O_TMPFILE),
+            "BUG: O_CREAT|O_TMPFILE flags are invalid, report a bug!"
+        );
+        flags.contains(OFlag::O_CREAT)
+    };
+    let enc_fd = if create {
+        do_creat(request, path, flags, mode, opts.force_umask)
+    } else {
+        do_open_crypt(path, flags)
     }?;
 
     // Hold a write-lock to ensure no concurrent Syd aes writes.
@@ -1112,7 +1092,7 @@ fn handle_crypt(
     retry_on_eintr(|| lock_fd(&enc_fd, true, create || wait))?;
 
     // Acquire encryption sockets.
-    let setup_fds = setup_fds?;
+    let setup_fds = opts.setup_fds?;
 
     // Map decrypted version to memory/temporary FD.
     // SAFETY: This does not read plaintext into memory!
@@ -1120,7 +1100,8 @@ fn handle_crypt(
     // Note, enc_fd is an OwnedFd so in the event of
     // an aes_ctr_tmp error it'll be closed and the lock
     // will be released.
-    let (fd, iv) = if let Some((fd, iv)) = aes_ctr_tmp(setup_fds, &enc_fd, safe_flags, tmpdir)? {
+    let (fd, iv) = if let Some((fd, iv)) = aes_ctr_tmp(setup_fds, &enc_fd, safe_flags, opts.tmpdir)?
+    {
         (fd, iv)
     } else {
         // SAFETY:
@@ -1151,28 +1132,31 @@ fn handle_crypt(
     // No need to wait on this lock, as we've just opened the file.
     // Close the send fd to get rid off our copy of the lock.
     retry_on_eintr(|| lock_fd(&aes_fd, false, false))?;
-    request.send_fd(aes_fd, o_cloexec, o_rand_fd)?;
+    request.send_fd(aes_fd, opts.o_cloexec, opts.o_rand_fd)?;
 
-    // Record encryption process information.
+    // Record encryption process information,
+    // and notify the syd_aes thread to wake up.
     {
-        files.write().unwrap_or_else(|err| err.into_inner()).insert(
+        let (aes_map, cvar) = &**files;
+        let mut aes_map = aes_map.lock().unwrap_or_else(|e| e.into_inner());
+        aes_map.insert(
             path.abs().to_owned(),
             AesVal {
                 info,
                 iv: Some(iv),
-                enc_fd: Some(enc_fd),
+                enc_fd: enc_fd.into_raw_fd(),
                 crypt_fd: fd.into_raw_fd(),
                 mode: my_mode,
-                spawned: false,
             },
         );
-    }
+        cvar.notify_one();
+    } // Lock is released here.
 
     Ok(None)
 }
 
-/// Handle open() calls that can potentially create files.
-fn handle_creat(
+/// Handle open(2) calls that can potentially create files.
+fn do_creat(
     request: &UNotifyEventRequest,
     path: &CanonicalPath,
     safe_flags: OFlag,
@@ -1239,6 +1223,37 @@ fn to_oflag(arg: u64, truncate: bool) -> Result<OFlag, Errno> {
     Ok(flags | preserved)
 }
 
+// Handle open(2) calls for Crypt sandboxing for existing paths.
+//
+// This function has no internal encryption logic,
+// and it's only used by Crypt sandboxing when opening existing paths
+// using proc(5) magic symlink, otherwise do_creat() is used.
+fn do_open_crypt(path: &CanonicalPath, mut safe_flags: OFlag) -> Result<OwnedFd, Errno> {
+    assert!(
+        path.base.is_empty(),
+        "BUG: non-canonical path passed to do_open_crypt, report a bug!"
+    );
+    assert!(
+        !safe_flags.contains(OFlag::O_CREAT),
+        "BUG: O_CREAT passed to do_open_crypt, report a bug!"
+    );
+    assert!(
+        !safe_flags.contains(OFlag::O_TMPFILE),
+        "BUG: O_TMPFILE passed to do_open_crypt, report a bug!"
+    );
+    safe_flags.remove(OFlag::O_NOFOLLOW);
+
+    // Construct path to proc(5) magic symlink.
+    //
+    // SAFETY: ^^ empty base asserts dir is Some.
+    #[allow(clippy::disallowed_methods)]
+    let fd = path.dir.as_ref().map(|fd| fd.as_raw_fd()).unwrap();
+    let pfd = XPathBuf::from_self_fd(fd);
+
+    // Reopen the `O_PATH` path fd with the requested flags.
+    safe_open_msym(PROC_FILE(), &pfd, safe_flags, ResolveFlag::empty())
+}
+
 #[inline]
 fn to_mode(arg: u64, flags: OFlag, ignore: bool) -> Result<Mode, Errno> {
     if flags.contains(OFlag::O_CREAT) || flags.contains(OFlag::O_TMPFILE) {
