@@ -158,6 +158,8 @@ pub mod path;
 pub mod proc;
 /// ptrace(2) utilities
 pub mod ptrace;
+/// OS Random Number Generator (RNG) interface
+pub mod rng;
 /// Sandbox configuration
 pub mod sandbox;
 /// Execute program as sealed anonymous file
@@ -175,6 +177,8 @@ pub mod sysinfo;
 pub mod syslog;
 /// Per-thread SIGALRM timer
 pub mod timer;
+/// Interface to uname(2)
+pub mod uts;
 /// Shell-style wildcard matching
 #[allow(clippy::arithmetic_side_effects)]
 pub mod wildmatch;
@@ -238,7 +242,10 @@ use serde::{Serialize, Serializer};
 
 use crate::{
     compat::{fstatx, lsm_list_modules},
-    confine::{is_coredump, lock_enabled, ns_enabled, seccomp_arch_native_name, SydPersona},
+    confine::{
+        check_cross_memory_attach, check_unix_diag, is_coredump, lock_enabled, ns_enabled,
+        seccomp_arch_native_name, vdso_list_calls, SydPersona,
+    },
     err::err2no,
     hash::{aes_ctr_info, check_setsockopt_serial_support, hmac_sha256_info, key_ring_validate},
     landlock::ABI,
@@ -278,19 +285,6 @@ macro_rules! main {
 
 /* Utilities */
 
-/// Check for CONFIG_CROSS_MEMORY_ATTACH support.
-pub fn check_cross_memory_attach() -> bool {
-    // SAFETY: We explicitly call the system call with
-    // invalid arguments just to check for host Linux
-    // kernel support.
-    !matches!(
-        Errno::result(unsafe {
-            libc::process_vm_readv(0, std::ptr::null(), 0, std::ptr::null(), 0, 0)
-        }),
-        Err(Errno::ENOSYS)
-    )
-}
-
 /// Print Syd version information,
 /// and information about the system to
 /// standard output.
@@ -409,6 +403,14 @@ pub fn syd_info(verbose: bool) -> Result<(), Errno> {
         if check_mseal_support() { "" } else { "not " }
     )?;
 
+    // Check CONFIG_UNIX_DIAG support.
+    let unix_diag = match check_unix_diag() {
+        Ok(true) => "supported".to_string(),
+        Ok(false) => "not supported".to_string(),
+        Err(errno) => format!("unknown (error: {errno})"),
+    };
+    printfln!("UNIX socket diagnostics are {unix_diag}.")?;
+
     // Print ALG_SET_KEY_BY_KEY_SERIAL support.
     printfln!(
         "Algorithm sockets {} keyrings(7) support.",
@@ -434,6 +436,7 @@ pub fn syd_info(verbose: bool) -> Result<(), Errno> {
     // Print hmac(sha256) kernel support.
     printfln!("{}", hmac_sha256_info())?;
 
+    // List LSMs.
     let lsms = match lsm_list_modules() {
         Ok(lsms) => lsms
             .into_iter()
@@ -445,6 +448,20 @@ pub fn syd_info(verbose: bool) -> Result<(), Errno> {
     };
     printfln!("LSMs: {lsms}.")?;
 
+    // List vDSO calls.
+    match vdso_list_calls() {
+        Ok(names) if names.is_empty() => printfln!("No vDSO calls found.")?,
+        Ok(names) => {
+            let names = names
+                .iter()
+                .map(|s| s.to_string_lossy())
+                .collect::<Vec<_>>()
+                .join(", ");
+            printfln!("List of vDSO calls: {names}.")?;
+        }
+        Err(error) => printfln!("List of vDSO calls: ? (error: {error}")?,
+    }
+
     // Print information on open file limits.
     let (nofile_soft, nofile_hard) = getrlimit(Resource::RLIMIT_NOFILE).unwrap_or((0, 0));
     printf!("Open file limits: {nofile_soft} soft, {nofile_hard} hard, ")?;
@@ -466,14 +483,39 @@ pub fn syd_info(verbose: bool) -> Result<(), Errno> {
 
     // Print detected host Linux kernel version and related features.
     printf!("Host Linux: {}.{} with", KERNEL_VERSION.0, KERNEL_VERSION.1)?;
+    printf!(" mmap_min_addr={}", *MMAP_MIN_ADDR)?;
     printf!(
-        " {}pidfd_thread",
+        ", {}at_execve_check",
+        if *HAVE_AT_EXECVE_CHECK { "+" } else { "-" }
+    )?;
+    printf!(
+        ", {}madv_guard_install",
+        if *HAVE_MADV_GUARD { "+" } else { "-" }
+    )?;
+    printf!(
+        ", {}namespaced_pid_max",
+        if *HAVE_NAMESPACED_PID_MAX { "+" } else { "-" }
+    )?;
+    printf!(
+        ", {}pidfd_thread",
         if *HAVE_PIDFD_THREAD { "+" } else { "-" }
     )?;
     printf!(
         ", {}procmap_query",
         if *HAVE_PROCMAP_QUERY { "+" } else { "-" }
     )?;
+    printf!(
+        ", {}proc_pid_fd_stat_size",
+        if *HAVE_PROC_PID_FD_STAT_SIZE {
+            "+"
+        } else {
+            "-"
+        }
+    )?;
+    printf!(
+        ", {}pwritev2_rwf_noappend",
+        if *HAVE_RWF_NOAPPEND { "+" } else { "-" }
+    )?;
     printf!(
         ", {}seccomp_user_notif_fd_sync_wake_up",
         if *HAVE_SECCOMP_USER_NOTIF_FD_SYNC_WAKE_UP {
@@ -482,15 +524,10 @@ pub fn syd_info(verbose: bool) -> Result<(), Errno> {
             "-"
         }
     )?;
-    printf!(
+    printfln!(
         ", {}statx_mnt_id_unique",
         if *HAVE_STATX_MNT_ID_UNIQUE { "+" } else { "-" }
     )?;
-    printf!(
-        ", {}at_execve_check",
-        if *HAVE_AT_EXECVE_CHECK { "+" } else { "-" }
-    )?;
-    printfln!(", mmap_min_addr={}", *MMAP_MIN_ADDR)?;
 
     // SAFETY: In libc we trust.
     // Note: nix version truncates unknown bits which we don't want.
