@@ -1047,6 +1047,21 @@ impl XPath {
         &self.0
     }
 
+    /// Create an `XPath` from a byte slice.
+    ///
+    /// Use the bytes up to the first nul byte if one exists;
+    /// otherwise use the whole byte slice.
+    #[inline]
+    pub fn from_bytes_until_nul(slice: &[u8]) -> &XPath {
+        // Find first nul byte if any.
+        // Use the whole slice otherwise.
+        let nullx = memchr(0, slice).unwrap_or(slice.len());
+        let slice = &slice[..nullx];
+
+        // SAFETY: XPath has repr(transparent)
+        unsafe { std::mem::transmute(slice) }
+    }
+
     /// Create a new `XPath` from a byte slice.
     pub const fn from_bytes(slice: &[u8]) -> &XPath {
         // SAFETY: XPath has repr(transparent)
