@@ -21,7 +21,7 @@
 // SAFETY: This module has (almost) been liberated from unsafe code!
 // proc_set_at_secure_* tests call fork(2) therefore they need unsafe.
 // pagemap_query uses ioctl(2) which calls into libc therefore needs unsafe.
-// Use deny rather than forbid so we can allow those cases.
+// Use deny rather than forbid so we can allow these cases.
 #![deny(unsafe_code)]
 
 use std::{
@@ -41,17 +41,17 @@ use std::{
 
 use bitflags::bitflags;
 use btoi::{btoi, btoi_radix};
-use indexmap::IndexSet;
+use memchr::{arch::all::is_prefix, memrchr};
 use nix::{
     errno::Errno,
-    fcntl::{open, readlinkat, OFlag, ResolveFlag},
+    fcntl::{readlinkat, OFlag, ResolveFlag},
     libc::pid_t,
     sys::{
         stat::Mode,
         sysinfo::sysinfo,
         uio::{process_vm_readv, process_vm_writev, RemoteIoVec},
     },
-    unistd::{getpid, lseek64, read, write, Gid, Pid, Uid, Whence},
+    unistd::{getpid, lseek64, write, Gid, Pid, Uid, Whence},
 };
 use procfs_core::{
     net::UnixNetEntries,
@@ -60,20 +60,21 @@ use procfs_core::{
 };
 
 use crate::{
-    compat::{getdents64, statx, trim_ascii, STATX_INO},
+    compat::{getdents64, statx, STATX_INO},
     config::*,
     confine::SydMemoryMap,
     cookie::{PROCMAP_QUERY_COOKIE_ARG3, PROCMAP_QUERY_COOKIE_ARG4, PROCMAP_QUERY_COOKIE_ARG5},
     elf::ElfType,
-    err::err2no,
-    fs::{is_open_fd, parse_fd, randint, safe_open, AT_BADFD},
-    hash::{SydHashMap, SydHashSet},
+    err::{err2no, proc_error_to_errno},
+    fs::{is_open_fd, parse_fd, read_all, retry_on_eintr, safe_open, write_all, AT_BADFD},
+    hash::{SydHashMap, SydHashSet, SydIndexSet},
     parsers::{
         map_result,
         proc::{parse_max_open_files, parse_stat, parse_statm, parse_status},
         read_to_end,
     },
     path::{XPathBuf, PATH_MAX},
+    rng::randint,
     sigset::SydSigSet,
     XPath,
 };
@@ -375,6 +376,14 @@ pub const RFLAG_PROC: ResolveFlag = ResolveFlag::from_bits_retain(
         | libc::RESOLVE_NO_XDEV,
 );
 
+// Function to map proc(5) errno ENOENT to ESRCH to reduce confusion.
+fn proc_errno(err: Errno) -> Errno {
+    match err {
+        Errno::ENOENT => Errno::ESRCH,
+        err => err,
+    }
+}
+
 /// Function to safely open a proc(5) file and return an `OwnedFd`.
 pub fn safe_open_proc(path: &XPath) -> Result<OwnedFd, Errno> {
     safe_open(PROC_FILE(), path, OFLAG_PROC, RFLAG_PROC)
@@ -445,12 +454,11 @@ pub fn proc_stat(pid: Pid) -> Result<Stat, Errno> {
     let mut path = XPathBuf::from_pid(pid);
     path.push(b"stat");
 
-    let mut file = safe_open_proc(&path)
-        .map(File::from)
-        .or(Err(Errno::ESRCH))?;
+    // Safely open proc(5) file.
+    let file = safe_open_proc(&path).map_err(proc_errno)?;
 
     let mut buf = [0; 1024]; // A typical stat file is about 300 bytes.
-    map_result(parse_stat(read_to_end(&mut file, &mut buf)?))
+    map_result(parse_stat(read_to_end(file, &mut buf)?))
 }
 
 /// Return the memory stat of the given process.
@@ -458,12 +466,11 @@ pub fn proc_statm(pid: Pid) -> Result<Statm, Errno> {
     let mut path = XPathBuf::from_pid(pid);
     path.push(b"statm");
 
-    let mut file = safe_open_proc(&path)
-        .map(File::from)
-        .or(Err(Errno::ESRCH))?;
+    // Safely open proc(5) file.
+    let file = safe_open_proc(&path).map_err(proc_errno)?;
 
     let mut buf = [0; 256]; // A typical statm file is about 25 bytes.
-    map_result(parse_statm(read_to_end(&mut file, &mut buf)?))
+    map_result(parse_statm(read_to_end(file, &mut buf)?))
 }
 
 /// Return the status of the given process.
@@ -477,7 +484,7 @@ pub fn proc_status_open(pid: Pid) -> Result<OwnedFd, Errno> {
     let mut path = XPathBuf::from_pid(pid);
     path.push(b"status");
 
-    safe_open_proc(&path)
+    safe_open_proc(&path).map_err(proc_errno)
 }
 
 /// Return the status of the given process from the given Fd.
@@ -530,31 +537,42 @@ pub fn proc_stack_pointer(pid: Pid) -> Result<u64, Errno> {
     let mut path = XPathBuf::from_pid(pid);
     path.push(b"syscall");
 
+    // Safely open proc(5) file.
     let mut file = safe_open_proc(&path)
         .map(File::from)
         .map(BufReader::new)
-        .or(Err(Errno::ESRCH))?;
+        .map_err(proc_errno)?;
 
     let mut data = Vec::with_capacity(128);
     file.read_to_end(&mut data).or(Err(Errno::EIO))?;
 
     // Check if the contents equal "running".
-    if data.starts_with(b"running") {
+    if is_prefix(&data, b"running") {
         return Err(Errno::EBUSY);
     }
 
-    // Split the contents by whitespace.
-    let parts: Vec<&[u8]> = data.split(|&b| b == b' ').collect();
-    if parts.len() >= 2 {
-        // Convert the rsp value from hex to u64 using btoi.
-        #[allow(clippy::arithmetic_side_effects)]
-        let rsp = parts[parts.len() - 2];
-        if let Some(rsp) = rsp.strip_prefix(b"0x") {
-            return btoi_radix::<u64>(rsp, 16).or(Err(Errno::EINVAL));
-        }
-    }
+    // Find the last space; if none, we don't have >= 2 parts.
+    let last_sp = match memrchr(b' ', &data) {
+        Some(i) => i,
+        None => return Err(Errno::ENOENT),
+    };
 
-    Err(Errno::ENOENT)
+    // Find the previous space (or start of slice) to get the second-to-last part.
+    let start = if let Some(prev) = memrchr(b' ', &data[..last_sp]) {
+        prev.checked_add(1).ok_or(Errno::EOVERFLOW)?
+    } else {
+        0
+    };
+
+    // Find the slice to the rsp.
+    let rsp = &data[start..last_sp];
+
+    // Convert the rsp value from hex to u64.
+    if let Some(hex) = rsp.strip_prefix(b"0x") {
+        btoi_radix::<u64>(hex, 16).or(Err(Errno::EINVAL))
+    } else {
+        Err(Errno::ENOENT)
+    }
 }
 
 /// Get the auxiliary vector of the given `Pid`.
@@ -562,9 +580,8 @@ pub fn proc_auxv(pid: Pid) -> Result<SydHashMap<u64, u64>, Errno> {
     let mut path = XPathBuf::from_pid(pid);
     path.push(b"auxv");
 
-    let mut file = safe_open_proc(&path)
-        .map(File::from)
-        .or(Err(Errno::ESRCH))?;
+    // Safely open proc(5) file.
+    let mut file = safe_open_proc(&path).map(File::from).map_err(proc_errno)?;
 
     let mut map = SydHashMap::default();
 
@@ -600,7 +617,7 @@ pub fn proc_cwd(pid: Pid) -> Result<XPathBuf, Errno> {
     path.push(b"cwd");
     readlinkat(PROC_FILE(), &path)
         .map(XPathBuf::from)
-        .or(Err(Errno::ESRCH))
+        .map_err(proc_errno)
 }
 
 /// Retrieves the command name (comm) of the specified process ID (PID)
@@ -609,31 +626,21 @@ pub fn proc_comm(pid: Pid) -> Result<XPathBuf, Errno> {
     let mut path = XPathBuf::from_pid(pid);
     path.push(b"comm");
 
-    let mut file = safe_open_proc(&path)
-        .or(Err(Errno::ESRCH))
-        .map(File::from)?;
+    // Safely open proc(5) file.
+    let file = safe_open_proc(&path).map_err(proc_errno)?;
 
     // Read up to 16 characters or until EOF.
     let mut comm = [0u8; 16];
-    let mut nread = 0;
-    while nread < comm.len() {
-        #[allow(clippy::arithmetic_side_effects)]
-        match file.read(&mut comm[nread..]).map_err(|e| err2no(&e)) {
-            Ok(0) => break,
-            Ok(n) => nread += n,
-            Err(Errno::EINTR) => continue,
-            Err(_) => return Err(Errno::ESRCH),
-        }
-    }
+    let mut nread = read_all(file, &mut comm)?;
 
     // Remove the trailing NUL-byte and return an `XPathBuf`.
     let idx = nread.saturating_sub(1);
     if comm[idx] == 0 {
         nread = idx;
     }
-    Ok(XPathBuf::from(OsStr::from_bytes(trim_ascii(
-        &comm[..nread],
-    ))))
+    Ok(XPathBuf::from(OsStr::from_bytes(
+        comm[..nread].trim_ascii(),
+    )))
 }
 
 /// Retrieves the command line of the specified process ID (PID)
@@ -647,23 +654,13 @@ pub fn proc_cmdline(pid: Pid) -> Result<XPathBuf, Errno> {
     let mut path = XPathBuf::from_pid(pid);
     path.push(b"cmdline");
 
-    let mut file = safe_open_proc(&path)
-        .or(Err(Errno::ESRCH))
-        .map(File::from)?;
+    // Safely open proc(5) file.
+    let file = safe_open_proc(&path).map_err(proc_errno)?;
 
     // Read up to 256 bytes.
     const LIMIT: usize = 256;
     let mut data = [0u8; LIMIT];
-    let mut nread = 0;
-    while nread < LIMIT {
-        #[allow(clippy::arithmetic_side_effects)]
-        match file.read(&mut data[nread..]).map_err(|e| err2no(&e)) {
-            Ok(0) => break,
-            Ok(n) => nread += n,
-            Err(Errno::EINTR) => continue,
-            Err(_) => return Err(Errno::ESRCH),
-        }
-    }
+    let nread = read_all(file, &mut data)?;
 
     let mut data = data.to_vec();
     // Determine if EOF was reached or if we hit the limit
@@ -707,7 +704,7 @@ pub fn proc_maps_open(pid: Pid) -> Result<OwnedFd, Errno> {
     let mut path = XPathBuf::from_pid(pid);
     path.push(b"maps");
 
-    safe_open_proc(&path)
+    safe_open_proc(&path).map_err(proc_errno)
 }
 
 /// Return the memory maps of the given process from the given Fd.
@@ -716,7 +713,8 @@ pub fn proc_maps_read(fd: OwnedFd) -> Result<Vec<SydMemoryMap>, Errno> {
 
     MemoryMaps::from_buf_read(reader)
         .map(|maps| maps.0.into_iter().map(SydMemoryMap).collect::<Vec<_>>())
-        .or(Err(Errno::ESRCH))
+        .map_err(|err| proc_error_to_errno(&err).unwrap_or(Errno::ENOSYS))
+        .map_err(proc_errno)
 }
 
 /// Return the memory maps of the given process.
@@ -724,14 +722,16 @@ pub fn proc_smaps(pid: Pid) -> Result<Vec<SydMemoryMap>, Errno> {
     let mut path = XPathBuf::from_pid(pid);
     path.push(b"smaps");
 
+    // Safely open proc(5) file.
     let reader = safe_open_proc(&path)
         .map(File::from)
         .map(BufReader::new)
-        .or(Err(Errno::ESRCH))?;
+        .map_err(proc_errno)?;
 
     MemoryMaps::from_buf_read(reader)
         .map(|maps| maps.0.into_iter().map(SydMemoryMap).collect::<Vec<_>>())
-        .or(Err(Errno::ESRCH))
+        .map_err(|err| proc_error_to_errno(&err).unwrap_or(Errno::ENOSYS))
+        .map_err(proc_errno)
 }
 
 /// Returns the memory map of an address by reading _proc_pid_maps_(5).
@@ -765,18 +765,9 @@ pub fn proc_pipemax_open() -> Result<OwnedFd, Errno> {
 pub fn proc_pipemax_read<Fd: AsFd>(fd: Fd) -> Result<libc::c_int, Errno> {
     // Read up to 24 bytes.
     let mut data = [0u8; 24];
-    let mut nread = 0;
-    while nread < data.len() {
-        #[allow(clippy::arithmetic_side_effects)]
-        match read(&fd, &mut data[nread..]) {
-            Ok(0) => break,
-            Ok(n) => nread += n,
-            Err(Errno::EINTR) => continue,
-            Err(_) => return Err(Errno::ESRCH),
-        }
-    }
+    let nread = read_all(fd, &mut data)?;
 
-    btoi::<libc::c_int>(trim_ascii(&data[..nread])).or(Err(Errno::EINVAL))
+    btoi::<libc::c_int>(data[..nread].trim_ascii()).or(Err(Errno::EINVAL))
 }
 
 /// Retrieves the value of /proc/sys/fs/file-max.
@@ -793,18 +784,9 @@ pub fn proc_fs_file_max() -> Result<u64, Errno> {
 
     // Read up to 24 bytes + 1 bytes for \n.
     let mut data = [0u8; 25];
-    let mut nread = 0;
-    while nread < data.len() {
-        #[allow(clippy::arithmetic_side_effects)]
-        match read(&fd, &mut data[nread..]) {
-            Ok(0) => break,
-            Ok(n) => nread += n,
-            Err(Errno::EINTR) => continue,
-            Err(_) => return Err(Errno::ESRCH),
-        }
-    }
+    let nread = read_all(fd, &mut data)?;
 
-    btoi::<u64>(trim_ascii(&data[..nread])).or(Err(Errno::EINVAL))
+    btoi::<u64>(data[..nread].trim_ascii()).or(Err(Errno::EINVAL))
 }
 
 /// Retrieves the value of /proc/sys/fs/nr_open.
@@ -821,22 +803,12 @@ pub fn proc_fs_nr_open() -> Result<u64, Errno> {
 
     // Read up to 24 bytes + 1 bytes for \n.
     let mut data = [0u8; 25];
-    let mut nread = 0;
-    while nread < data.len() {
-        #[allow(clippy::arithmetic_side_effects)]
-        match read(&fd, &mut data[nread..]) {
-            Ok(0) => break,
-            Ok(n) => nread += n,
-            Err(Errno::EINTR) => continue,
-            Err(_) => return Err(Errno::ESRCH),
-        }
-    }
+    let nread = read_all(fd, &mut data)?;
 
-    btoi::<u64>(trim_ascii(&data[..nread])).or(Err(Errno::EINVAL))
+    btoi::<u64>(data[..nread].trim_ascii()).or(Err(Errno::EINVAL))
 }
 
 /// Retrieves the value of /proc/sys/vm/mmap_min_addr.
-#[allow(clippy::disallowed_methods)]
 pub fn proc_mmap_min_addr() -> Result<u64, Errno> {
     // SAFETY: Informational command,
     // runs before syd::config::proc_init.
@@ -850,18 +822,25 @@ pub fn proc_mmap_min_addr() -> Result<u64, Errno> {
 
     // Read up to 24 bytes + 1 bytes for \n.
     let mut data = [0u8; 25];
-    let mut nread = 0;
-    while nread < data.len() {
-        #[allow(clippy::arithmetic_side_effects)]
-        match read(&fd, &mut data[nread..]) {
-            Ok(0) => break,
-            Ok(n) => nread += n,
-            Err(Errno::EINTR) => continue,
-            Err(_) => return Err(Errno::ESRCH),
-        }
-    }
+    let nread = read_all(fd, &mut data)?;
 
-    btoi::<u64>(trim_ascii(&data[..nread])).or(Err(Errno::EINVAL))
+    btoi::<u64>(data[..nread].trim_ascii()).or(Err(Errno::EINVAL))
+}
+
+/// Sets the value of /proc/sys/kernel/pid_max.
+pub fn proc_pid_max(max: u64) -> Result<(), Errno> {
+    // SAFETY: System configuration command,
+    // runs during syd::config::proc_init.
+    #[allow(clippy::disallowed_methods)]
+    let fd = nix::fcntl::openat(
+        AT_BADFD,
+        XPath::from_bytes(b"/proc/sys/kernel/pid_max"),
+        OFLAG_PROC | OFlag::O_WRONLY,
+        Mode::empty(),
+    )?;
+
+    let mut buf = itoa::Buffer::new();
+    write_all(fd, buf.format(max).as_bytes())
 }
 
 /// Retrieves the value of /proc/sys/kernel/randomize_va_space
@@ -879,18 +858,9 @@ pub fn proc_kernel_randomize_va_space() -> Result<u8, Errno> {
 
     // Read up to 1 byte + 1 byte for \n.
     let mut data = [0u8; 2];
-    let mut nread = 0;
-    while nread < data.len() {
-        #[allow(clippy::arithmetic_side_effects)]
-        match read(&fd, &mut data[nread..]) {
-            Ok(0) => break,
-            Ok(n) => nread += n,
-            Err(Errno::EINTR) => continue,
-            Err(_) => return Err(Errno::ESRCH),
-        }
-    }
+    let nread = read_all(fd, &mut data)?;
 
-    btoi::<u8>(trim_ascii(&data[..nread])).or(Err(Errno::EINVAL))
+    btoi::<u8>(data[..nread].trim_ascii()).or(Err(Errno::EINVAL))
 }
 
 /// Reads the tty number from /proc/[pid]/stat and figures out the corresponding /dev/tty device node path.
@@ -1154,9 +1124,9 @@ pub fn proc_executables(pid: Pid) -> Result<Vec<(XPathBuf, u64, i32, i32)>, Errn
 
     let maps = proc_maps(pid)?;
 
-    // Create an IndexSet to store unique executable paths while
+    // Create a SydIndexSet to store unique executable paths while
     // preserving insertion order.
-    let mut paths = IndexSet::new();
+    let mut paths = SydIndexSet::default();
 
     // Iterate over each memory map entry.
     for map in maps {
@@ -1164,7 +1134,7 @@ pub fn proc_executables(pid: Pid) -> Result<Vec<(XPathBuf, u64, i32, i32)>, Errn
         if let MMapPath::Path(path) = map.0.pathname {
             // Check if the map has executable permissions.
             if map.0.perms.contains(MMPermissions::EXECUTE) {
-                // Insert the path into the IndexSet.
+                // Insert the path into the SydIndexSet.
                 paths.insert((path.into(), map.0.inode, map.0.dev.0, map.0.dev.1));
             }
         }
@@ -1175,7 +1145,7 @@ pub fn proc_executables(pid: Pid) -> Result<Vec<(XPathBuf, u64, i32, i32)>, Errn
         return Err(Errno::ENOEXEC);
     }
 
-    // Convert the IndexSet to a vector and return.
+    // Convert the SydIndexSet to a vector and return.
     Ok(paths.into_iter().collect())
 }
 
@@ -1200,7 +1170,7 @@ pub fn procmap_query_executables(pid: Pid) -> Result<Vec<(XPathBuf, u64, i32, i3
 
     let mut path = [0u8; PATH_MAX];
     let mut addr = 0u64;
-    let mut paths = IndexSet::new();
+    let mut paths = SydIndexSet::default();
 
     loop {
         match procmap_query(&maps, flags, addr, Some(&mut path), None) {
@@ -1231,7 +1201,7 @@ pub fn procmap_query_executables(pid: Pid) -> Result<Vec<(XPathBuf, u64, i32, i3
         return Err(Errno::ENOEXEC);
     }
 
-    // Convert the IndexSet to a vector and return.
+    // Convert the SydIndexSet to a vector and return.
     Ok(paths.into_iter().collect())
 }
 
@@ -1338,7 +1308,7 @@ pub fn proc_environ_open(pid: Pid) -> Result<OwnedFd, Errno> {
     let mut path = XPathBuf::from_pid(pid);
     path.push(b"environ");
 
-    safe_open_proc(&path)
+    safe_open_proc(&path).map_err(proc_errno)
 }
 
 /// Parses /proc/pid/environ from given Fd and returns a HashMap.
@@ -1378,7 +1348,13 @@ pub fn proc_namespaces(pid: Pid) -> Result<Namespaces, Errno> {
     ns.push_pid(pid);
     ns.push(b"ns");
     #[allow(clippy::disallowed_methods)]
-    let dirfd = open(&ns, OFLAG_PROC | OFlag::O_DIRECTORY, Mode::empty())?;
+    let dirfd = nix::fcntl::openat(
+        AT_BADFD,
+        &ns,
+        OFLAG_PROC | OFlag::O_DIRECTORY,
+        Mode::empty(),
+    )
+    .map_err(proc_errno)?;
 
     // Read directory entries using getdents64 in a loop.
     #[allow(clippy::disallowed_types)]
@@ -1509,10 +1485,11 @@ pub fn proc_set_at_secure2(
         let mut flags = OFLAG_PROC;
         flags.remove(OFlag::O_RDONLY);
         flags.insert(OFlag::O_RDWR);
-        match safe_open(PROC_FILE(), &path, flags, RFLAG_PROC) {
-            Ok(fd) => Some(File::from(fd)),
-            Err(_) => return Err(Errno::EACCES),
-        }
+        Some(
+            safe_open(PROC_FILE(), &path, flags, RFLAG_PROC)
+                .map(File::from)
+                .map_err(proc_errno)?,
+        )
     } else {
         None
     };
@@ -1527,19 +1504,7 @@ pub fn proc_set_at_secure2(
             Some(file) => {
                 file.seek(SeekFrom::Start(base_addr as u64))
                     .map_err(|err| err2no(&err))?;
-
-                let mut nread = 0;
-                while nread < buf.len() {
-                    #[allow(clippy::arithmetic_side_effects)]
-                    match read(&file, &mut buf[nread..]) {
-                        Ok(0) => return Err(Errno::EIO),
-                        Ok(n) => nread += n,
-                        Err(Errno::EINTR) => {}
-                        Err(errno) => return Err(errno),
-                    }
-                }
-
-                Ok(nread)
+                read_all(file, buf)
             }
             None => {
                 let local_len = buf.len();
@@ -1566,12 +1531,9 @@ pub fn proc_set_at_secure2(
 
                 let mut nwrite = 0;
                 while nwrite < buf.len() {
-                    #[allow(clippy::arithmetic_side_effects)]
-                    match write(&file, &buf[nwrite..]) {
-                        Ok(0) => return Err(Errno::EIO),
-                        Ok(n) => nwrite += n,
-                        Err(Errno::EINTR) => {}
-                        Err(errno) => return Err(errno),
+                    match retry_on_eintr(|| write(&file, &buf[nwrite..]))? {
+                        0 => break,
+                        n => nwrite = nwrite.checked_add(n).ok_or(Errno::EOVERFLOW)?,
                     }
                 }
 
@@ -1833,12 +1795,11 @@ pub fn proc_max_open_files(pid: Pid) -> Result<LimitValue, Errno> {
     let mut path = XPathBuf::from_pid(pid);
     path.push(b"limits");
 
-    let mut file = safe_open_proc(&path)
-        .map(File::from)
-        .or(Err(Errno::ESRCH))?;
+    // Safely open proc(5) file.
+    let file = safe_open_proc(&path)?;
 
     let mut buf = [0; 1792]; // A typical limits file is about 1500 bytes.
-    map_result(parse_max_open_files(read_to_end(&mut file, &mut buf)?))
+    map_result(parse_max_open_files(read_to_end(file, &mut buf)?))
 }
 
 /// Pick a random fd number and check via KCMP_FILE whether it’s open in `pid`.
@@ -2009,19 +1970,10 @@ pub fn proc_kernel_taint() -> Result<KernelTaintFlags, Errno> {
 
     // Read up to 24 bytes + 1 byte for \n.
     let mut data = [0u8; 25];
-    let mut nread = 0usize;
-    while nread < data.len() {
-        #[allow(clippy::arithmetic_side_effects)]
-        match read(&fd, &mut data[nread..]) {
-            Ok(0) => break,
-            Ok(n) => nread += n,
-            Err(Errno::EINTR) => continue,
-            Err(_) => return Err(Errno::ESRCH),
-        }
-    }
+    let nread = read_all(fd, &mut data)?;
 
     // Parse decimal -> u64, then map to flags (retain unknown higher bits).
-    let val = btoi::<u64>(trim_ascii(&data[..nread])).or(Err(Errno::EINVAL))?;
+    let val = btoi::<u64>(data[..nread].trim_ascii()).or(Err(Errno::EINVAL))?;
     Ok(KernelTaintFlags::from_bits_retain(val))
 }
 
@@ -2424,7 +2376,7 @@ mod tests {
 
         // Ensure uniqueness and preserved order.
         let list = proc_executables(Pid::this()).unwrap();
-        let mut seen = IndexSet::new();
+        let mut seen = SydIndexSet::default();
         for (path, _, _, _) in &list {
             assert!(seen.insert(path.clone()), "duplicate path {path}!");
         }
