@@ -87,6 +87,17 @@ pub fn ensure_sealed() -> Result<(), Errno> {
             command.arg0(arg0);
         }
         command.args(args);
+
+        // Force RUST_BACKTRACE environment variable to 0 for Syd.
+        // Passthrough the original value to the sandbox process.
+        // See syd.rs for the other branch.
+        // Rest is handled in unshare/child.rs.
+        match std::env::var_os("RUST_BACKTRACE") {
+            Some(val) => command.env("SYD_RUST_BACKTRACE", val),
+            None => command.env_remove("SYD_RUST_BACKTRACE"),
+        };
+        command.env("RUST_BACKTRACE", "0");
+
         Err(err2no(&command.exec()))
     }
 }
