@@ -53,7 +53,7 @@
 use std::{
     env,
     env::VarError,
-    ffi::{CString, OsStr, OsString},
+    ffi::{OsStr, OsString},
     fs::{File, OpenOptions},
     io::{BufWriter, Write},
     net::{IpAddr, SocketAddrV4, SocketAddrV6},
@@ -81,7 +81,6 @@ use nix::{
     },
     time::{clock_gettime, ClockId},
     unistd::{fork, getgid, getpid, getuid, isatty, sethostname, ForkResult},
-    NixPath,
 };
 use sendfd::SendWithFd;
 use syd::{
@@ -102,7 +101,7 @@ use syd::{
     printf,
     sandbox::Sandbox,
     seal::ensure_sealed,
-    syd_code_name, syd_info,
+    set_sigpipe_dfl, syd_code_name, syd_info,
     syslog::LogLevel,
     unshare::{GidMap, UidMap},
     warn, IgnoreSignalOpts,
@@ -139,22 +138,27 @@ syd::main! {
             if let Some(Some(arg)) = raw.peek().map(|arg| arg.to_str()) {
                 match arg {
                     "-h" | "--help" => {
+                        set_sigpipe_dfl()?;
                         help();
                         return Ok(ExitCode::SUCCESS);
                     }
                     "-C" | "--check" => {
+                        set_sigpipe_dfl()?;
                         syd_info(true)?;
                         return Ok(ExitCode::SUCCESS);
                     }
                     "-V" | "--version" => {
+                        set_sigpipe_dfl()?;
                         syd_info(false)?;
                         return Ok(ExitCode::SUCCESS);
                     }
                     "--el" => {
+                        set_sigpipe_dfl()?;
                         printf!("{SYD_EL}")?;
                         return Ok(ExitCode::SUCCESS);
                     }
                     "--sh" => {
+                        set_sigpipe_dfl()?;
                         printf!("{ESYD_SH}")?;
                         return Ok(ExitCode::SUCCESS);
                     }
@@ -211,6 +215,13 @@ syd::main! {
             }
         }
     } else {
+        // See seal.rs for the other branch.
+        // Rest is handled in unshare/child.rs
+        match env::var_os("RUST_BACKTRACE") {
+            Some(val) => env::set_var("SYD_RUST_BACKTRACE", val),
+            None => env::remove_var("SYD_RUST_BACKTRACE"),
+        };
+        env::set_var("RUST_BACKTRACE", "0");
         env::set_var(ENV_RAND, get_at_random_hex(false));
         env::var(ENV_RAND).unwrap()
     };
@@ -333,32 +344,38 @@ syd::main! {
              * Basic options
              */
             Short('h') | Long("help") => {
+                set_sigpipe_dfl()?;
                 help();
                 return Ok(ExitCode::SUCCESS);
             }
             Short('C') | Long("check") => {
+                set_sigpipe_dfl()?;
                 syd_info(true)?;
                 return Ok(ExitCode::SUCCESS);
             }
             // syd -V is called often by paludis.
             // We want to keep its output short and parseable.
             Short('V') | Long("version") => {
+                set_sigpipe_dfl()?;
                 syd_info(false)?;
                 return Ok(ExitCode::SUCCESS);
             }
             Short('v') | Long("verbose") => sandbox.verbose = true,
             Long("el") => {
-                println!("{SYD_EL}");
+                set_sigpipe_dfl()?;
+                printf!("{SYD_EL}")?;
                 return Ok(ExitCode::SUCCESS);
             }
             Long("sh") => {
-                println!("{ESYD_SH}");
+                set_sigpipe_dfl()?;
+                printf!("{ESYD_SH}")?;
                 return Ok(ExitCode::SUCCESS);
             }
             Long("api") => {
+                set_sigpipe_dfl()?;
                 #[allow(clippy::disallowed_methods)]
                 let api = serde_json::to_string_pretty(&*syd::api::API_SPEC).expect("JSON");
-                println!("{api}");
+                printf!("{api}")?;
                 return Ok(ExitCode::SUCCESS);
             }
             Short('q') => {} // Ignore, must be first!
@@ -608,7 +625,7 @@ syd::main! {
             let errno = err2no(&error);
             error!("ctx": "spawn_pty",
                 "msg": format!("syd-pty spawn error: {error}"),
-                "tip": "put syd-pty(1) in PATH or set CARGO_BIN_EXE_syd-pty",
+                "tip": "put syd-pty(1) in PATH, set CARGO_BIN_EXE_syd-pty, or set sandbox/pty:off",
                 "err": errno as i32);
             return Err(error.into());
         }
@@ -695,7 +712,7 @@ syd::main! {
             let errno = err2no(&error);
             error!("ctx": "spawn_tor",
                 "msg": format!("syd-tor spawn error: {error}"),
-                "tip": "put syd-tor(1) in PATH or set CARGO_BIN_EXE_syd-tor",
+                "tip": "put syd-tor(1) in PATH, set CARGO_BIN_EXE_syd-tor, or set sandbox/proxy:off",
                 "err": errno as i32);
             return Err(error.into());
         }
@@ -928,15 +945,26 @@ syd::main! {
     }
 
     // Set host and domain name for uts namespace.
-    // Ignore errors.
+    // Log on errors and continue.
     if sandbox.flags.unshare_uts() {
-        // SAFETY: Domain name must not contain a nul-byte.
-        #[allow(clippy::disallowed_methods)]
-        let domainname = CString::new(sandbox.domainname.clone()).unwrap();
+        let domainname = sandbox.domainname.as_c_str();
+        let domainsize = domainname.to_bytes().len(); // without NUL.
+        let domainname = domainname.as_ptr().cast();
 
-        // SAFETY: There's no setdomainname wrapper in nix.
-        let _ = unsafe { setdomainname(domainname.as_ptr() as *const _, domainname.len()) };
-        let _ = sethostname(&sandbox.hostname);
+        // SAFETY: No setdomainname(2) wrapper in nix yet.
+        if let Err(errno) = Errno::result(unsafe { setdomainname(domainname, domainsize) }) {
+            error!("ctx": "uts", "op": "setdomainname",
+                "msg": format!("set NIS/YP domain name failed: {errno}"),
+                "err": errno as i32);
+        }
+
+        let hostname = sandbox.hostname.as_c_str();
+        let hostname = OsStr::from_bytes(hostname.to_bytes());
+        if let Err(errno) = sethostname(hostname) {
+            error!("ctx": "uts", "op": "sethostname",
+                "msg": format!("set host name failed: {errno}"),
+                "err": errno as i32);
+        }
     }
 
     if sandbox.flags.unshare_time() {
