@@ -29,8 +29,6 @@
 use btoi::btoi;
 use memchr::arch::all::is_equal;
 
-use crate::compat::trim_ascii;
-
 /// Kernel log levels (KERN_*).
 #[derive(Copy, Clone, Debug)]
 pub enum LogLevel {
@@ -108,7 +106,7 @@ impl From<i64> for LogLevel {
 /// Translate a string log level from environment to `LogLevel`
 /// or fallback to the given default `LogLevel`.
 pub fn parse_loglevel(level: &[u8], default: LogLevel) -> LogLevel {
-    let level = trim_ascii(level);
+    let level = level.trim_ascii();
     if level.is_empty() {
         default
     } else if let Ok(level) = btoi::<i64>(level) {
@@ -778,11 +776,9 @@ pub fn current_loglevel() -> u8 {
 #[cfg(not(feature = "log"))]
 #[inline(always)]
 pub fn current_loglevel() -> u8 {
-    use std::os::unix::ffi::OsStrExt;
+    use std::{os::unix::ffi::OsStrExt, sync::LazyLock};
 
-    use once_cell::sync::Lazy;
-
-    static LOGLEVEL: Lazy<u8> = Lazy::new(|| {
+    static LOGLEVEL: LazyLock<u8> = LazyLock::new(|| {
         std::env::var_os(crate::config::ENV_LOG)
             .map(|val| parse_loglevel(val.as_os_str().as_bytes(), LogLevel::Warn))
             .unwrap_or(LogLevel::Warn)
