@@ -20,6 +20,7 @@ use std::{
     os::fd::RawFd,
     path::{Path, PathBuf},
     process::{Child, Command, ExitStatus, Output, Stdio},
+    sync::LazyLock,
     time::Duration,
 };
 
@@ -30,7 +31,6 @@ use nix::{
         utsname::uname,
     },
 };
-use once_cell::sync::Lazy;
 use syd::config::*;
 
 #[derive(Debug)]
@@ -196,12 +196,19 @@ macro_rules! assert_status_bad_message {
 }
 
 #[macro_export]
-macro_rules! assert_status_denied {
+macro_rules! assert_status_access_denied {
     ($status:expr) => {
         assert_status_code!($status, libc::EACCES);
     };
 }
 
+#[macro_export]
+macro_rules! assert_status_permission_denied {
+    ($status:expr) => {
+        assert_status_code!($status, libc::EPERM);
+    };
+}
+
 #[macro_export]
 macro_rules! assert_status_hidden {
     ($status:expr) => {
@@ -271,6 +278,18 @@ macro_rules! assert_status_not_supported {
     };
 }
 
+#[macro_export]
+macro_rules! assert_status_operation_not_supported {
+    ($status:expr) => {
+        let code = $status.code().unwrap_or(127);
+        assert!(
+            matches!(code, libc::EOPNOTSUPP),
+            "code:{code} status:{:?}",
+            $status
+        );
+    };
+}
+
 #[macro_export]
 macro_rules! skip_unless_available {
     ($($program:expr),* $(,)?) => {{
@@ -300,6 +319,22 @@ macro_rules! skip_if_32bin_64host {
     }};
 }
 
+#[macro_export]
+macro_rules! skip_if_mips {
+    () => {{
+        if cfg!(any(
+            target_arch = "mips",
+            target_arch = "mips32r6",
+            target_arch = "mips64",
+            target_arch = "mips64r6",
+        )) {
+            eprintln!("Test does not work on MIPS, skipping!");
+            std::env::set_var("SYD_TEST_SOFT_FAIL", "1");
+            return Ok(());
+        }
+    }};
+}
+
 #[macro_export]
 macro_rules! skip_if_root {
     () => {{
@@ -313,6 +348,38 @@ macro_rules! skip_if_root {
     }};
 }
 
+#[macro_export]
+macro_rules! skip_unless_iproute2 {
+    () => {{
+        if !check_iproute2() {
+            eprintln!("ip is not iproute2, cannot run test, skipping!");
+            std::env::set_var("SYD_TEST_SOFT_FAIL", "1");
+            return Ok(());
+        }
+    }};
+}
+
+#[macro_export]
+macro_rules! skip_unless_linux {
+    ($need_major:expr, $need_minor:expr) => {{
+        let (maj, min) = *syd::config::KERNEL_VERSION;
+        let too_old = maj < $need_major || (maj == $need_major && min < $need_minor);
+        if too_old {
+            eprintln!(
+                "Test requires Linux kernel >= {}.{} (current: {maj}.{min}), skipping!",
+                $need_major, $need_minor,
+            );
+            std::env::set_var("SYD_TEST_SOFT_FAIL", "1");
+            return Ok(());
+        } else {
+            eprintln!(
+                "Linux kernel {maj}.{min} satisfies >= {}.{}, proceeding!",
+                $need_major, $need_minor,
+            );
+        }
+    }};
+}
+
 #[macro_export]
 macro_rules! skip_unless_stdin_is_a_tty {
     () => {{
@@ -343,13 +410,68 @@ macro_rules! skip_unless_stdout_is_a_tty {
 macro_rules! skip_unless_kernel_crypto_is_supported {
     () => {{
         let key = syd::hash::Key::random().unwrap();
-        match syd::hash::aes_ctr_setup(&key) {
+        let key_id = match syd::hash::add_key(
+            "user",
+            "SYD-3-CRYPT-TEST",
+            key.as_ref(),
+            syd::hash::KEY_SPEC_USER_KEYRING,
+        ) {
+            Ok(key_id) => key_id,
+            Err(nix::errno::Errno::EAFNOSUPPORT | nix::errno::Errno::ENOSYS) => {
+                eprintln!("Test requires Linux keyrings(7) API, skipping!");
+                std::env::set_var("SYD_TEST_SOFT_FAIL", "1");
+                return Ok(());
+            }
+            Err(nix::errno::Errno::EPERM) => {
+                eprintln!("Are you in a container without keyrings access?");
+                eprintln!("Test requires Linux keyrings(7) API, skipping!");
+                std::env::set_var("SYD_TEST_SOFT_FAIL", "1");
+                return Ok(());
+            }
+            Err(nix::errno::Errno::EACCES) => {
+                eprintln!("Is your session keyring attached to your user keyring?");
+                eprintln!("Test requires Linux keyrings(7) API, skipping!");
+                std::env::set_var("SYD_TEST_SOFT_FAIL", "1");
+                return Ok(());
+            }
+            Err(errno) => {
+                return Err(TestError(format!(
+                    "Failed to test for Linux keyrings(7) API: {errno}"
+                )));
+            }
+        };
+        match syd::hash::aes_ctr_setup(key_id) {
             Ok(fd) => drop(fd),
             Err(nix::errno::Errno::EAFNOSUPPORT) => {
                 eprintln!("Test requires Linux Kernel Cryptography API, skipping!");
                 std::env::set_var("SYD_TEST_SOFT_FAIL", "1");
                 return Ok(());
             }
+            Err(nix::errno::Errno::EACCES) => {
+                eprintln!("Is your session keyring attached to your user keyring?");
+                eprintln!("Test requires Linux keyrings(7) API, skipping!");
+                std::env::set_var("SYD_TEST_SOFT_FAIL", "1");
+                return Ok(());
+            }
+            Err(errno) => {
+                return Err(TestError(format!(
+                    "Failed to test for Linux Kernel Cryptography API: {errno}"
+                )));
+            }
+        }
+        match syd::hash::hmac_sha256_setup(key_id) {
+            Ok(fd) => drop(fd),
+            Err(nix::errno::Errno::EAFNOSUPPORT) => {
+                eprintln!("Test requires Linux Kernel Cryptography API, skipping!");
+                std::env::set_var("SYD_TEST_SOFT_FAIL", "1");
+                return Ok(());
+            }
+            Err(nix::errno::Errno::EACCES) => {
+                eprintln!("Is your session keyring attached to your user keyring?");
+                eprintln!("Test requires Linux keyrings(7) API, skipping!");
+                std::env::set_var("SYD_TEST_SOFT_FAIL", "1");
+                return Ok(());
+            }
             Err(errno) => {
                 return Err(TestError(format!(
                     "Failed to test for Linux Kernel Cryptography API: {errno}"
@@ -392,17 +514,61 @@ macro_rules! skip_unless_xattrs_are_supported {
 }
 
 #[macro_export]
-macro_rules! skip_unless_unshare {
+macro_rules! skip_unless_unix_diag_is_supported {
     () => {{
-        match check_unshare() {
-            Some(false) => {
-                eprintln!("Test requires Linux namespaces, skipping!");
+        match syd::confine::check_unix_diag() {
+            Ok(true) => {
+                eprintln!("Unix socket diagnostics are supported, proceeding with test...");
+            }
+            Ok(false) => {
+                eprintln!("Test requires UNIX socket diagnostics support, skipping!");
                 std::env::set_var("SYD_TEST_SOFT_FAIL", "1");
                 return Ok(());
             }
-            None => {
+            Err(errno) => {
+                return Err(TestError(format!(
+                    "Failed to test for UNIX socket diagnostics: {errno}"
+                )));
+            }
+        }
+    }};
+}
+
+#[macro_export]
+macro_rules! skip_unless_unshare {
+    ( $( $ns:expr ),* $(,)? ) => {{
+        use nix::sched::CloneFlags;
+
+        // Build combined CloneFlags from the provided string literals.
+        let mut flags = CloneFlags::empty();
+        $(
+            flags |= match $ns {
+                "mount"   => CloneFlags::CLONE_NEWNS,
+                "uts"     => CloneFlags::CLONE_NEWUTS,
+                "ipc"     => CloneFlags::CLONE_NEWIPC,
+                "user"    => CloneFlags::CLONE_NEWUSER,
+                "pid"     => CloneFlags::CLONE_NEWPID,
+                "net"     => CloneFlags::CLONE_NEWNET,
+                "cgroup"  => CloneFlags::CLONE_NEWCGROUP,
+                "time"    => syd::confine::CLONE_NEWTIME,
+                "all"     => CloneFlags::CLONE_NEWNS |  CloneFlags::CLONE_NEWUTS | CloneFlags::CLONE_NEWIPC | CloneFlags::CLONE_NEWUSER | CloneFlags::CLONE_NEWPID | CloneFlags::CLONE_NEWNET | CloneFlags::CLONE_NEWCGROUP | syd::confine::CLONE_NEWTIME,
+                other => panic!("Unknown namespace: {other}"),
+            };
+        )*
+
+        if flags.is_empty() {
+            panic!("No namespace given!");
+        }
+
+        match syd::confine::ns_enabled(flags) {
+            Ok(false) => {
+                eprintln!("Test requires Linux namespaces ({:?}), skipping!", [$($ns),*]);
+                std::env::set_var("SYD_TEST_SOFT_FAIL", "1");
+                return Ok(());
+            }
+            Err(errno) => {
                 return Err(TestError(
-                    "Failed to test for Linux namespaces!".to_string(),
+                    format!("Failed to test for Linux namespaces: {errno}"),
                 ));
             }
             _ => {}
@@ -503,6 +669,17 @@ macro_rules! skip_unless_landlock_abi_supported {
     }};
 }
 
+#[macro_export]
+macro_rules! skip_if_cross_memory_attach_is_not_enabled {
+    () => {
+        if !*syd::config::HAVE_CROSS_MEMORY_ATTACH {
+            eprintln!("skipping: kernel not configured with CONFIG_CROSS_MEMORY_ATTACH");
+            std::env::set_var("SYD_TEST_SOFT_FAIL", "1");
+            return Ok(());
+        }
+    };
+}
+
 #[macro_export]
 macro_rules! skip_if_strace {
     () => {{
@@ -514,6 +691,17 @@ macro_rules! skip_if_strace {
     }};
 }
 
+#[macro_export]
+macro_rules! skip_unless_strace_can_inject {
+    () => {{
+        if !check_strace_inject() {
+            eprintln!("strace can't inject syscalls, cannot run test, skipping!");
+            std::env::set_var("SYD_TEST_SOFT_FAIL", "1");
+            return Ok(());
+        }
+    }};
+}
+
 #[macro_export]
 macro_rules! skip_unless_pty {
     () => {{
@@ -553,117 +741,138 @@ macro_rules! skip_unless_at_execve_check_is_supported {
 
 /// Holds a `String` to run `syd`.
 /// Honours CARGO_BIN_EXE_syd environment variable.
-pub static SYD: Lazy<String> =
-    Lazy::new(|| env::var("CARGO_BIN_EXE_syd").unwrap_or("syd".to_string()));
+pub static SYD: LazyLock<String> =
+    LazyLock::new(|| env::var("CARGO_BIN_EXE_syd").unwrap_or("syd".to_string()));
 
 /// Holds a `String` to run `syd-aux`.
 /// Honours CARGO_BIN_EXE_syd-aux environment variable.
-pub static SYD_AUX: Lazy<String> =
-    Lazy::new(|| env::var("CARGO_BIN_EXE_syd-aux").unwrap_or("syd-aux".to_string()));
+pub static SYD_AUX: LazyLock<String> =
+    LazyLock::new(|| env::var("CARGO_BIN_EXE_syd-aux").unwrap_or("syd-aux".to_string()));
 
 /// Holds a `String` to run `syd-bit`.
 /// Honours CARGO_BIN_EXE_syd-aux environment variable.
-pub static SYD_BIT: Lazy<String> =
-    Lazy::new(|| env::var("CARGO_BIN_EXE_syd-bit").unwrap_or("syd-bit".to_string()));
+pub static SYD_BIT: LazyLock<String> =
+    LazyLock::new(|| env::var("CARGO_BIN_EXE_syd-bit").unwrap_or("syd-bit".to_string()));
 
 /// Holds a `String` to run `syd-cap`.
 /// Honours CARGO_BIN_EXE_syd-cap environment variable.
-pub static SYD_CAP: Lazy<String> =
-    Lazy::new(|| env::var("CARGO_BIN_EXE_syd-cap").unwrap_or("syd-cap".to_string()));
+pub static SYD_CAP: LazyLock<String> =
+    LazyLock::new(|| env::var("CARGO_BIN_EXE_syd-cap").unwrap_or("syd-cap".to_string()));
 
 /// Holds a `String` to run `syd-dns`.
 /// Honours CARGO_BIN_EXE_syd-dns environment variable.
-pub static SYD_DNS: Lazy<String> =
-    Lazy::new(|| env::var("CARGO_BIN_EXE_syd-dns").unwrap_or("syd-dns".to_string()));
+pub static SYD_DNS: LazyLock<String> =
+    LazyLock::new(|| env::var("CARGO_BIN_EXE_syd-dns").unwrap_or("syd-dns".to_string()));
 
 /// Holds a `String` to run `syd-env`.
 /// Honours CARGO_BIN_EXE_syd-env environment variable.
-pub static SYD_ENV: Lazy<String> =
-    Lazy::new(|| env::var("CARGO_BIN_EXE_syd-env").unwrap_or("syd-env".to_string()));
+pub static SYD_ENV: LazyLock<String> =
+    LazyLock::new(|| env::var("CARGO_BIN_EXE_syd-env").unwrap_or("syd-env".to_string()));
 
 /// Holds a `String` to run `syd-exec`.
 /// Honours CARGO_BIN_EXE_syd-exec environment variable.
-pub static SYD_EXEC: Lazy<String> =
-    Lazy::new(|| env::var("CARGO_BIN_EXE_syd-exec").unwrap_or("syd-exec".to_string()));
+pub static SYD_EXEC: LazyLock<String> =
+    LazyLock::new(|| env::var("CARGO_BIN_EXE_syd-exec").unwrap_or("syd-exec".to_string()));
 
 /// Holds a `String` to run `syd-aes`.
 /// Honours CARGO_BIN_EXE_syd-aes environment variable.
-pub static SYD_AES: Lazy<String> =
-    Lazy::new(|| env::var("CARGO_BIN_EXE_syd-aes").unwrap_or("syd-aes".to_string()));
+pub static SYD_AES: LazyLock<String> =
+    LazyLock::new(|| env::var("CARGO_BIN_EXE_syd-aes").unwrap_or("syd-aes".to_string()));
 
 /// Holds a `String` to run `syd-key`.
 /// Honours CARGO_BIN_EXE_syd-key environment variable.
-pub static SYD_KEY: Lazy<String> =
-    Lazy::new(|| env::var("CARGO_BIN_EXE_syd-key").unwrap_or("syd-key".to_string()));
+pub static _SYD_KEY: LazyLock<String> =
+    LazyLock::new(|| env::var("CARGO_BIN_EXE_syd-key").unwrap_or("syd-key".to_string()));
 
 /// Holds a `String` to run `syd-elf`.
 /// Honours CARGO_BIN_EXE_syd-elf environment variable.
-pub static SYD_ELF: Lazy<String> =
-    Lazy::new(|| env::var("CARGO_BIN_EXE_syd-elf").unwrap_or("syd-elf".to_string()));
+pub static SYD_ELF: LazyLock<String> =
+    LazyLock::new(|| env::var("CARGO_BIN_EXE_syd-elf").unwrap_or("syd-elf".to_string()));
 
 /// Holds a `String` to run `syd-cpu`.
 /// Honours CARGO_BIN_EXE_syd-cpu environment variable.
-pub static SYD_CPU: Lazy<String> =
-    Lazy::new(|| env::var("CARGO_BIN_EXE_syd-cpu").unwrap_or("syd-cpu".to_string()));
+pub static SYD_CPU: LazyLock<String> =
+    LazyLock::new(|| env::var("CARGO_BIN_EXE_syd-cpu").unwrap_or("syd-cpu".to_string()));
 
 /// Holds a `String` to run `syd-hex`.
 /// Honours CARGO_BIN_EXE_syd-hex environment variable.
-pub static SYD_HEX: Lazy<String> =
-    Lazy::new(|| env::var("CARGO_BIN_EXE_syd-hex").unwrap_or("syd-hex".to_string()));
+pub static SYD_HEX: LazyLock<String> =
+    LazyLock::new(|| env::var("CARGO_BIN_EXE_syd-hex").unwrap_or("syd-hex".to_string()));
 
 /// Holds a `String` to run `syd-info`.
 /// Honours CARGO_BIN_EXE_syd-info environment variable.
-pub static SYD_INFO: Lazy<String> =
-    Lazy::new(|| env::var("CARGO_BIN_EXE_syd-info").unwrap_or("syd-info".to_string()));
+pub static SYD_INFO: LazyLock<String> =
+    LazyLock::new(|| env::var("CARGO_BIN_EXE_syd-info").unwrap_or("syd-info".to_string()));
 
 /// Holds a `String` to run `syd-lock`.
 /// Honours CARGO_BIN_EXE_syd-lock environment variable.
-pub static SYD_LOCK: Lazy<String> =
-    Lazy::new(|| env::var("CARGO_BIN_EXE_syd-lock").unwrap_or("syd-lock".to_string()));
+pub static SYD_LOCK: LazyLock<String> =
+    LazyLock::new(|| env::var("CARGO_BIN_EXE_syd-lock").unwrap_or("syd-lock".to_string()));
+
+/// Holds a `String` to run `syd-mdwe`.
+/// Honours CARGO_BIN_EXE_syd-mdwe environment variable.
+pub static SYD_MDWE: LazyLock<String> =
+    LazyLock::new(|| env::var("CARGO_BIN_EXE_syd-mdwe").unwrap_or("syd-mdwe".to_string()));
+
+/// Holds a `String` to run `syd-ofd`.
+/// Honours CARGO_BIN_EXE_syd-ofd environment variable.
+pub static SYD_OFD: LazyLock<String> =
+    LazyLock::new(|| env::var("CARGO_BIN_EXE_syd-ofd").unwrap_or("syd-ofd".to_string()));
+
+/// Holds a `String` to run `syd-pause`.
+/// Honours CARGO_BIN_EXE_syd-pause environment variable.
+pub static SYD_PAUSE: LazyLock<String> =
+    LazyLock::new(|| env::var("CARGO_BIN_EXE_syd-pause").unwrap_or("syd-pause".to_string()));
 
 /// Holds a `String` to run `syd-pds`.
 /// Honours CARGO_BIN_EXE_syd-pds environment variable.
-pub static SYD_PDS: Lazy<String> =
-    Lazy::new(|| env::var("CARGO_BIN_EXE_syd-pds").unwrap_or("syd-pds".to_string()));
+pub static SYD_PDS: LazyLock<String> =
+    LazyLock::new(|| env::var("CARGO_BIN_EXE_syd-pds").unwrap_or("syd-pds".to_string()));
 
 /// Holds a `String` to run `syd-size`.
 /// Honours CARGO_BIN_EXE_syd-size environment variable.
-pub static SYD_SIZE: Lazy<String> =
-    Lazy::new(|| env::var("CARGO_BIN_EXE_syd-size").unwrap_or("syd-size".to_string()));
+pub static SYD_SIZE: LazyLock<String> =
+    LazyLock::new(|| env::var("CARGO_BIN_EXE_syd-size").unwrap_or("syd-size".to_string()));
 
 /// Holds a `String` to run `syd-test-do`.
 /// Honours CARGO_BIN_EXE_syd-test-do environment variable.
 /// This path is a canonicalized for ease of use in sandboxing tests.
-pub static SYD_DO: Lazy<String> = Lazy::new(|| match env::var("CARGO_BIN_EXE_syd-test-do") {
-    Ok(var) => Path::new(&var)
-        .canonicalize()
-        .expect("CARGO_BIN_EXE_syd-test-do")
-        .to_string_lossy()
-        .into_owned(),
-    Err(_) => which(if env::var("SYD_TEST_32").is_ok() {
-        "syd-test-do32"
-    } else {
-        "syd-test-do"
-    })
-    .expect("syd-test-do"),
-});
+pub static SYD_DO: LazyLock<String> =
+    LazyLock::new(|| match env::var("CARGO_BIN_EXE_syd-test-do") {
+        Ok(var) => Path::new(&var)
+            .canonicalize()
+            .expect("CARGO_BIN_EXE_syd-test-do")
+            .to_string_lossy()
+            .into_owned(),
+        Err(_) => which(if env::var("SYD_TEST_32").is_ok() {
+            "syd-test-do32"
+        } else {
+            "syd-test-do"
+        })
+        .expect("syd-test-do"),
+    });
+
+/// Holds a `String` to run `syd-uts`.
+/// Honours CARGO_BIN_EXE_syd-uts environment variable.
+pub static SYD_UTS: LazyLock<String> =
+    LazyLock::new(|| env::var("CARGO_BIN_EXE_syd-uts").unwrap_or("syd-uts".to_string()));
 
 /// Holds a `String` to run `syd-x`.
 /// Honours CARGO_BIN_EXE_syd-x environment variable.
-pub static SYD_X: Lazy<String> =
-    Lazy::new(|| env::var("CARGO_BIN_EXE_syd-x").unwrap_or("syd-x".to_string()));
+pub static SYD_X: LazyLock<String> =
+    LazyLock::new(|| env::var("CARGO_BIN_EXE_syd-x").unwrap_or("syd-x".to_string()));
 
 #[cfg(feature = "oci")]
 /// Holds a `String` to run `syd-oci`.
 /// Honours CARGO_BIN_EXE_syd-oci environment variable.
-pub static SYD_OCI: Lazy<String> =
-    Lazy::new(|| env::var("CARGO_BIN_EXE_syd-oci").unwrap_or("syd-oci".to_string()));
+pub static SYD_OCI: LazyLock<String> =
+    LazyLock::new(|| env::var("CARGO_BIN_EXE_syd-oci").unwrap_or("syd-oci".to_string()));
 
 /// A boolean which specifies if we're running under SourceHut CI.
-pub static CI_BUILD: Lazy<bool> = Lazy::new(|| env::var("JOB_ID").ok().is_some());
+pub static CI_BUILD: LazyLock<bool> = LazyLock::new(|| env::var("JOB_ID").ok().is_some());
 
 /// A boolean which specifies if we're running under GitLab CI.
-pub static GL_BUILD: Lazy<bool> = Lazy::new(|| env::var("CI_PROJECT_ID").ok().is_some());
+pub static GL_BUILD: LazyLock<bool> = LazyLock::new(|| env::var("CI_PROJECT_ID").ok().is_some());
 
 pub struct Syd(Command);
 
@@ -697,6 +906,11 @@ impl Syd {
         self
     }
 
+    pub fn current_dir<S: ToString>(&mut self, arg: S) -> &mut Self {
+        self.0.current_dir(arg.to_string());
+        self
+    }
+
     pub fn log(&mut self, value: &str) -> &mut Self {
         self.0
             .env(ENV_LOG, env::var(ENV_LOG).unwrap_or(value.to_string()));
@@ -806,11 +1020,13 @@ impl Syd {
 /// Returns a `Command` to run `syd`.
 /// Honours CARGO_BIN_EXE_syd environment variable.
 pub fn syd() -> Syd {
-    static USE_PERF: Lazy<bool> = Lazy::new(|| env::var_os("SYD_TEST_PERF").is_some());
-    static USE_TRACE: Lazy<bool> = Lazy::new(|| env::var_os("SYD_TEST_TRACE").is_some());
-    static USE_STRACE: Lazy<bool> = Lazy::new(|| env::var_os("SYD_TEST_STRACE").is_some());
-    static USE_VALGRIND: Lazy<bool> = Lazy::new(|| env::var_os("SYD_TEST_VALGRIND").is_some());
-    static USE_HELGRIND: Lazy<bool> = Lazy::new(|| env::var_os("SYD_TEST_HELGRIND").is_some());
+    static USE_PERF: LazyLock<bool> = LazyLock::new(|| env::var_os("SYD_TEST_PERF").is_some());
+    static USE_TRACE: LazyLock<bool> = LazyLock::new(|| env::var_os("SYD_TEST_TRACE").is_some());
+    static USE_STRACE: LazyLock<bool> = LazyLock::new(|| env::var_os("SYD_TEST_STRACE").is_some());
+    static USE_VALGRIND: LazyLock<bool> =
+        LazyLock::new(|| env::var_os("SYD_TEST_VALGRIND").is_some());
+    static USE_HELGRIND: LazyLock<bool> =
+        LazyLock::new(|| env::var_os("SYD_TEST_HELGRIND").is_some());
     let mut cmd = Syd::new("timeout");
     if check_timeout_foreground() {
         cmd.arg("--foreground");
@@ -818,7 +1034,7 @@ pub fn syd() -> Syd {
         cmd.arg("--verbose");
     }
     cmd.arg("-sKILL");
-    cmd.arg(env::var("SYD_TEST_TIMEOUT").unwrap_or("10m".to_string()));
+    cmd.arg(env::var("SYD_TEST_TIMEOUT").unwrap_or("15m".to_string()));
     if *USE_PERF {
         cmd.arg("perf");
         cmd.arg("record");
@@ -828,7 +1044,11 @@ pub fn syd() -> Syd {
         cmd.arg("--");
     } else if *USE_STRACE {
         cmd.arg("strace");
-        cmd.arg("-f");
+        cmd.arg("-yyY");
+
+        if env::var_os("SYD_TEST_STRACE_NOFORK").is_none() {
+            cmd.arg("-f");
+        }
 
         if env::var_os("SYD_TEST_STRACE_VERBOSE").is_none() {
             cmd.arg("-s256");
@@ -864,6 +1084,8 @@ pub fn syd() -> Syd {
     // Set logging level to warning.
     // Allow user to override.
     cmd.env(ENV_LOG, env::var(ENV_LOG).unwrap_or("warn".to_string()));
+    // Disallow backtraces.
+    cmd.env("RUST_BACKTRACE", "0");
     // Set log/verbose to false to avoid noisy test logs.
     cmd.m("log/verbose:0");
     // Quiet TTY output to avoid noisy test logs.
@@ -874,7 +1096,7 @@ pub fn syd() -> Syd {
     // Allow coredumps and non-PIE.
     cmd.m("trace/allow_unsafe_prlimit:1"); // Allow coredumps.
     cmd.m("trace/allow_unsafe_nopie:1"); // Allow non-PIE.
-    if *USE_STRACE || *USE_TRACE {
+    if *USE_TRACE || (*USE_STRACE && env::var_os("SYD_TEST_STRACE_NOFORK").is_none()) {
         cmd.m("trace/allow_unsafe_ptrace:1"); // Disable ptrace.
     }
     cmd
@@ -928,17 +1150,6 @@ pub fn check_ipv6() -> bool {
     }
 }
 
-/// Check if namespaces are supported.
-/// Returns None if syd process was terminated by a signal.
-pub fn check_unshare() -> Option<bool> {
-    syd()
-        .args(["-poff", "-pD", "-pcontainer", "true"])
-        .status()
-        .map(|stat| stat.code())
-        .ok()?
-        .map(|code| code == 0)
-}
-
 /// Checks if the C compiler allows nested routines.
 /// Returns `true` if supported, `false` otherwise.
 /// If successful, creates the executable `nested` in CWD.
@@ -973,6 +1184,8 @@ int main(int argc, char *argv[])
     // Compile the C code using the C compiler
     let compile_status = Command::new("cc")
         .arg("-Wall")
+        .arg("-Wextra")
+        .arg("-zexecstack") // required on ppc64
         .arg("nested.c")
         .arg("-o")
         .arg("nested")
@@ -1203,6 +1416,26 @@ pub fn check_32bin_64host() -> bool {
     }
 }
 
+/// Checks whether strace supports syscall injection.
+pub fn check_strace_inject() -> bool {
+    let status = Command::new("strace")
+        .arg("-qq")
+        .arg("-e")
+        .arg("trace=getpid")
+        .arg("-e")
+        .arg("inject=getpid:retval=2525")
+        .arg("--")
+        .arg("sh")
+        .arg("-c")
+        .arg("test $$ -eq 2525")
+        .status();
+
+    match status {
+        Ok(status) => status.success(),
+        Err(_) => false,
+    }
+}
+
 /// Enable coredumps.
 pub fn enable_coredumps() -> Result<(), Errno> {
     // Set both the soft and hard limits
@@ -1216,6 +1449,18 @@ pub fn check_timeout_foreground() -> bool {
         .arg("-sKILL")
         .arg("60s")
         .arg("true")
+        .stdout(Stdio::null())
+        .stderr(Stdio::null())
+        .status()
+        .map(|status| status.success())
+        .unwrap_or(false)
+}
+
+/// Check if the `ip` command is of `iproute2` (not `busybox`).
+pub fn check_iproute2() -> bool {
+    Command::new("sh")
+        .arg("-cex")
+        .arg("ip -V | grep -iq iproute2")
         .status()
         .map(|status| status.success())
         .unwrap_or(false)
@@ -1230,10 +1475,7 @@ pub fn current_dir(base: bool) -> std::io::Result<PathBuf> {
         // Extract the basename
         let basename = current_dir
             .file_name()
-            .ok_or(std::io::Error::new(
-                std::io::ErrorKind::Other,
-                "Failed to get the basename",
-            ))
+            .ok_or(std::io::Error::other("Failed to get the basename"))
             .map(PathBuf::from)?;
 
         Ok(basename)
