@@ -37,7 +37,7 @@ unsafe fn fail_errno(code: Err, errno: i32) -> ! {
         Err::ParentDeathSignal => c"syd: parent-death-signal error".as_ptr(),
         Err::PreExec => c"syd: pre-exec error".as_ptr(),
         Err::ProcessStop => c"syd: error stopping process".as_ptr(),
-        Err::ResetSignal => c"syd: error reseting signals".as_ptr(),
+        Err::ResetSignal => c"syd: error resetting signals".as_ptr(),
         Err::Seccomp => c"syd: seccomp error".as_ptr(),
         Err::SeccompFilterIoctl => c"syd: seccomp filter ioctl error".as_ptr(),
         Err::SeccompFilterAppendOnly => c"syd: seccomp filter pwritev2 error".as_ptr(),
@@ -164,6 +164,12 @@ pub extern "C" fn child_after_clone(arg: *mut libc::c_void) -> libc::c_int {
         }
     }
 
+    // Passthrough RUST_BACKTRACE to the sandbox process.
+    match env::var_os("SYD_RUST_BACKTRACE") {
+        Some(val) => env::set_var("RUST_BACKTRACE", val),
+        None => env::remove_var("RUST_BACKTRACE"),
+    }
+
     // SAFETY: Clean Syd environment variables from process environment.
     // Note, we have just used ENV_LOG_FD above and do not need it anymore.
     for (key, _) in env::vars_os() {
@@ -196,8 +202,8 @@ pub extern "C" fn child_after_clone(arg: *mut libc::c_void) -> libc::c_int {
         // Step 1: Reset the signal mask using pthread_sigmask.
         unsafe {
             let mut sigmask: libc::sigset_t = mem::zeroed();
-            libc::sigemptyset(&mut sigmask);
-            libc::pthread_sigmask(libc::SIG_SETMASK, &sigmask, ptr::null_mut());
+            libc::sigemptyset(&raw mut sigmask);
+            libc::pthread_sigmask(libc::SIG_SETMASK, &raw const sigmask, ptr::null_mut());
         }
         // Step 2: Unblock all signals using sigprocmask.
         let sigmask = SigSet::all();
