@@ -1,6 +1,6 @@
 //
 // Syd: rock-solid application kernel
-// src/syd-asm.rs: Disassemble raw instructions using objdump
+// src/utils/syd-asm.rs: Disassemble raw instructions using objdump
 //
 // Copyright (c) 2024, 2025 Ali Polatel <alip@chesswob.org>
 //
@@ -16,7 +16,7 @@ use std::{
 use data_encoding::HEXLOWER_PERMISSIVE;
 use libseccomp::ScmpArch;
 use nix::{errno::Errno, sys::utsname::uname};
-use syd::elf::disasm;
+use syd::asm::disasm;
 
 syd::main! {
     use lexopt::prelude::*;
@@ -36,7 +36,7 @@ syd::main! {
             Short('a') => {
                 let value = parser.value()?.parse::<String>()?;
                 if matches!(value.to_ascii_lowercase().as_str(), "help" | "list") {
-                    syd::print_seccomp_architectures();
+                    syd::confine::print_seccomp_architectures();
                     return Ok(ExitCode::SUCCESS);
                 }
                 opt_arch = match ScmpArch::from_str(&format!(
