@@ -1,6 +1,6 @@
 //
 // Syd: rock-solid application kernel
-// src/syd-fork.rs: Fork fast in an infinite loop.
+// src/utils/syd-fork.rs: Fork fast in an infinite loop.
 //
 // Copyright (c) 2024, 2025 Ali Polatel <alip@chesswob.org>
 //
@@ -8,6 +8,8 @@
 
 use std::process::ExitCode;
 
+use syd::confine::fork_fast;
+
 syd::main! {
     use lexopt::prelude::*;
 
@@ -28,7 +30,7 @@ syd::main! {
 
     // SAFETY: Do not try this at home!
     loop {
-        unsafe { syd::fork_fast() };
+        unsafe { fork_fast() };
     }
 }
 
