@@ -1,6 +1,6 @@
 //
 // Syd: rock-solid application kernel
-// src/syd-sys.rs: Calculate the memory usage of a given process or the parent process.
+// src/utils/syd-sys.rs: Calculate the memory usage of a given process or the parent process.
 //
 // Copyright (c) 2024, 2025 Ali Polatel <alip@chesswob.org>
 //
@@ -9,7 +9,7 @@
 use std::process::ExitCode;
 
 use nix::{errno::Errno, libc::pid_t, unistd::Pid};
-use procfs::process::MMapPath;
+use procfs_core::process::MMapPath;
 use syd::{
     human_size,
     proc::{proc_smaps, proc_statm},
