@@ -1,6 +1,6 @@
 //
 // Syd: rock-solid application kernel
-// src/syd-oci.rs: syd's OCI container runtime
+// src/utils/syd-oci.rs: syd's OCI container runtime
 //
 // Copyright (c) 2024, 2025 Ali Polatel <alip@chesswob.org>
 // Based in part upon youki which is:
@@ -1490,6 +1490,14 @@ syd::main! {
     // Note: syd's procfs protections is another layer of defense against this.
     if env::var_os(syd::config::ENV_QUICK_BOOT).is_none() {
         syd::seal::ensure_sealed()?;
+    } else {
+        // See seal.rs for the other branch.
+        // Rest is handled in unshare/child.rs
+        match env::var_os("RUST_BACKTRACE") {
+            Some(val) => env::set_var("SYD_RUST_BACKTRACE", val),
+            None => env::remove_var("SYD_RUST_BACKTRACE"),
+        };
+        env::set_var("RUST_BACKTRACE", "0");
     }
 
     // Dump is only used for syd(1) binary,
