@@ -31,13 +31,14 @@ use syd::{
     compat::epoll_ctl_safe,
     confine::{confine_mdwe, extend_ioctl},
     err::SydResult,
-    fs::{closeexcept, duprand, set_exclusive, set_nonblock, winsize_get, winsize_set},
+    fs::{closeexcept, set_exclusive, set_nonblock, winsize_get, winsize_set},
     ignore_signals,
     ioctl::{
         TCGETS, TCGETS2, TCSETS, TCSETS2, TCSETSF, TCSETSF2, TCSETSW, TCSETSW2, TIOCGWINSZ,
         TIOCSWINSZ,
     },
     landlock_policy::LandlockPolicy,
+    rng::duprand,
     IgnoreSignalOpts,
 };
 
