@@ -1,6 +1,6 @@
 //
 // Syd: rock-solid application kernel
-// src/syd-read.rs: Print the canonicalized path name followed by a newline and exit.
+// src/utils/syd-read.rs: Print the canonicalized path name followed by a newline and exit.
 //
 // Copyright (c) 2024, 2025 Ali Polatel <alip@chesswob.org>
 //
@@ -12,6 +12,7 @@ use std::{
     process::ExitCode,
 };
 
+use bitflags::Flags as BitFlags;
 use btoi::btoi;
 use libc::pid_t;
 use nix::{
@@ -23,7 +24,8 @@ use nix::{
 use syd::{
     fs::{safe_canonicalize, FsFlags},
     path::XPathBuf,
-    sandbox::Flags,
+    sandbox::{Flags, Sandbox},
+    syslog::LogLevel,
 };
 
 syd::main! {
@@ -32,6 +34,9 @@ syd::main! {
     // Set SIGPIPE handler to default.
     syd::set_sigpipe_dfl()?;
 
+    // Initialize logging.
+    syd::log::log_init_simple(LogLevel::Warn)?;
+
     // Parse CLI options.
     //
     // Note, option parsing is POSIXly correct:
@@ -111,6 +116,10 @@ syd::main! {
     // which is a requirement.
     syd::config::proc_init()?;
 
+    let mut sandbox = Sandbox::default();
+    sandbox.flags.clear();
+    sandbox.flags.insert(flags);
+
     for (idx, path) in paths
         .iter()
         .cycle()
@@ -122,7 +131,7 @@ syd::main! {
             opt_dir.as_ref().map(|fd| fd.as_raw_fd()),
             path,
             fsflags,
-            flags,
+            Some(&sandbox),
         ) {
             Ok(path) => path.take(),
             Err(errno) => {
