@@ -1,13 +1,11 @@
 //
 // Syd: rock-solid application kernel
-// src/syd-stat.rs: Print process status of the given PID or the current process.
+// src/utils/syd-stat.rs: Print process status of the given PID or the current process.
 //
 // Copyright (c) 2024, 2025 Ali Polatel <alip@chesswob.org>
 //
 // SPDX-License-Identifier: GPL-3.0
 
-#![recursion_limit = "256"]
-
 use std::process::ExitCode;
 
 use nix::{libc::pid_t, unistd::Pid};
@@ -50,7 +48,7 @@ syd::main! {
     });
 
     #[allow(clippy::disallowed_methods)]
-    let status = serde_json::to_string_pretty(&status).expect("JSON");
+    let status = serde_json::to_string(&status).expect("JSON");
     println!("{status}");
 
     Ok(ExitCode::SUCCESS)
