@@ -51,12 +51,13 @@ use syd::{
         SOCKET_COOKIE_ARG3, SOCKET_COOKIE_ARG4, SOCKET_COOKIE_ARG5,
     },
     err::{SydError, SydResult},
-    fs::{closeexcept, duprand, fillrandom, randint, retry_on_eintr},
+    fs::{closeexcept, retry_on_eintr},
     hash::SydHashMap,
     ignore_signals,
     landlock_policy::LandlockPolicy,
     parsers::sandbox::PortSet,
     path::XPathBuf,
+    rng::{duprand, fillrandom, randint},
     sealbox::{
         check_madvise_guard_support, madvise_guard_install, mprotect_none, mprotect_readonly, mseal,
     },
