@@ -1,6 +1,6 @@
 //
 // Syd: rock-solid application kernel
-// src/syd-x.rs: Check executability of the given files
+// src/utils/syd-x.rs: Check executability of the given files
 //
 // Copyright (c) 2025 Ali Polatel <alip@chesswob.org>
 //
@@ -16,11 +16,10 @@ use nix::{
 };
 use serde_json::json;
 use syd::{
-    config::HAVE_AT_EXECVE_CHECK, err::SydResult, fs::check_executable, path::XPathBuf,
-    proc::proc_executables,
+    config::HAVE_AT_EXECVE_CHECK, fs::check_executable, path::XPathBuf, proc::proc_executables,
 };
 
-fn main() -> SydResult<ExitCode> {
+syd::main! {
     use lexopt::prelude::*;
 
     syd::set_sigpipe_dfl()?;
