@@ -56,7 +56,8 @@ use crate::{
         SOCKET_COOKIE_ARG3, SOCKET_COOKIE_ARG4, SOCKET_COOKIE_ARG5, SYS_ACCEPT4, SYS_BIND,
         SYS_CONNECT, SYS_SOCKET, TRUNCATE64_COOKIE_ARG4, TRUNCATE64_COOKIE_ARG5,
         TRUNCATE_COOKIE_ARG2, TRUNCATE_COOKIE_ARG3, TRUNCATE_COOKIE_ARG4, TRUNCATE_COOKIE_ARG5,
-        UNLINKAT_COOKIE_ARG3, UNLINKAT_COOKIE_ARG4, UNLINKAT_COOKIE_ARG5,
+        UNAME_COOKIE_ARG1, UNAME_COOKIE_ARG2, UNAME_COOKIE_ARG3, UNAME_COOKIE_ARG4,
+        UNAME_COOKIE_ARG5, UNLINKAT_COOKIE_ARG3, UNLINKAT_COOKIE_ARG4, UNLINKAT_COOKIE_ARG5,
     },
     err::{err2no, SydJoinHandle, SydResult},
     fs::{
@@ -69,7 +70,7 @@ use crate::{
     proc::proc_mmap,
     sandbox::{Flags, Sandbox, SandboxGuard},
     timer::AlarmTimer,
-    workers::{aes::AesMap, UnixMap, WorkerCache, WorkerData},
+    workers::{aes::AesLock, UnixMap, WorkerCache, WorkerData},
 };
 
 #[derive(Clone)]
@@ -82,7 +83,7 @@ pub(crate) struct Worker {
     keep_alive: Option<u16>,
     should_exit: Arc<AtomicBool>,
     worker_data: Arc<WorkerData>,
-    crypt_map: Option<AesMap>,
+    crypt_map: Option<AesLock>,
     unix_map: UnixMap,
 }
 
@@ -96,7 +97,7 @@ impl Worker {
         keep_alive: Option<u16>,
         should_exit: Arc<AtomicBool>,
         worker_data: Arc<WorkerData>,
-        crypt_map: Option<AesMap>,
+        crypt_map: Option<AesLock>,
         unix_map: UnixMap,
     ) -> Self {
         let my_sandbox = SandboxGuard::Read(sandbox.read().unwrap_or_else(|err| err.into_inner()));
@@ -360,8 +361,14 @@ impl Worker {
         if ghost {
             let _ = close(self.fd);
 
-            // Inform the monitor thread and other emulator threads to exit.
+            // Inform the monitor thread and other emulator threads to exit,
+            // and notify the syd_aes thread as necessary.
             self.should_exit.store(true, Ordering::Relaxed);
+            if let Some(ref crypt_map) = self.crypt_map {
+                let (aes_map, cvar) = &**crypt_map;
+                let _aes_map = aes_map.lock().unwrap_or_else(|e| e.into_inner());
+                cvar.notify_one();
+            } // Lock is released here.
         }
     }
 
@@ -756,6 +763,29 @@ impl Worker {
                         "msg": format!("invalid or unsupported syscall {sysname}"));
                 }
             }
+
+            // uname(2) may be used only with syscall argument cookies.
+            let sysname = "uname";
+            #[allow(clippy::useless_conversion)]
+            match ScmpSyscall::from_name(sysname) {
+                Ok(syscall) => {
+                    ctx.add_rule_conditional(
+                        ScmpAction::Allow,
+                        syscall,
+                        &[
+                            scmp_cmp!($arg1 == (*UNAME_COOKIE_ARG1).into()),
+                            scmp_cmp!($arg2 == (*UNAME_COOKIE_ARG2).into()),
+                            scmp_cmp!($arg3 == (*UNAME_COOKIE_ARG3).into()),
+                            scmp_cmp!($arg4 == (*UNAME_COOKIE_ARG4).into()),
+                            scmp_cmp!($arg5 == (*UNAME_COOKIE_ARG5).into()),
+                        ],
+                    )?;
+                }
+                Err(_) => {
+                    info!("ctx": "confine", "op": "allow_emu_syscall",
+                        "msg": format!("invalid or unsupported syscall {sysname}"));
+                }
+            }
         } else {
             // trace/allow_unsafe_nocookie: Allow access without cookies.
             for sysname in [
@@ -768,6 +798,7 @@ impl Worker {
                 "linkat",
                 "unlinkat",
                 "openat2",
+                "uname",
             ] {
                 match ScmpSyscall::from_name(sysname) {
                     Ok(syscall) => {
