@@ -45,11 +45,12 @@ use crate::{
     config::*,
     confine::{confine_scmp_write, scmp_add_setid_rules},
     err::{err2no, scmp2no, SydJoinHandle, SydResult},
-    fs::{closeexcept, duprand, peer_cred, retry_on_eintr},
+    fs::{closeexcept, peer_cred, retry_on_eintr},
     hash::SydHashMap,
     info,
     landlock_policy::LandlockPolicy,
     path::{XPath, XPathBuf},
+    rng::duprand,
     sandbox::{Flags, Sandbox, LINE_MAX},
 };
 
